package com.bocloud.ams.booter.controller.association;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.association.AutoRelationRuleItem;
import com.bocloud.ams.service.association.AutoRelationRuleService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/rule")
@Tag(name = "关联规则管理")
public class AutoRelationRuleController {

    @Autowired
    private AutoRelationRuleService autoRelationRuleService;

    /**
     * 关联规则展示
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "关联规则列表")
    @GetMapping
    public GeneralResult<GridBean<AutoRelationRule>> list(Pager pager, @RequestParam(value = "positionKey", required = false) String positionKey) {
        return autoRelationRuleService.list(pager, positionKey);
    }

    /**
     * 新增关联规则
     *
     * @param rule
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "关联规则新增")
    @PostMapping
    public GeneralResult<Void> save(@RequestBody AutoRelationRule rule,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return autoRelationRuleService.save(rule, context.getTarget());
    }

    /**
     * 更新关联规则
     *
     * @param id
     * @param rule
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "关联规则修改")
    @PutMapping(value = "/{id}")
    public GeneralResult<Void> modify(@PathVariable(value = Common.ID) Long id, @RequestBody AutoRelationRule rule,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return autoRelationRuleService.modify(id, rule, context.getTarget());
    }

    /**
     * 关联规则详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "关联规则详情")
    @GetMapping(value = "/{id}")
    public GeneralResult<AutoRelationRule> detail(@PathVariable(value = Common.ID) Long id,
                                                  @RequestParam(value = "positionKey", required = false) String positionKey) {
        return autoRelationRuleService.detail(id, positionKey);
    }

    /**
     * 关联规则明细详情
     *
     * @param itemId
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "关联规则明细详情")
    @GetMapping(value = "/{id}/items/{itemId}")
    public GeneralResult<AutoRelationRuleItem> itemDetail(@PathVariable("itemId") Long itemId,
                                                          @RequestParam(value = "positionKey", required = false) String positionKey) {
        return autoRelationRuleService.itemDetail(itemId, positionKey);
    }

    /**
     * 删除关联规则
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "关联规则删除")
    @DeleteMapping(value = "/{id}")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return autoRelationRuleService.remove(id, context.getTarget());
    }
}
