package com.bocloud.ams.booter.controller.collection;

import com.bocloud.ams.entity.collection.CollectionHistory;
import com.bocloud.ams.entity.model.CollectionHistoryModel;
import com.bocloud.ams.service.collection.CollectionHistoryService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

@RestController
@RequestMapping("/v1/collection/history")
@Tag(name = "采集历史管理")
public class CollectionHistoryController {

    @Autowired
    private CollectionHistoryService collectionHistoryService;

    @Operation(tags = {"CMC"}, summary = "实例采集历史列表")
    @GetMapping("/instances")
    public GeneralResult<GridBean<Map<String, Object>>> list(Pager pager,
                                                             @Value(Common.REQ_CONTEXT) RequestContext context,
                                                             @RequestParam(value = "positionKey", required = false) String positionKey) {
        return collectionHistoryService.instances(pager, context.getTarget(), positionKey);
    }

    @Operation(tags = {"CMC"}, summary = "采集历史列表")
    @GetMapping
    public GeneralResult<GridBean<CollectionHistory>> history(Pager pager,
                                                              @RequestParam(value = "positionKey", required = false) String positionKey) {
        return collectionHistoryService.list(pager, positionKey);
    }

    @Operation(tags = {"CMC"}, summary = "采集历史新增")
    @PostMapping
    public GeneralResult<Void> save(@RequestBody CollectionHistory history,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionHistoryService.create(history);
    }

    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC"}, summary = "采集历史修改")
    public GeneralResult<Void> modify(@RequestBody CollectionHistory history,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionHistoryService.modify(history);
    }

    @PutMapping(value = "/batch")
    @Operation(tags = {"CMC"}, summary = "采集历史批量新增")
    public GeneralResult<String> batchModify(@RequestBody CollectionHistoryModel historyModel,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return collectionHistoryService.batchModify(historyModel, context.getTarget());
    }

    @Operation(tags = {"CMC"}, summary = "采集历史备份列表")
    @GetMapping("/backup")
    public GeneralResult<GridBean<CollectionHistory>> backupHistory(Pager pager,
                                                                    @RequestParam(value = "positionKey", required = false) String positionKey) {
        return collectionHistoryService.backupHistory(pager, positionKey);
    }

    @Operation(tags = {"CMC"}, summary = "实例采集历史备份列表")
    @GetMapping("/backup/instances")
    public GeneralResult<GridBean<Map<String, Object>>> backupList(Pager pager,
                                                                    @Value(Common.REQ_CONTEXT) RequestContext context,
                                                                   @RequestParam(value = "positionKey", required = false) String positionKey) {
        return collectionHistoryService.backupList(pager, context.getTarget(), positionKey);
    }

}
