
package com.bocloud.ams.booter.controller.instance;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.service.instance.InstanceAutoRelationService;
import com.bocloud.ams.service.instance.InstanceRelationService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping(value = "/v1/instances/relations")
@Tag(name = "实例关系管理")
public class InstanceRelationController {

    @Autowired
    private InstanceRelationService instanceRelationService;

    @Autowired
    private InstanceAutoRelationService instanceAutoRelationService;

    /***
     * 新增实例关系
     *
     * @param relation
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例关系关联")
    @PostMapping
    public GeneralResult<Void> save(@RequestBody InstanceRelation relation,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceRelationService.create(relation, context);
    }

    /***
     * 删除实例关系
     *
     * @param id
     * @param relationCategoryCode
     * @param currentInstanceId
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例关系取消关联")
    @DeleteMapping(value = "/{id}")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
        @RequestParam(value = "relationCategoryCode") String relationCategoryCode,
        @RequestParam(value = "currentModuleId") Long currentModuleId,
        @RequestParam(value = "currentInstanceId") Long currentInstanceId,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceRelationService.remove(id, relationCategoryCode, currentModuleId, currentInstanceId, context);
    }

    @Operation(tags = {"CMC"}, summary = "实例关系实例数据自动关联")
    @PostMapping(value = "/auto/relation")
    public GeneralResult<Void> instanceAutoRelation(@RequestBody Instance instance,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceAutoRelationService.instanceAutoRelation(instance, context.getTarget());
    }

    @Operation(tags = {"CMC"}, summary = "实例关系全量数据自动关联")
    @GetMapping(value = "/auto/relation/all")
    public GeneralResult<Void> moduleAutoRelation() {
        return instanceAutoRelationService.autoRelation();
    }

    @Operation(tags = {"CMC"}, summary = "实例关系模型数据自动关联")
    @GetMapping(value = "/auto/relation/{id}/module")
    public GeneralResult<Void> moduleAutoRelationByModuleId(@PathVariable(value = Common.ID) Long id) {
        return instanceAutoRelationService.autoRelationByModuleId(id);
    }
}
