package com.bocloud.ams.booter.scheduler;

import com.bocloud.ams.entity.TableLog;
import com.bocloud.ams.entity.collection.CollectionHistory;
import com.bocloud.ams.entity.collection.CollectionHistoryBak;
import com.bocloud.ams.entity.collection.CollectionHistoryItem;
import com.bocloud.ams.entity.collection.CollectionHistoryItemBak;
import com.bocloud.ams.entity.collection.CollectionTaskInstance;
import com.bocloud.ams.entity.collection.CollectionTaskInstanceBak;
import com.bocloud.ams.entity.config.CmdbConfig;
import com.bocloud.ams.entity.enums.Status;
import com.bocloud.ams.repository.TableLogRepository;
import com.bocloud.ams.repository.collection.CollectionHistoryBakRepository;
import com.bocloud.ams.repository.collection.CollectionHistoryItemBakRepository;
import com.bocloud.ams.repository.collection.CollectionHistoryItemRepository;
import com.bocloud.ams.repository.collection.CollectionHistoryRepository;
import com.bocloud.ams.repository.collection.CollectionTaskInstanceBakRepository;
import com.bocloud.ams.repository.collection.CollectionTaskInstanceRepository;
import com.bocloud.ams.repository.config.CmdbConfigRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author huliwei
 * @date 2023/4/24
 */

@Slf4j
@Component
public class CollectionTaskInstanceBakScheduler {

    private static final String CMDB_COLLECTION_TASK_INSTANCE_BAK_ = "ci_collection_task_instance_bak_";
    private static final String CMDB_COLLECTION_HISTORY_BAK_ = "ci_collection_history_bak_";
    private static final String CMDB_COLLECTION_HISTORY_ITEM_BAK_ = "ci_collection_history_item_bak_";

    private static final String CMDB_COLLECTION_TASK_INSTANCE_BAK = "ci_collection_task_instance_bak";
    private static final String CMDB_COLLECTION_HISTORY_BAK = "ci_collection_history_bak";
    private static final String CMDB_COLLECTION_HISTORY_ITEM_BAK = "ci_collection_history_item_bak";

    @Autowired
    private CurrentService service;
    @Autowired
    private CollectionTaskInstanceRepository collectionTaskInstanceRepository;
    @Autowired
    private CollectionHistoryRepository collectionHistoryRepository;
    @Autowired
    private CollectionHistoryItemRepository collectionHistoryItemRepository;
    @Autowired
    private CmdbConfigRepository cmdbConfigRepository;
    @Autowired
    private TableLogRepository tableLogRepository;
    @Autowired
    private CollectionTaskInstanceBakRepository collectionTaskInstanceBakRepository;
    @Autowired
    private CollectionHistoryBakRepository collectionHistoryBakRepository;
    @Autowired
    private CollectionHistoryItemBakRepository collectionHistoryItemBakRepository;


    //每月第一天的0时1分开始创建当月的备份表
    @Scheduled(cron = "${collection.task.instance.create-table:0 1 0 1 * ? }")
    private void createTable() {
        try {
            //ci_collection_history_bak_202304
            String tableNameTaskInstanceBak = CMDB_COLLECTION_TASK_INSTANCE_BAK_ + this.getTableSuffix(LocalDateTime.now());
            String tableNameHistoryBak = CMDB_COLLECTION_HISTORY_BAK_ + this.getTableSuffix(LocalDateTime.now());
            String tableNameHistoryItemBak = CMDB_COLLECTION_HISTORY_ITEM_BAK_ + this.getTableSuffix(LocalDateTime.now());
            this.createCollectionTaskInstanceBakTable(tableNameTaskInstanceBak);
            this.createCollectionHistoryBakTable(tableNameHistoryBak);
            this.createCollectionHistoryItemBakTable(tableNameHistoryItemBak);
        } catch (Exception e) {
            log.error("定时创建库表失败", e);
            return;
        }
    }

    @Scheduled(cron = "${collection.task.instance.bakup:0 0 4 * * ?}")
    private void instanceDataBak() {
        if (!service.isLeader()) {
            return;
        }
        log.info("开始备份任务数据");
        List<CollectionTaskInstance> instances = null;
        int stepSize = 200;
        try {
            CmdbConfig switchConfig = cmdbConfigRepository.queryByCode("CmdbCollectionTaskInstanceDataBakSwitch");
            if (null == switchConfig || StringUtils.isEmpty(switchConfig.getValue()) || !switchConfig.getValue().equalsIgnoreCase("true")) {
                log.info("备份任务未开启");
                return;
            }
            Integer days = null;
            CmdbConfig config = cmdbConfigRepository.queryByCode("CmdbCollectionTaskInstanceDataBakDaysAgo");
            if (null != config && StringUtils.isNotEmpty(config.getValue())) {
                days = Integer.valueOf(config.getValue());
            }
            days = null == days ? 90 : days;
            LocalDate localDate = LocalDate.now().minusDays(days);
            LocalDateTime localDateTime = LocalDateTime.of(localDate, LocalTime.MIN);
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            String formatTime = dateTimeFormatter.format(localDateTime);
            log.info("备份{}之前的数据", formatTime);
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("gmtCreate", formatTime), Sign.LET));
            instances = this.collectionTaskInstanceRepository.list(params);
        } catch (Exception e) {
            log.error("查询任务实例失败", e);
            return;
        }
        if (ListTool.isEmpty(instances)) {
            log.info("结束备份任务数据，没有查询到需要备份的数据");
            return;
        }
        int size = instances.size();
        log.info("备份任务数{}，每批任务数{}", size, stepSize);
        if (size > stepSize) {
            int num = size % stepSize == 0 ? size / stepSize : size / stepSize + 1;
            for (int i = 0; i < num; i++) {
                log.info("备份第{}批数据", i + 1);
                if (i + 1 == num) {
                    batchBak(instances.subList(i * stepSize, size));
                } else {
                    batchBak(instances.subList(i * stepSize, (i + 1) * stepSize));
                }
            }

        } else {
            batchBak(instances);
        }
        log.info("结束备份任务数据");
    }

    private void batchBak(List<CollectionTaskInstance> instances) {
        try {
            //根据年和月分组
            Map<String, List<CollectionTaskInstance>> map = new HashMap<>();
            for (CollectionTaskInstance instance : instances) {
                LocalDateTime localDateTime = LocalDateTime.ofInstant(instance.getGmtCreate().toInstant(), ZoneId.systemDefault());
                String tableSuffix = this.getTableSuffix(localDateTime);
                if (map.containsKey(tableSuffix)) {
                    List<CollectionTaskInstance> taskInstanceList = map.get(tableSuffix);
                    taskInstanceList.add(instance);
                } else {
                    List<CollectionTaskInstance> taskInstances = new ArrayList<>();
                    taskInstances.add(instance);
                    map.put(tableSuffix, taskInstances);
                }
            }
            for (String tableSuffix : map.keySet()) {
                //备份ci_collection_task_instance
                List<CollectionTaskInstance> taskInstances = map.get(tableSuffix);
                List<Long> instanceIds = taskInstances.stream().map(CollectionTaskInstance::getId).collect(Collectors.toList());
                List<CollectionTaskInstanceBak> instanceBaks = Lists.newArrayList();
                for (CollectionTaskInstance instance : instances) {
                    CollectionTaskInstanceBak bak = new CollectionTaskInstanceBak();
                    BeanUtils.copyProperties(instance, bak);
                    instanceBaks.add(bak);
                }
                if (!instanceBaks.isEmpty()) {
                    String tableName = CMDB_COLLECTION_TASK_INSTANCE_BAK_ + tableSuffix;
                    GeneralResult<Void> taskInstanceBakTable = this.createCollectionTaskInstanceBakTable(tableName);
                    if (taskInstanceBakTable.isSuccess()) {
                        this.batchSaveCollectionTaskInstanceBak(instanceBaks, tableName);
                        collectionTaskInstanceRepository.delete(instanceIds);
                    }
                }
                //备份ci_collection_history
                List<CollectionHistory> collectionHistories = collectionHistoryRepository.listByInstances(instanceIds);
                List<Long> historyIds = collectionHistories.stream().map(CollectionHistory::getId).collect(Collectors.toList());
                List<CollectionHistoryBak> collectionHistoryBaks = Lists.newArrayList();
                for (CollectionHistory collectionHistory : collectionHistories) {
                    CollectionHistoryBak collectionHistoryBak = new CollectionHistoryBak();
                    BeanUtils.copyProperties(collectionHistory, collectionHistoryBak);
                    collectionHistoryBaks.add(collectionHistoryBak);
                }
                if (!collectionHistoryBaks.isEmpty()) {
                    String tableName = CMDB_COLLECTION_HISTORY_BAK_ + tableSuffix;
                    GeneralResult<Void> collectionHistoryBakTable = this.createCollectionHistoryBakTable(tableName);
                    if (collectionHistoryBakTable.isSuccess()) {
                        this.batchSaveCollectionHistoryBak(collectionHistoryBaks, tableName);
                        collectionHistoryRepository.deleteByInstanceIds(instanceIds);
                    }
                }
                //备份ci_collection_history_item
                if (!ListTool.isEmpty(historyIds)) {
                    List<CollectionHistoryItem> collectionHistoryItems = collectionHistoryItemRepository.listByHistoryIds(historyIds);
                    List<CollectionHistoryItemBak> collectionHistoryItemBaks = Lists.newArrayList();
                    for (CollectionHistoryItem relation : collectionHistoryItems) {
                        CollectionHistoryItemBak collectionHistoryItemBak = new CollectionHistoryItemBak();
                        BeanUtils.copyProperties(relation, collectionHistoryItemBak);
                        collectionHistoryItemBaks.add(collectionHistoryItemBak);
                    }
                    if (!collectionHistoryItemBaks.isEmpty()) {
                        String tableName = CMDB_COLLECTION_HISTORY_ITEM_BAK_ + tableSuffix;
                        GeneralResult<Void> collectionHistoryItemBakTable = this.createCollectionHistoryItemBakTable(tableName);
                        if (collectionHistoryItemBakTable.isSuccess()) {
                            this.batchSaveCollectionHistoryItemBak(collectionHistoryItemBaks, tableName);
                            collectionHistoryItemRepository.deleteByHistoryIds(historyIds);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("任务备份异常", e);
        }
    }

    /**
     * 批量保存
     *
     * @param collectionTaskInstanceBaks 需要保存的数据
     * @param tableName                  备份表名称
     */
    private void batchSaveCollectionTaskInstanceBak(List<CollectionTaskInstanceBak> collectionTaskInstanceBaks, String tableName) {
        synchronized (CollectionTaskInstanceBak.class) {
            CmdbUtils.changeTableAnnotationValue(CollectionTaskInstanceBak.class, tableName);
            collectionTaskInstanceBakRepository.batchSave(collectionTaskInstanceBaks, collectionTaskInstanceBaks.size(), false);
            //还原操作
            CmdbUtils.changeTableAnnotationValue(CollectionTaskInstanceBak.class, CMDB_COLLECTION_TASK_INSTANCE_BAK);
        }
    }

    /**
     * 批量保存
     *
     * @param collectionHistoryBaks 需要保存的数据
     * @param tableName             备份表名称
     */
    private void batchSaveCollectionHistoryBak(List<CollectionHistoryBak> collectionHistoryBaks, String tableName) {
        synchronized (CollectionHistoryBak.class) {
            CmdbUtils.changeTableAnnotationValue(CollectionHistoryBak.class, tableName);
            collectionHistoryBakRepository.batchSave(collectionHistoryBaks, collectionHistoryBaks.size(), false);
            //还原操作
            CmdbUtils.changeTableAnnotationValue(CollectionHistoryBak.class, CMDB_COLLECTION_HISTORY_BAK);
        }
    }

    /**
     * 批量保存
     *
     * @param collectionHistoryItemBaks 需要保存的数据
     * @param tableName                 备份表名称
     */
    private void batchSaveCollectionHistoryItemBak(List<CollectionHistoryItemBak> collectionHistoryItemBaks, String tableName) {
        synchronized (CollectionHistoryItemBak.class) {
            CmdbUtils.changeTableAnnotationValue(CollectionHistoryItemBak.class, tableName);
            collectionHistoryItemBakRepository.batchSave(collectionHistoryItemBaks, collectionHistoryItemBaks.size(), false);
            //还原操作
            CmdbUtils.changeTableAnnotationValue(CollectionHistoryItemBak.class, CMDB_COLLECTION_HISTORY_ITEM_BAK);
        }

    }

    /**
     * 动态创建表
     *
     * @param tableName 需要创建的表名称
     * @return
     */
    private GeneralResult<Void> createCollectionTaskInstanceBakTable(String tableName) {
        synchronized (CollectionTaskInstanceBak.class) {
            String templateTable = CMDB_COLLECTION_TASK_INSTANCE_BAK;
            try {
                if (collectionTaskInstanceBakRepository.countTable(tableName) <= 0) {
                    collectionTaskInstanceBakRepository.createTable(tableName, new CollectionTaskInstanceBak());
                    this.saveTableLog(tableName, templateTable, Status.Inst.SUCCESS.name());
                }
                return new GeneralResult<>(true, "创建成功");
            } catch (Exception e) {
                this.saveTableLog(tableName, templateTable, Status.Inst.FAILED.name());
                log.error("create ci_collection_task_instance_bak table error:", e);
                return new GeneralResult<>(false, "创建失败");
            }
        }
    }

    /**
     * 动态创建表
     *
     * @param tableName 需要创建的表名称
     * @return
     */
    private GeneralResult<Void> createCollectionHistoryBakTable(String tableName) {
        synchronized (CollectionHistoryBak.class) {
            String templateTable = CMDB_COLLECTION_HISTORY_BAK;
            try {
                if (collectionHistoryBakRepository.countTable(tableName) <= 0) {
                    collectionHistoryBakRepository.createTable(tableName, new CollectionHistoryBak());
                    this.saveTableLog(tableName, templateTable, Status.Inst.SUCCESS.name());
                }
                return new GeneralResult<>(true, "创建成功");
            } catch (Exception e) {
                this.saveTableLog(tableName, templateTable, Status.Inst.FAILED.name());
                log.error("create ci_collection_history_bak table error:", e);
                return new GeneralResult<>(false, "创建失败");
            }
        }
    }

    /**
     * 动态创建表
     *
     * @param tableName 需要创建的表名称
     * @return
     */
    private GeneralResult<Void> createCollectionHistoryItemBakTable(String tableName) {
        synchronized (CollectionHistoryItemBak.class) {
            String templateTable = CMDB_COLLECTION_HISTORY_ITEM_BAK;
            try {
                if (collectionHistoryItemBakRepository.countTable(tableName) <= 0) {
                    collectionHistoryItemBakRepository.createTable(tableName, new CollectionHistoryItemBak());
                    this.saveTableLog(tableName, templateTable, Status.Inst.SUCCESS.name());
                }
                return new GeneralResult<>(true, "创建成功");
            } catch (Exception e) {
                this.saveTableLog(tableName, templateTable, Status.Inst.FAILED.name());
                log.error("create ci_collection_history_item_bak table error:", e);
                return new GeneralResult<>(false, "创建失败");
            }
        }
    }

    private void saveTableLog(String tableName, String templateTableName, String status) {
        TableLog tableLog = new TableLog();
        tableLog.setName(tableName);
        tableLog.setCode("BACKUP");
        tableLog.setStatus(status);
        tableLog.setTemplateTable(templateTableName);
        tableLog.setGmtCreate(new Date());
        tableLog.setDeleted(false);
        tableLogRepository.save(tableLog);
    }

    /**
     * 获取动态表的后缀
     *
     * @param localDate
     * @return
     */
    private String getTableSuffix(LocalDateTime localDate) {
        //2023
        int year = localDate.getYear();
        //4
        int month = localDate.getMonth().getValue();
        String months = String.valueOf(month).length() > 1 ? String.valueOf(month) : "0" + month;
        //202304
        String suffix = year + months;
        return suffix;
    }
}
