package com.bocloud.ams.entity.module;

import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.List;

/**
 * @Description: cmdb属性池
 * @Author majiali
 * @CreateTime 2022/5/5 14:02
 */
@Data
@Table("ci_property_pool")
@EqualsAndHashCode(callSuper = true)
public class PropertyPool extends GenericEntity {
    @PK(value = PKStrategy.AUTO)
    private Long id;

    @Column("code")
    private String code;

    /**
     * 0:用户定义 1:系统内置
     */
    @Column("is_reserved")
    private Boolean isReserved = false;

    /**
     * TEXTFIELD:单行文本
     * TEXTAREA:多行文本
     * SINGLESELECT:下拉单选
     * MULTISELECT:下拉多选
     * QUOTE:引用
     * DATE:日期
     * TIME:时间
     * TEXTENCRYPT:数据加密
     * INTEGER:整数
     * FLOAT:小数
     * SINGLELINETABLE:单行表格
     * MULTILINETABLE:多行表格
     * JSON:json
     * BOOLEAN:布尔值
     */
    @Column("type")
    private String type;

    @Column("json_schema")
    private String jsonSchema;

    /**
     * 最小值
     */
    @Column("min_value")
    private Integer minValue;

    /**
     * 最大值
     */
    @Column("max_value")
    private Integer maxValue;

    /**
     * 默认值类型  固定值：FIXED 内置函数：INNERFUN 自增ID：AUTOINCRID 流水号：SERIALNUM
     */
    @Column("default_value_type")
    private String defaultValueType;

    /**
     * 前缀标识
     */
    @Column("prefix_identifier")
    private String prefixIdentifier;

    /**
     *内置函数：INNERFUN  只支持 全局唯一标识符对应GLOBALLYUNIQUEIDENTIFIER
     * 流水号：SERIALNUM  默认长度为1，存到默认值中
     *
     */
    @Column("default_value")
    private String defaultValue;

    /**
     * 起始值
     */
    @Column("start_num")
    private Integer startNum;

    /**
     * 最大长度
     */
    @Column("length")
    private Integer length;

    /**
     * 小数位数
     */
    @Column("decimal_num")
    private Integer decimalNum;

    /**
     * 0:可为空，1:必填
     */
    @Column("is_required")
    private Boolean isRequired = false;

    /**
     * 是否唯一 0:不唯一，1:唯一
     */
    @Column("is_uniqued")
    private Boolean isUniqued = false;

    /**
     * 是否只读 0:否，1:是
     */
    @Column("is_only_reader")
    private Boolean isOnlyReader  = false;

    /**
     * 是否搜索项 0:否，1:是
     */
    @Column("is_search")
    private Boolean isSearch = false;

    /**
     * 单位
     */
    @Column("unit")
    private String unit;

    /**
     * 校验规则ID
     */
    @Column("regexp_id")
    private Long regexpId;

    /**
     * 表格类型属性结构体定义类型 0：新建定义  1：从已有模型中引入
     */
    @Column("table_type")
    private String tableType;

    /**
     * 表格类型属性引入已有模型的模型ID
     */
    @Column("link_module_id")
    private Long linkModuleId;

    /**
     * 表格类型属性引入已有模型的属性id，以逗号隔开
     */
    @Column("property_ids")
    private String propertyIds;

    /**
     * 列表项
     */
    @Column("is_title")
    private Boolean isTitle = false;

    /**
     * @Description: 校验规则表达式
     */
    @IgnoreAll
    private String expression;

    @IgnoreAll
    private List<PropertyPoolItem> itemList;

    @IgnoreAll
    private List<Property> propertyList;


}
