package com.bocloud.ams.repository.collection;

import com.bocloud.ams.entity.collection.CollectionHistoryItem;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;


/**
 * @Author yangzhenrong
 * @Date 2022/6/15 20:37
 * @Version 1.0
 **/
@Repository
public class CollectionHistoryItemRepository extends BasicGenericDao<CollectionHistoryItem, Long> {

    public CollectionHistoryItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<CollectionHistoryItem> list(Pager pager) throws Exception {
        String sql = "select a.* from (select a.*,cp.name as property_name from ci_collection_history_item a left join ci_property cp on a.property_id = cp.id) a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CollectionHistoryItem.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<CollectionHistoryItem> list(Pager pager, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.* from (select a.*,cp.name as property_name from " + positionKey + "." +
                    "ci_collection_history_item a left join " + positionKey + "." +
                    "ci_property cp on a.property_id = cp.id) a where 1=1 ";
        } else {
            sql = "select a.* from (select a.*,cp.name as property_name from ci_collection_history_item a left join ci_property cp on a.property_id = cp.id) a where 1=1 ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CollectionHistoryItem.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from (select a.*,cp.name as property_name from ci_collection_history_item a left join ci_property cp on a.property_id = cp.id) a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (!StringUtils.isEmpty(positionKey)) {
             sql = "select count(1) from (select a.*,cp.name as property_name from " + positionKey + "." +
                     "ci_collection_history_item a left join " + positionKey + "." +
                     "ci_property cp on a.property_id = cp.id) a where 1=1 ";
        } else {
             sql = "select count(1) from (select a.*,cp.name as property_name from ci_collection_history_item a left join ci_property cp on a.property_id = cp.id) a where 1=1 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public List<CollectionHistoryItem> listByHistoryId(Long historyId) throws Exception {
        String sql = "select a.* from ci_collection_history_item a where a.history_id = :historyId";
        return this.list(CollectionHistoryItem.class, sql, MapTools.simpleMap("historyId", historyId));
    }

    public List<CollectionHistoryItem> listByHistoryIds(List<Long> historyIds) throws Exception {
        String sql = "select a.* from ci_collection_history_item a where a.history_id in (:historyIds) ";
        return this.list(CollectionHistoryItem.class, sql, MapTools.simpleMap("historyIds", historyIds));
    }

    public boolean deleteByHistoryIds(List<Long> historyIds) {
        String sql = "delete from ci_collection_history where history_id in (:historyIds)";
        Map<String, Object> params = MapTools.simpleMap("historyIds", historyIds);
        return this.execute(sql, params) > 0;
    }

    public List<CollectionHistoryItem> list(String tableName, Pager pager) throws Exception {
        String sql = "select a.* from (select a.*,cp.name as property_name from " + tableName + " a left join ci_property cp on a.property_id = cp.id) a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CollectionHistoryItem.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<CollectionHistoryItem> list(String tableName, Pager pager, String positionKey) throws Exception {
        if (StringUtils.isNotBlank(positionKey)) {
            tableName = positionKey + "." + tableName;
        }
        String sql = "select a.* from (select a.*,cp.name as property_name from " + tableName + " a left join ci_property cp on a.property_id = cp.id) a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CollectionHistoryItem.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(String tableName, List<Param> params) throws Exception {
        String sql = "select count(1) from (select a.*,cp.name as property_name from " + tableName + " a left join ci_property cp on a.property_id = cp.id) a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(String tableName, List<Param> params, String positionKey) throws Exception {
        if (StringUtils.isNotEmpty(positionKey)) {
            tableName = positionKey + "." + tableName;
        }
        String sql = "select count(1) from (select a.*,cp.name as property_name from " + tableName + " a left join ci_property cp on a.property_id = cp.id) a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

}
