package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.PropertyPoolItem;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description: cmdb属性池配置项
 * @Author: majiali
 * @CreateTime: 2022/5/5 14:04
 */
@Repository
public class PropertyPoolItemRepository extends BasicGenericDao<PropertyPoolItem, Long> {
    public PropertyPoolItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_property_pool_item set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean deleteByIds(List<Long> ids, Long userId) throws Exception {
        String sql = "update ci_property_pool_item set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id in (:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean deleteByPropertyPoolId(Long propertyPoolId, Long userId) throws Exception {
        String sql = "update ci_property_pool_item set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where property_pool_id = :propertyPoolId";
        Map<String, Object> params = MapTools.simpleMap("propertyPoolId", propertyPoolId);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<PropertyPoolItem> listByPropertyPoolId(Long propertyPoolId) throws Exception {
        String sql = "select a.* from ci_property_pool_item a where a.is_deleted=0 and a.property_pool_id = :propertyPoolId";
        return this.list(PropertyPoolItem.class, sql, MapTools.simpleMap("propertyPoolId", propertyPoolId));
    }

    public List<PropertyPoolItem> listByPropertyPoolId(Long propertyPoolId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from "+positionKey+".ci_property_pool_item a where a.is_deleted=0 and a.property_pool_id = :propertyPoolId";
        } else {
            sql = "select a.* from ci_property_pool_item a where a.is_deleted=0 and a.property_pool_id = :propertyPoolId";
        }
        return this.list(PropertyPoolItem.class, sql, MapTools.simpleMap("propertyPoolId", propertyPoolId));
    }

    public List<PropertyPoolItem> listByPropertyPoolIds(List<Long> propertyPoolIds) throws Exception {
        String sql = "select a.* from ci_property_pool_item a where a.is_deleted=0 and a.property_pool_id in (:propertyPoolIds)";
        return this.list(PropertyPoolItem.class, sql, MapTools.simpleMap("propertyPoolIds", propertyPoolIds));
    }

}
