package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.Property;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author 胡立伟
 * @program: bocloud.ams
 * @description:
 * @date 2022/5/5 12:01
 */
@Repository
public class PropertyRepository extends BasicGenericDao<Property, Long> {

    public PropertyRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean deleteByModuleId(Long moduleId, Long userId) throws Exception {
        String sql = "update ci_property set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where module_id = :moduleId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<Property> listNotHiddenByCmdbModuleId(Long moduleId) {
        // 模型界面显示未隐藏字段
        String sql = "select a.* from ci_property a where a.is_deleted = 0  and a.module_id = :moduleId order by a.order_num";
        return this.list(Property.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public List<Property> listNotHiddenByCmdbModuleId(Long moduleId, String positionKey) {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_property a where a.is_deleted = 0  and a.module_id = :moduleId order by a.order_num";
        } else {
            sql = "select a.* from ci_property a where a.is_deleted = 0  and a.module_id = :moduleId order by a.order_num";
        }
        // 模型界面显示未隐藏字段
        return this.list(Property.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public Property query(Long id) {
        String sql = "select * from ci_property a where a.id = :id and is_deleted=0 ";
        List<Property> list = this.list(Property.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public Property query(Long id, String positionKey) {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select * from "+positionKey+".ci_property a where a.id = :id and is_deleted=0 ";
        } else {
            sql = "select * from ci_property a where a.id = :id and is_deleted=0 ";

        }
        List<Property> list = this.list(Property.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public Property queryByCode(Long moduleId, String code) throws Exception {
        String sql = "select * from ci_property a where a.module_id = :moduleId and a.code = :code and a.is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("code", code);
        List<Property> list = this.list(Property.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<Property> queryParams(List<Param> params, Long moduleId) throws Exception {
        String sql = "select * from ci_property a where a.module_id = :moduleId and a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        final Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        paramMap.put("moduleId", moduleId);
        return this.list(Property.class, sql, paramMap);
    }

    public List<Property> list(final Pager pager, final Long moduleId) throws Exception {
        String sql = "select * from ci_property a where a.module_id = :moduleId and a.is_deleted=0 and a.type not in ('SINGLELINETABLE','MULTILINETABLE') ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        paramMap.put("moduleId", moduleId);
        return this.list(Property.class, sql, paramMap);
    }

    public List<Property> list(final Pager pager, final Long moduleId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select * from "+positionKey+".ci_property a where a.module_id = :moduleId and a.is_deleted=0 and a.type not in ('SINGLELINETABLE','MULTILINETABLE') ";
        } else {
            sql = "select * from ci_property a where a.module_id = :moduleId and a.is_deleted=0 and a.type not in ('SINGLELINETABLE','MULTILINETABLE') ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        paramMap.put("moduleId", moduleId);
        return this.list(Property.class, sql, paramMap);
    }

    public int count(final List<Param> params, final Long moduleId) throws Exception {
        String sql = "select count(1) from ci_property a where a.module_id = :moduleId and a.is_deleted=0 and a.type not in ('SINGLELINETABLE','MULTILINETABLE') ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        param.put("moduleId", moduleId);
        return this.countQuery(sql, param).intValue();
    }

    public int count(final List<Param> params, final Long moduleId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select count(1) from "+positionKey+".ci_property a where a.module_id = :moduleId and a.is_deleted=0 and a.type not in ('SINGLELINETABLE','MULTILINETABLE') ";
        } else {
            sql = "select count(1) from ci_property a where a.module_id = :moduleId and a.is_deleted=0 and a.type not in ('SINGLELINETABLE','MULTILINETABLE') ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        param.put("moduleId", moduleId);
        return this.countQuery(sql, param).intValue();
    }

    public List<Property> listIsTitleByModuleId(Long moduleId) {
        // 模型界面显示未隐藏字段
        String sql = "select a.* from ci_property a where a.is_deleted = 0  and a.module_id = :moduleId and is_title = 1  order by a.order_num";
        return this.list(Property.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public List<Property> listIsTitleByModuleId(Long moduleId, String positionKey) {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from "+positionKey+".ci_property a where a.is_deleted = 0  and a.module_id = :moduleId and is_title = 1  order by a.order_num";
        } else {
            sql = "select a.* from ci_property a where a.is_deleted = 0  and a.module_id = :moduleId and is_title = 1  order by a.order_num";
        }
        // 模型界面显示未隐藏字段
        return this.list(Property.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public List<Property> appointPropertylist(List<Long> ids) {
        // 模型界面显示未隐藏字段
        String sql = "select a.* from ci_property a where a.is_deleted = 0 and a.id in (:ids) order by a.order_num";
        return this.list(Property.class, sql, MapTools.simpleMap("ids", ids));
    }

    // 查询不是引入属性池中的属性信息
    public List<Property> queryListNotLink(List<Long> ids) {
        String sql = "select a.* from ci_property a where a.is_deleted = 0 and a.property_pool_id is null and a.id in (:ids) order by a.order_num";
        return this.list(Property.class, sql, MapTools.simpleMap("ids", ids));
    }

    public List<Property> queryListByPoolIds(Long propertyPoolId) {
        // 模型界面显示未隐藏字段
        String sql = "select a.* from ci_property a where a.is_deleted = 0 and a.property_pool_id = :propertyPoolId";
        return this.list(Property.class, sql, MapTools.simpleMap("propertyPoolId", propertyPoolId));
    }

    public List<Property> queryByLinkModuleId(Long id) {
        // 模型界面显示未隐藏字段
        String sql = "select a.* from ci_property a where a.is_deleted = 0 and a.link_module_id = :id";
        return this.list(Property.class, sql, MapTools.simpleMap("id", id));
    }

}
