package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.RelationCategory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description: 关联关系分组Dao
 * @Author: majiali
 * @CreateTime: 2022/5/5 14:03
 */
@Repository
public class RelationCategoryRepository extends BasicGenericDao<RelationCategory, Long> {
    public RelationCategoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_relation_category set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<RelationCategory> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.* from ci_relation_category a where is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        return this.list(RelationCategory.class, sql, getQueryBuilder().getParam(params));
    }

    public List<RelationCategory> list(List<Param> params, Map<String, String> sorter, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_relation_category a where is_deleted=0 ";
        } else {
            sql = "select a.* from ci_relation_category a where is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        return this.list(RelationCategory.class, sql, getQueryBuilder().getParam(params));
    }

    public RelationCategory queryByCode(String code) throws Exception {
        String sql = "select a.* from ci_relation_category a where a.code = :code and is_deleted=0 ";
        List<RelationCategory> list = this.list(RelationCategory.class, sql, MapTools.simpleMap("code", code));
        return list.isEmpty() ? null : list.get(0);
    }

    public RelationCategory query(Long id) throws Exception {
        String sql = "select a.* from ci_relation_category a where a.id = :id and is_deleted=0 ";
        List<RelationCategory> list = this.list(RelationCategory.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public RelationCategory query(Long id, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_relation_category a where a.id = :id and is_deleted=0 ";
        } else {
            sql = "select a.* from ci_relation_category a where a.id = :id and is_deleted=0 ";
        }
        List<RelationCategory> list = this.list(RelationCategory.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<RelationCategory> list(Pager pager) throws Exception {
        String sql = "select a.* from ci_relation_category a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(RelationCategory.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<RelationCategory> list(Pager pager, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_relation_category a where a.is_deleted=0 ";
        } else {
            sql = "select a.* from ci_relation_category a where a.is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(RelationCategory.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_relation_category a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select count(1) from " + positionKey + ".ci_relation_category a where a.is_deleted=0 ";
        } else {
            sql = "select count(1) from ci_relation_category a where a.is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public List<RelationCategory> listByModuleId(Long moduleId) throws Exception {
        String sql = "select a.* from ci_relation_category a where is_deleted = 0 and a.id in (select distinct relation_category_id from ci_module_relation where is_deleted=0 and (source_id = :moduleId OR target_id = :moduleId))  order by a.gmt_modify desc";
        return this.list(RelationCategory.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public List<RelationCategory> listByModuleId(Long moduleId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_relation_category a where is_deleted = 0 and a.id in (select distinct relation_category_id from " + positionKey + ".ci_module_relation where is_deleted=0 and (source_id = :moduleId OR target_id = :moduleId))  order by a.gmt_modify desc";
        } else {
            sql = "select a.* from ci_relation_category a where is_deleted = 0 and a.id in (select distinct relation_category_id from ci_module_relation where is_deleted=0 and (source_id = :moduleId OR target_id = :moduleId))  order by a.gmt_modify desc";
        }
        return this.list(RelationCategory.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }


    public RelationCategory queryByName(String name) throws Exception {
        String sql = "select a.* from ci_relation_category a where a.name = :name and is_deleted=0 ";
        List<RelationCategory> list = this.list(RelationCategory.class, sql, MapTools.simpleMap("name", name));
        return list.isEmpty() ? null : list.get(0);
    }

}
