package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.TopologyConfig;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author 胡立伟
 * @program: bocloud.ams
 * @description:
 * @date 2022/5/12 17:56
 */
@Repository
public class TopologyConfigRepository extends BasicGenericDao<TopologyConfig, Long> {

    public TopologyConfigRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public int count(List<Param> params) throws Exception {
        String sql = " select count(1) from ci_topology_config a where is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = " select count(1) from " + positionKey + ".ci_topology_config a where is_deleted=0 ";
        } else {
            sql = " select count(1) from ci_topology_config a where is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public List<TopologyConfig> list(Pager pager) throws Exception {
        String sql = "select a.* from ci_topology_config a where is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TopologyConfig.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<TopologyConfig> list(Pager pager, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_topology_config a where is_deleted=0 ";
        } else {
            sql = "select a.* from ci_topology_config a where is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TopologyConfig.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public TopologyConfig query(Long id) throws Exception {
        String sql = "select * from ci_topology_config  where is_deleted = 0 and id = :id ";
        List<TopologyConfig> list = this.list(TopologyConfig.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public TopologyConfig query(Long id, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select * from " + positionKey + ".ci_topology_config  where is_deleted = 0 and id = :id ";
        } else {
            sql = "select * from ci_topology_config  where is_deleted = 0 and id = :id ";
        }
        List<TopologyConfig> list = this.list(TopologyConfig.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = " update ci_topology_config set is_deleted = 1 , gmt_modify = :gmtModify , mender_id = :userId where id = :id ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public int countSameNameOrCode(String name, Long id, String configType) throws Exception {
        String sql = "select count(1) from ci_topology_config where is_deleted = 0 and name = :name ";
        final Map<String, Object> params = MapTools.simpleMap("name", name);
        if (null != id) {
            sql = sql + " and id != :id ";
            params.put("id", id);
        }
        if (StringUtils.isNotEmpty(configType)) {
            sql = sql + " and config_type = :configType ";
            params.put("configType", configType);
        }
        return this.countQuery(sql, params).intValue();
    }

    public List<TopologyConfig> listAll() throws Exception {
        String sql = "select a.* from ci_topology_config a where is_deleted=0 ";
        return this.list(TopologyConfig.class, sql);
    }

    public List<TopologyConfig> queryListBycategoryId(Long categoryId) throws Exception {
        String sql = "select a.* from ci_topology_config a where is_deleted=0 and a.category_id = :categoryId ";
        return this.list(TopologyConfig.class, sql, MapTools.simpleMap("categoryId", categoryId));
    }

}
