package com.bocloud.ams.service.collection.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.TableLog;
import com.bocloud.ams.entity.collection.CollectionTaskInstance;
import com.bocloud.ams.entity.model.TaskInstanceBean;
import com.bocloud.ams.repository.TableLogRepository;
import com.bocloud.ams.repository.collection.CollectionTaskInstanceRepository;
import com.bocloud.ams.service.collection.CollectionTaskInstanceService;
import com.bocloud.ams.task.model.HostResult;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @Author liuyuanyuan
 * @Date 2022/6/13 0013 20:55
 * @Description:
 */
@Slf4j
@Service("collectionTaskInstanceService")
public class CollectionTaskInstanceServiceImpl implements CollectionTaskInstanceService {
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CollectionTaskInstanceRepository collectionTaskInstanceRepository;
    @Autowired
    private TableLogRepository tableLogRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public GeneralResult<Void> updateStatus(Long id, String status) {
        String path = CollectionTaskInstance.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CollectionTaskInstance temp = collectionTaskInstanceRepository.query(id);
            if (null == temp) {
                return new GeneralResult<>(false, "任务实例不存在");
            }
            collectionTaskInstanceRepository.updateStatus(id, status);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("Task instance modify fail:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    public static String[] getNullProperties (TaskInstanceBean bean) {
        final BeanWrapper src = new BeanWrapperImpl(bean);
        java.beans.PropertyDescriptor[] pds = src.getPropertyDescriptors();

        Set<String> emptyNames = new HashSet<>();
        for(java.beans.PropertyDescriptor pd : pds) {
            Object srcValue = src.getPropertyValue(pd.getName());
            if (srcValue == null) emptyNames.add(pd.getName());
        }
        String[] result = new String[emptyNames.size()];
        return emptyNames.toArray(result);
    }

    @Override
    public GeneralResult<CollectionTaskInstance> modify(TaskInstanceBean bean, Long target) {
        String path = CollectionTaskInstance.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CollectionTaskInstance temp = collectionTaskInstanceRepository.query(bean.getId());
            if (null == temp) {
                return new GeneralResult<>(false, "任务实例不存在");
            }
            BeanUtils.copyProperties(bean, temp, getNullProperties(bean));
            temp.setGmtModify(new Date());
            temp.setMenderId(target);
            collectionTaskInstanceRepository.update(temp);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("Task instance modify fail:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<CollectionTaskInstance> queryByTaskId(String taskId) {
        try {
            CollectionTaskInstance instance = collectionTaskInstanceRepository.queryByTaskId(taskId);
            return new GeneralResult<>(true, instance, "查询成功");
        } catch (Exception e) {
            log.error("query instance is fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean<CollectionTaskInstance>> list(Pager pager, String positionKey) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = 0;
            if (StringUtils.isNotEmpty(positionKey)) {
                total = this.collectionTaskInstanceRepository.countByPositionKey(pager.getParams(), positionKey);
            } else {
                total = this.collectionTaskInstanceRepository.count(pager.getParams());
            }
            if (pager.getSimple()) {
                List<SimpleBean> beans =
                        this.collectionTaskInstanceRepository.list(pager.getParams(), pager.getSorter(), positionKey);
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<CollectionTaskInstance> list = this.collectionTaskInstanceRepository.list(pager.getPage(),
                        pager.getRows(), pager.getParams(), pager.getSorter(), positionKey);
                list.forEach(instance -> instance
                        .setCreator(redisTemplate.opsForValue().get("User_Name_" + instance.getCreatorId())));
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<CollectionTaskInstance>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query Collection Task instance list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean<CollectionTaskInstance>> bakList(Pager pager, String positionKey) {
        GridBean gridBean;
        try {
            long start = System.currentTimeMillis();
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            String startTime = null;
            String endTime = null;
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("gmtCreate")) {
                    String time = paramMap.get("gmtCreate").toString();
                    if (Sign.GET.name().equals(param.getSign().name())) {
                        startTime = time;
                    } else if (Sign.LET.name().equals(param.getSign().name())) {
                        endTime = time;
                    }
                }
            }
            if (StringUtils.isEmpty(startTime) || StringUtils.isEmpty(endTime)) {
                return new GeneralResult<>(false, "日期必传！");
            }
            int total = 0;
            //根据模板表查询出所有日志表 然后 使用并行流查询包含参数日期的库表
            List<TableLog> tableLogs = tableLogRepository.query("ci_collection_task_instance_bak", positionKey);
            List<String> tableNames = tableLogs.stream().map(TableLog::getName).collect(Collectors.toList());
            String finalTimeGet = startTime;
            String finalTimeLet = endTime;
            //获取到数据中包含所查询的日期的历史备份表
            //long start1 = System.currentTimeMillis();
            List<String> tableList = tableNames.parallelStream().filter(item -> {
                List<CollectionTaskInstance> query = new ArrayList<>();
                try {
                    query = collectionTaskInstanceRepository.query(item, finalTimeGet, finalTimeLet, positionKey);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return !ListTool.isEmpty(query);
            }).collect(Collectors.toList());
            //long end1 = System.currentTimeMillis();
            //log.info("耗时1>>>>>>::::"+(end1-start1));
            //并行流计算count
            //long start2 = System.currentTimeMillis();
            List<Integer> totals = tableList.parallelStream().map(item -> {
                try {
                    return collectionTaskInstanceRepository.count(pager.getParams(), item, positionKey);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return 0;
            }).collect(Collectors.toList());
            //long end2 = System.currentTimeMillis();
            //log.info("耗时2>>>>>>::::"+(end2-start2));
            total = totals.stream().collect(Collectors.summingInt(Integer::valueOf));
            //先计算出需要id集合
            //long start3 = System.currentTimeMillis();
            List<CollectionTaskInstance> collectionTaskInstances = tableList.stream().map(item ->
                            CompletableFuture.supplyAsync(() -> {
                                try {
                                    return this.collectionTaskInstanceRepository.listIdBak(pager.getParams(), pager.getSorter(), item, positionKey);
                                } catch (Exception e) {
                                    e.printStackTrace();
                                }
                                return new ArrayList<CollectionTaskInstance>();
                            })
                    )
                    .collect(Collectors.toList())
                    .stream()
                    .map(item -> item.join())
                    .flatMap(item -> item.stream())
                    .sorted((o1, o2) -> o2.getId().compareTo(o1.getId()))
                    .skip((pager.getPage() - 1) * pager.getRows())
                    .limit(pager.getRows())
                    .collect(Collectors.toList());
            //long end3 = System.currentTimeMillis();
            //log.info("耗时3>>>>>>::::"+(end3-start3));
            Map<String, List<CollectionTaskInstance>> collectMaps = collectionTaskInstances.stream().collect(Collectors.groupingBy(CollectionTaskInstance::getTableName));
            Set<String> keySet = collectMaps.keySet();
            List<String> keyList = new ArrayList<>(keySet);
            //long start4 = System.currentTimeMillis();
            //根据id集合获取最终数据
            List<CollectionTaskInstance> collectionTaskInstanceList = keyList.stream().map(item ->
                            CompletableFuture.supplyAsync(() -> {
                                try {
                                    String ids = collectMaps.get(item).stream().map(m -> m.getId().toString()).collect(Collectors.joining(","));
                                    Pager pg = new Pager(null, null, new ArrayList<Param>(), null);
                                    pg.getParams().add(new Param(MapTools.simpleMap("id", ids), Sign.IN));
                                    return this.collectionTaskInstanceRepository.list(pg, item, positionKey);
                                } catch (Exception e) {
                                    e.printStackTrace();
                                }
                                return new ArrayList<CollectionTaskInstance>();
                            })
                    )
                    .collect(Collectors.toList())
                    .stream()
                    .map(item -> item.join())
                    .flatMap(item -> item.stream())
                    .sorted((o1, o2) -> o2.getId().compareTo(o1.getId()))
                    .collect(Collectors.toList());
            //long end4 = System.currentTimeMillis();
            //log.info("耗时4>>>>>>::::"+(end4-start4));
            collectionTaskInstanceList.forEach(instance -> instance.setCreator(redisTemplate.opsForValue().get("User_Name_" + instance.getCreatorId())));
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, collectionTaskInstanceList);
            long end = System.currentTimeMillis();
            log.info("总耗时>>>>>>::::"+(end-start));
            return new GeneralResult<GridBean<CollectionTaskInstance>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query Collection Task instance list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<CollectionTaskInstance> detail(Long id, String positionKey) {
        try {
            CollectionTaskInstance instance = null;
            if (StringUtils.isEmpty(positionKey)) {
                instance = this.collectionTaskInstanceRepository.query(id);

            } else {
                instance = this.collectionTaskInstanceRepository.queryByPositionKey(id, positionKey);
            }
            Assert.notNull(instance, "数据不存在！");
            List<Object> successes = Lists.newArrayList();
            List<Object> failures = Lists.newArrayList();
            if (null != instance.getOutput()) {
                GeneralResult result = JSONObject.parseObject(instance.getOutput(), GeneralResult.class);
                if (!instance.getCode().contains("method") && !instance.getCode().contains("database")) {
                    if (null != result.getData()) {
                        List<HostResult> results =
                                JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
                        for (HostResult hostResult : results) {
                            // 查询主机名称
                            if ("task.maintain.script".equalsIgnoreCase(instance.getCode())) {
                                List<JSONObject> targets = JSONObject.parseObject(instance.getInput())
                                        .getJSONObject("target").getJSONArray("data").toJavaList(JSONObject.class);
                                Map<String, JSONObject> hosts = Maps.newHashMap();
                                for (JSONObject target : targets) {
                                    hosts.put(target.getString("host"), target);
                                }
                                if (hosts.containsKey(hostResult.getHost())) {
                                    hostResult.setRawName(hosts.get(hostResult.getHost()).getString("name"));
                                }
                            }
                            if (hostResult.isSuccess()) {
                                successes.add(hostResult);
                            } else {
                                failures.add(hostResult);
                            }
                        }
                    }
                }
            }
            instance.setSuccesses(successes);
            instance.setFailures(failures);
            return new GeneralResult<>(true, instance, "查询任务详细成功");
        } catch (Exception e) {
            log.error("Query task instance detail fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<CollectionTaskInstance> bakDetail(Long id, String name, String positionKey) {
        try {
            CollectionTaskInstance instance = this.collectionTaskInstanceRepository.query(id, name, positionKey);
            Assert.notNull(instance, "数据不存在！");
            List<Object> successes = Lists.newArrayList();
            List<Object> failures = Lists.newArrayList();
            if (null != instance.getOutput()) {
                GeneralResult result = JSONObject.parseObject(instance.getOutput(), GeneralResult.class);
                if (!instance.getCode().contains("method") && !instance.getCode().contains("database")) {
                    if (null != result.getData()) {
                        List<HostResult> results =
                                JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
                        for (HostResult hostResult : results) {
                            // 查询主机名称
                            if ("task.maintain.script".equalsIgnoreCase(instance.getCode())) {
                                List<JSONObject> targets = JSONObject.parseObject(instance.getInput())
                                        .getJSONObject("target").getJSONArray("data").toJavaList(JSONObject.class);
                                Map<String, JSONObject> hosts = Maps.newHashMap();
                                for (JSONObject target : targets) {
                                    hosts.put(target.getString("host"), target);
                                }
                                if (hosts.containsKey(hostResult.getHost())) {
                                    hostResult.setRawName(hosts.get(hostResult.getHost()).getString("name"));
                                }
                            }
                            if (hostResult.isSuccess()) {
                                successes.add(hostResult);
                            } else {
                                failures.add(hostResult);
                            }
                        }
                    }
                }
            }
            instance.setSuccesses(successes);
            instance.setFailures(failures);
            int i = name.lastIndexOf("_");
            String substring = name.substring(i + 1);
            instance.setTableName("ci_collection_history_bak_" + substring);
            return new GeneralResult<>(true, instance, "查询任务详细成功");
        } catch (Exception e) {
            log.error("Query task instance detail fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

}
