package com.bocloud.ams.service.collection.impl;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.collection.CollectionComponent;
import com.bocloud.ams.entity.collection.CollectionTask;
import com.bocloud.ams.entity.collection.CollectionTemplate;
import com.bocloud.ams.entity.enums.CollectionEnum;
import com.bocloud.ams.entity.model.TaskGraph;
import com.bocloud.ams.repository.collection.CollectionComponentRepository;
import com.bocloud.ams.repository.collection.CollectionTaskRepository;
import com.bocloud.ams.repository.collection.CollectionTemplateRepository;
import com.bocloud.ams.service.collection.CollectionTemplateService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @Author liuyuanyuan
 * @Date 2022/6/7 0007 14:32
 * @Description:
 */
@Slf4j
@Service("collectionTemplateService")
public class CollectionTemplateServiceImpl implements CollectionTemplateService {
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CollectionTemplateRepository collectionTemplateRepository;
    @Autowired
    private CollectionComponentRepository collectionComponentRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private CollectionTaskRepository collectionTaskRepository;

    @Override
    public GeneralResult<GridBean<CollectionComponent>> components(Pager pager, String positionKey) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.collectionComponentRepository.count(pager.getParams(), positionKey);
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.collectionComponentRepository.list(pager.getParams(), pager.getSorter(), positionKey);
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<CollectionComponent> list = this.collectionComponentRepository.list(pager.getPage(),
                    pager.getRows(), pager.getParams(), pager.getSorter());
                list.forEach(template -> template
                    .setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId())));
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<CollectionComponent>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query Collection Component list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean<CollectionTemplate>> list(Pager pager, String positionKey) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.collectionTemplateRepository.count(pager.getParams(), positionKey);
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.collectionTemplateRepository.list(pager.getParams(), pager.getSorter(), positionKey);
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<CollectionTemplate> list = this.collectionTemplateRepository.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(), positionKey);
                list.forEach(template -> {
                    template.setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId()));
                    List<CollectionComponent> components = null;
                    try {
                        components = collectionComponentRepository.queryByTemplateId(template.getId(), positionKey);
                    } catch (Exception e) {
                        log.error("Query Collection Components list fail:", e);
                    }
                    template.setComponents(components);
                });
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<CollectionTemplate>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query Collection Template list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<CollectionTemplate> save(CollectionTemplate template, Long target) {
        try {
            // 判断某种类的作业模板名称是否存在
            boolean flag = collectionTemplateRepository.exists(template.getName());
            if (flag) {
                return new GeneralResult<>(false, "该插件名称已存在，无法添加");
            }
            GeneralResult<CollectionTemplate> result = saveTask(template, true);
            if (result.isFailed()) {
                return result;
            }
            result = saveComponent(template, true, target);
            if (result.isFailed()) {
                return result;
            }
            return new GeneralResult<>(true, "添加采集模板成功");
        } catch (Exception e) {
            log.error("save collection template error:", e);
            return new GeneralResult<>(false, "添加采集模板失败");
        }
    }

    private GeneralResult<CollectionTemplate> saveComponent(CollectionTemplate template, boolean isAdd, Long userId) {
        try {
            List<CollectionComponent> collectionComponents = template.getComponents();
            List<CollectionComponent> components = collectionComponents.stream().map(component -> {
                component.setTemplateId(template.getId());
                component.setCreatorId(userId);
                component.setGmtCreate(new Date());
                component.setId(null);
                return component;
            }).collect(Collectors.toList());
            if (!isAdd) {
                collectionComponentRepository.deleteByTemplate(template.getId(), userId);
            }
            if (!collectionComponents.isEmpty()) {
                collectionComponentRepository.batchSave(components, 100);
            }
        } catch (Exception e) {
            log.error("Save collection components fail:", e);
            return new GeneralResult<>(false, "保存采集模板组件失败");
        }
        return new GeneralResult<>(true, template, "保存采集模板组件成功");
    }

    private GeneralResult<CollectionTemplate> saveTask(CollectionTemplate template, boolean isAdd) {
        TaskGraph graph = JSONObject.parseObject(JSONObject.toJSONString(template.getGraph()), TaskGraph.class);
        if (CollectionEnum.collectionType.SCRIPT.getValue().equalsIgnoreCase(graph.getCollectionType())) {
            if (null == graph.getContent()) {
                return new GeneralResult<>(false, "脚本内容为空");
            }
            if (null == graph.getCategory()) {
                return new GeneralResult<>(false, "脚本类型为空");
            }
        }
        try {
            // isAdd为true新增作业模板
            template.setGraph(JSONObject.toJSONString(template.getGraph()));
            if (isAdd) {
                template.setGmtCreate(new Date());
                this.collectionTemplateRepository.save(template);
            } else {
                // 更新作业模板
                template.setGmtModify(new Date());
                this.collectionTemplateRepository.update(template);
            }
        } catch (Exception e) {
            log.error("Save collection template fail:", e);
            return new GeneralResult<>(false, "保存采集模板失败");
        }
        return new GeneralResult<>(true, template, "保存采集模板成功");
    }

    @Override
    public GeneralResult<CollectionTemplate> modify(CollectionTemplate template, Long target) {
        String path = CollectionTemplate.class.getSimpleName() + "_" + template.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CollectionTemplate collectionTemplate = this.collectionTemplateRepository.query(template.getId());
            Assert.notNull(collectionTemplate, "数据不存在！");
            if (!collectionTemplate.getName().equals(template.getName())) {
                // 判断某种类的作业模板名称是否存在
                boolean flag = collectionTemplateRepository.exists(template.getName());
                if (flag) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
            }
            collectionTemplate.setGraph(template.getGraph());
            collectionTemplate.setName(template.getName());
            collectionTemplate.setRemark(template.getRemark());
            collectionTemplate.setCollectionType(template.getCollectionType());
            GeneralResult<CollectionTemplate> result = saveTask(collectionTemplate, false);
            if (result.isFailed()) {
                return result;
            }
            result = saveComponent(template, false, target);
            if (result.isFailed()) {
                return result;
            }
            return new GeneralResult<>(true, "修改采集模板成功");
        } catch (Exception e) {
            log.error("Update collection template fail:", e);
            return new GeneralResult<>(false, "修改采集模板失败");
        }
    }

    @Override
    public GeneralResult<CollectionTemplate> detail(Long id, String positionKey) {
        try {
            CollectionTemplate template = collectionTemplateRepository.query(id, positionKey);
            template.setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId()));
            Assert.notNull(template, "数据不存在！");
            List<CollectionComponent> components = collectionComponentRepository.queryByTemplateId(id);
            template.setComponents(components);
            return new GeneralResult<>(true, template, "查询采集模板详细成功");
        } catch (Exception e) {
            log.error("Query collection detail template fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long target) {
        String path = CollectionTemplate.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 判断当前插件是否有被引用
            List<CollectionTask> tasks = collectionTaskRepository.queryByTemplateId(id);
            if (0 < tasks.size()) {
                return new GeneralResult<>(false, "当前插件存在采集任务，不允许删除");
            }
            collectionTemplateRepository.delete(id, target);
            collectionComponentRepository.deleteByTemplate(id, target);
            return new GeneralResult<>(true, "删除采集模板成功");
        } catch (Exception e) {
            log.error("Remove collection template fail:", e);
            return new GeneralResult<>(false, "删除采集模板失败");
        }
    }

}
