package com.bocloud.ams.service.core;

import com.bocloud.ams.entity.config.CmdbConfig;
import com.bocloud.ams.repository.config.CmdbConfigRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.List;

@Slf4j
@Component
public class CmdbConfigCache implements InitializingBean {

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private CmdbConfigRepository cmdbConfigRepository;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;


    public void putConfigCache(String code, String value) {
        this.redisTemplate.opsForValue().set(code, value);
    }

    public String getConfigValue(String code) {
        return redisTemplate.opsForValue().get(code);
    }

    @Override
    public void afterPropertiesSet() {
        taskExecutor.execute(this::addConfigCache);
    }

    public void addConfigCache() {
        try {
            List<CmdbConfig> configs = cmdbConfigRepository.listAll();
            for (CmdbConfig config : configs) {
                if (null == config.getValue()) {
                    continue;
                }
                if (!StringUtils.hasText(getConfigValue(config.getCode()))) {
                    putConfigCache(config.getCode(), config.getValue());
                }
            }
        } catch (Exception e) {
            log.error("put cmdb config cache error,", e);
        }
    }

}
