package com.bocloud.ams.service.instance.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.ams.entity.Role;
import com.bocloud.ams.entity.audit.OperationalAudit;
import com.bocloud.ams.entity.collection.CollectionTask;
import com.bocloud.ams.entity.enums.InstanceEnum;
import com.bocloud.ams.entity.enums.OperationalAuditType;
import com.bocloud.ams.entity.instance.*;
import com.bocloud.ams.entity.model.*;
import com.bocloud.ams.entity.module.*;
import com.bocloud.ams.repository.audit.OperationalAuditRepository;
import com.bocloud.ams.repository.collection.CollectionTaskRepository;
import com.bocloud.ams.repository.instance.*;
import com.bocloud.ams.repository.module.*;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.audit.OperationalAuditService;
import com.bocloud.ams.service.instance.InstanceAutoRelationService;
import com.bocloud.ams.service.instance.InstanceService;
import com.bocloud.ams.service.instance.InstanceValueService;
import com.bocloud.ams.service.internal.SmsInternalService;
import com.bocloud.ams.service.module.PropertyService;
import com.bocloud.ams.service.utils.*;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URLDecoder;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @ClassName: CmdbInstanceServiceImpl
 * @Description
 * @Author yangzhenrong
 * @Date 2022/5/11 11:25
 * @Version 1.0
 **/
@Slf4j
@Service("instanceService")
public class InstanceServiceImpl implements InstanceService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private InstanceValueService instanceValueService;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private PropertyRepository propertyRepository;
    @Autowired
    private PropertyItemRepository propertyItemRepository;
    @Autowired
    private RegularExpressionRepository regularExpressionRepository;
    @Autowired
    private InstanceAuthorityRepository instanceAuthorityRepository;
    @Autowired
    private SmsInternalService smsInternalService;
    @Autowired
    private InstanceHistoryHelper instanceHistoryHelper;
    @Autowired
    private InstanceHistoryRepository instanceHistoryRepository;
    @Autowired
    private RepositoryHandler handler;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;
    @Autowired
    private InstanceExcelHelper excelHelper;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;
    @Autowired
    private RelationCategoryRepository relationCategoryRepository;
    @Autowired
    private ModuleRelationRepository moduleRelationRepository;
    @Autowired
    private OperationalAuditRepository operationalAuditRepository;
    @Autowired
    private OperationalAuditService operationalAuditService;
    @Autowired
    private InstanceAutoRelationService instanceAutoRelationService;
    @Autowired
    private PropertyService propertyService;
    @Autowired
    PropertyGroupRepository propertyGroupRepository;
    @Autowired
    private CollectionTaskRepository collectionTaskRepository;

    @Autowired
    private EventPublisher eventPublisher;
    double progressLast = 0L;

    @Override
    public GeneralResult<Void> createTable(String moduleCode, Long userId) {
        try {
            String instanceTable = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
            if (instanceRepository.countTable(instanceTable) > 0) {
                log.warn("创建表空间{}失败", instanceTable);
                return new GeneralResult<>(false, "创建失败");
            }
            instanceRepository.createTable(instanceTable, new Instance());
            instanceValueService.createTable(moduleCode, userId);
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("create module instance table error:", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    public GeneralResult<Void> dropTable(String moduleCode, Long userId) {
        try {
            String instanceTable = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
            if (instanceRepository.countTable(instanceTable) == 0) {
                log.warn("删除表空间{}失败", instanceTable);
                return new GeneralResult<>(false, "删除失败");
            }
            instanceRepository.dropTable(instanceTable);
            instanceValueService.dropTable(moduleCode, userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("drop module instance table error:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult list(Pager pager, Long userId, String positionKey) {
        try {
            List<Map<String, Object>> list = new ArrayList<>();
            if (CollectionUtils.isEmpty(pager.getParams())) {
                GridBean<Map<String, Object>> bean = new GridBean<>(pager.getPage(), pager.getRows(), 0, list);
                return new GeneralResult<>(true, bean, "查询成功");
            }
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            pager.getSorter().put("id", Common.ONE);
            // 模型
            Long moduleId = null;
            String moduleCode = "";
            // 实例属性条件查询
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                Map<String, Object> newMap = new HashMap<>();
                for (String key : paramMap.keySet()) {
                    if ("moduleCode".equals(key)) {
                        if (!StringUtils.isEmpty(paramMap.get(key).toString())) {
                            moduleCode = paramMap.get(key).toString();
                        }
                    } else if ("moduleId".equals(key)) {
                        if (!StringUtils.isEmpty(paramMap.get(key).toString())) {
                            moduleId = Long.valueOf(paramMap.get(key).toString());
                        }
                        newMap.put(key, paramMap.get(key));
                    } else {
                        // 将大写字母转化为小写（moduleCode和moduleId之外的参数均为模型属性code参数）
                        newMap.put(key.toLowerCase(), paramMap.get(key));
                    }
                }
                param.setParam(newMap);
            }
            if (StringUtils.isEmpty(moduleCode)) {
                CiModule modle = moduleRepository.query(moduleId);
                if (null == modle) {
                    return new GeneralResult<>(false, "模型不存在");
                }

                moduleCode = modle.getCode();

            }

            String table = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            boolean hasAdminRole = checkAdminRole(userId);
            int total = instanceRepository.count(pager.getParams(), table, valueTable, userId, hasAdminRole, positionKey);
            list = this.instanceRepository.list(pager, table, valueTable, userId, hasAdminRole, positionKey);
            convertInstanceValue(list, moduleId, userId, hasAdminRole, null);
            GridBean<Map<String, Object>> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("instance and value list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult create(JSONObject object, Long userId, boolean isAutoRelation) {
        try {
            // 填充默认值、实例预校验（检验实例的属性值是否满足规则，包括是否必填，最大值、最小值、规则表达式、唯一性等 如果必填，需要补充默认值）
            GeneralResult initResult = initInstance(object, InstanceEnum.category.CREATE.name());
            if (initResult.isFailed()) {
                log.error("实例预校验失败{}", initResult.getMessage());
                return initResult;
            }
            Instance instance = (Instance) initResult.getData();
            String table = CmdbUtils.CMDB_INSTANCE + instance.getModuleCode().toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + instance.getModuleCode().toLowerCase();
            Instance cmdbInstance = instanceRepository.queryByUuid(table, instance.getUuid(), instance.getModuleId());
            if (null != cmdbInstance) {
                return new GeneralResult<>(false, "实例唯一标识已存在");
            }
            instance.setDeleted(false);
            instance.setCreatorId(userId);
            instance.setGmtCreate(new Date());
            handler.saveInstance(instance, table);

            List<InstanceValue> instanceValues = instance.getInstanceValues();
            // 实例值记录中查询出实例名称
            List<Property> propertyList = queryModulePropertys(instance.getModuleId());
            Map<String, String> propertyMap = new HashMap<>();
            propertyList.forEach(item -> {
                propertyMap.put(item.getCode(), item.getName());
            });

            instanceValues.forEach(item -> {
                item.setName(propertyMap.get(item.getPropertyCode()));
            });
            String instanceName = null;
            if (CollectionUtils.isNotEmpty(instanceValues)) {
                for (InstanceValue instanceValue : instanceValues) {
                    // 处理属性默认类型为自增ID及流水号的属性值
                    Property property = instanceValue.getProperty();
                    if (null != property) {
                        // 初始化时只保留了前缀
                        String propertyValue = instanceValue.getPropertyValue();
                        String defaultValueType = property.getDefaultValueType();
                        Integer startNum = property.getStartNum();
                        if (null == startNum) {
                            startNum = 0;
                        }
                        // 计算后面部分的数值，采用当前实例ID加上开始数值
                        Long endNum = instance.getId() + startNum.longValue();
                        if (StringUtils.isNotEmpty(defaultValueType) && !"null".equalsIgnoreCase(defaultValueType)) {
                            if (PropertyConstant.DefaultValueType.SERIALNUM.equalsIgnoreCase(defaultValueType)) {
                                String value = propertyValue + endNum;
                                // defaultValue是设置的流水号长度
                                String defaultValue = property.getDefaultValue();
                                if (StringUtils.isEmpty(defaultValue) || "null".equalsIgnoreCase(defaultValue)) {
                                    defaultValue = "1";
                                }
                                // 如果当前属性值大于流水号长度则删除已经生成的实例
                                if (value.length() > Integer.valueOf(defaultValue)) {
                                    instanceRepository.deleteById(instance.getId(), table);
                                    return new GeneralResult<>(false, "实例新增失败:属性" + property.getName() + "内容超过流水号长度");
                                }
                                String valueMiddle = "";
                                Integer remainderLength;
                                remainderLength = Integer.valueOf(defaultValue) - value.length();
                                if (remainderLength > 0) {
                                    for (int i = 0; i < remainderLength; i++) {
                                        valueMiddle += "0";
                                    }
                                }
                                propertyValue = propertyValue + valueMiddle + endNum;
                            } else if (PropertyConstant.DefaultValueType.AUTOINCRID
                                    .equalsIgnoreCase(defaultValueType)) {
                                propertyValue = propertyValue + endNum;
                            }
                            instanceValue.setPropertyValue(propertyValue);
                        }
                    }
                    if (Common.NAME.equals(instanceValue.getPropertyCode())) {
                        instanceName = instanceValue.getPropertyValue();
                    }
                    instanceValue.setDeleted(false);
                    instanceValue.setCreatorId(userId);
                    instanceValue.setGmtCreate(new Date());
                    instanceValue.setInstanceId(instance.getId());
                }
                handler.batchSaveInstanceValue(instanceValues, valueTable);
            }
            // 变更记录
            instanceHistoryHelper.history(instance, InstanceEnum.catalog.INSTANCE.name(),
                    InstanceEnum.category.CREATE.name(), userId);
            // 操作审计
            Map<String, Object> afterMap =
                    JSON.parseObject(JSON.toJSONString(instance), new TypeReference<Map<String, Object>>() {
                    });
            OperationalAudit cmdbOperationalAudit = operationalAuditService.buildLog(OperationalAuditType.ADD, "实例",
                    instanceName, instance.getModuleCode(), instance.getId(), userId, null, afterMap);
            operationalAuditRepository.save(cmdbOperationalAudit);
            if (isAutoRelation) {
                // 触发与已有实例之间绑定关系的任务
                instanceAutoRelationService.instanceAutoRelation(instance, userId);
            }

            return new GeneralResult<>(true, instance, "实例新增成功");
        } catch (Exception e) {
            log.error("instance and value create fail:", e);
            return new GeneralResult<>(false, "实例新增失败");
        }
    }

    @Override
    public GeneralResult<Void> batchRemove(InstanceBean model, Long userId) {
        try {
            String moduleCode = model.getModuleCode();
            CiModule module = moduleRepository.queryByCode(moduleCode);
            if (null == module) {
                return new GeneralResult<>(false, "模型不存在");
            }
            Long moduleId = module.getId();
            String table = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            List<Instance> instances = instanceRepository.queryList(table, model.getIds());
            if (CollectionUtils.isEmpty(instances)) {
                return new GeneralResult<>(false, "实例不存在");
            }
            GeneralResult<Void> checkRuslt = batchRemoveCheck(model.getIds(), moduleId);
            if (checkRuslt.isFailed()) {
                return new GeneralResult<>(false, "存在被采集任务使用的实例，不可删除！");
            }
            Map<Long, Instance> instanceMap = instances.stream()
                    .collect(Collectors.toMap(Instance::getId, Function.identity(), (oldData, newData) -> newData));
            boolean hasAdminRole = checkAdminRole(userId);

            // 先校验当前用户是否拥有删除权限，存在没权限直接跳过
            List<Long> instanceIds = new ArrayList<>();
            List<Instance> instanceList = new ArrayList<>();
            if (!hasAdminRole) {
                List<InstanceAuthority> instanceAuthoritys = instanceAuthorityRepository.queryByAuth(moduleId,
                        model.getIds(), userId, InstanceEnum.authority.DEL.name());
                Map<Long, InstanceAuthority> authorityMap = instanceAuthoritys.stream().collect(Collectors
                        .toMap(InstanceAuthority::getInstanceId, Function.identity(), (oldData, newData) -> newData));

                for (Long id : instanceMap.keySet()) {
                    Instance temp = instanceMap.get(id);
                    if (userId.equals(temp.getCreatorId())) {
                        instanceIds.add(id);
                        instanceList.add(temp);
                        continue;
                    }
                    InstanceAuthority authority = authorityMap.get(id);
                    if (null != authority) {
                        instanceIds.add(id);
                        instanceList.add(temp);
                    }
                }

            } else {
                instanceIds = model.getIds();
                instanceList = instances;
            }

            if (CollectionUtils.isEmpty(instanceIds)) {
                return new GeneralResult<>(false, "可操作的实例不存在");
            }

            // 审计日志记录，在删除前查询实例value信息
            List<InstanceValue> values = instanceValueRepository.queryByInstanceIds(valueTable, instanceIds);
            instanceValueRepository.batchRemoveByInstanceIds(instanceIds, valueTable, userId);
            instanceRepository.batchRemove(instanceIds, table, userId);
            instanceAuthorityRepository.batchRemove(moduleId, instanceIds, userId);
            instanceHistoryRepository.batchRemove(moduleId, instanceIds);
            // 删除关系
            List<ModuleRelation> relations = moduleRelationRepository.queryByModuleId(moduleId);
            for (ModuleRelation relation : relations) {
                String relationTable =
                        CmdbUtils.CMDB_INSTANCE_RELATION + relation.getRelationCategoryCode().toLowerCase();
                instanceRelationRepository.batchRemoveByInstanceIds(instanceIds, moduleId, relationTable, userId);
            }
            // 实例值记录中查询出实例名称
            List<Property> propertyList = queryModulePropertys(moduleId);
            Map<String, String> propertyMap = new HashMap<>();
            propertyList.forEach(item -> {
                propertyMap.put(item.getCode(), item.getName());
            });

            values.forEach(item -> {
                item.setName(propertyMap.get(item.getPropertyCode()));
            });
            Map<Long, List<InstanceValue>> valueMap =
                    values.stream().collect(Collectors.groupingBy(InstanceValue::getInstanceId));
            Map<Long, InstanceValue> valueNameMap =
                    values.stream().filter(item -> item.getPropertyCode().equals(Common.NAME)).collect(
                            Collectors.toMap(InstanceValue::getInstanceId, Function.identity(), (oldData, newData) -> newData));

            // 审计操作
            List<OperationalAudit> audits = new ArrayList<>();
            for (Instance temp : instanceList) {
                InstanceValue value = valueNameMap.get(temp.getId());
                if (null != value) {
                    temp.setInstanceValues(valueMap.get(temp.getId()));
                    // 操作审计
                    Map<String, Object> beforeMap =
                            JSON.parseObject(JSON.toJSONString(temp), new TypeReference<Map<String, Object>>() {
                            });
                    OperationalAudit audit = operationalAuditService.buildLog(OperationalAuditType.BATCH_DELETE, "实例",
                            value.getPropertyValue(), moduleCode, temp.getId(), userId, beforeMap, null);
                    audits.add(audit);
                }

            }
            if (CollectionUtils.isNotEmpty(audits)) {
                operationalAuditRepository.batchSave(audits, 100);
            }

            return new GeneralResult<>(true, "实例批量删除成功");
        } catch (Exception e) {
            log.error("instance and value batch remove fail:", e);
            return new GeneralResult<>(false, "实例批量删除失败");
        }
    }

    public GeneralResult<Void> removeCheck(Long id, Long moduleId) {
        try {
            List<CollectionTask> tasks = collectionTaskRepository.findScriptTaskByModuleId(moduleId);
            if (CollectionUtils.isEmpty(tasks)) {
                return new GeneralResult<>(true, "校验检查成功");
            }

            for (CollectionTask task : tasks) {
                if (StringUtils.isEmpty(String.valueOf(task.getGraph()))) {
                    break;
                }
                TaskGraph taskGraph = JSONObject.parseObject(String.valueOf(task.getGraph()), TaskGraph.class);
                if (null == taskGraph) {
                    break;
                }
                if (StringUtils.isEmpty(taskGraph.getTarget())) {
                    break;
                }
                JSONObject target = JSONObject.parseObject(taskGraph.getTarget());
                if (null == target) {
                    break;
                }
                if (CollectionUtils.isEmpty(target.getJSONArray("data"))) {
                    break;
                }
                List<JSONObject> data = target.getJSONArray("data").toJavaList(JSONObject.class);
                for (JSONObject targetData : data) {
                    Long instanceId = targetData.getLong("id");
                    if (instanceId != null && id.equals(instanceId)) {
                        return new GeneralResult<>(false, "校验检查失败");
                    }
                }
            }
            return new GeneralResult<>(true, "校验检查成功");
        } catch (Exception e) {
            log.error("instance remove check fail:", e);
            return new GeneralResult<>(false, "校验检查失败");
        }
    }

    public GeneralResult<Void> batchRemoveCheck(List<Long> ids, Long moduleId) {
        try {
            List<CollectionTask> tasks = collectionTaskRepository.findScriptTaskByModuleId(moduleId);
            if (CollectionUtils.isEmpty(tasks)) {
                return new GeneralResult<>(true, "校验检查成功");
            }

            for (CollectionTask task : tasks) {
                if (StringUtils.isEmpty(String.valueOf(task.getGraph()))) {
                    break;
                }
                TaskGraph taskGraph = JSONObject.parseObject(String.valueOf(task.getGraph()), TaskGraph.class);
                if (null == taskGraph) {
                    break;
                }
                if (StringUtils.isEmpty(taskGraph.getTarget())) {
                    break;
                }
                JSONObject target = JSONObject.parseObject(taskGraph.getTarget());
                if (null == target) {
                    break;
                }
                if (CollectionUtils.isEmpty(target.getJSONArray("data"))) {
                    break;
                }
                List<JSONObject> data = target.getJSONArray("data").toJavaList(JSONObject.class);
                for (JSONObject targetData : data) {
                    Long instanceId = targetData.getLong("id");
                    if (instanceId != null && ids.contains(instanceId)) {
                        return new GeneralResult<>(false, "校验检查失败");
                    }
                }
            }
            return new GeneralResult<>(true, "校验检查成功");
        } catch (Exception e) {
            log.error("instance remove check fail:", e);
            return new GeneralResult<>(false, "校验检查失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, String moduleCode, Long userId) {
        String path = Instance.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            String table = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            CiModule module = moduleRepository.queryByCode(moduleCode);
            if (null == module) {
                return new GeneralResult<>(false, "模型不存在");
            }
            Long moduleId = module.getId();
            Instance temp = instanceRepository.query(table, id);
            if (null == temp) {
                return new GeneralResult<>(false, "实例不存在");
            }
            GeneralResult<Void> checkRuslt = removeCheck(id, moduleId);
            if (checkRuslt.isFailed()) {
                return new GeneralResult<>(false, "该实例已被采集任务使用，不可删除！");
            }
            boolean hasAdminRole = checkAdminRole(userId);
            if (!hasAdminRole && !temp.getCreatorId().equals(userId)) {
                InstanceAuthority authority =
                        instanceAuthorityRepository.queryByAuth(moduleId, id, userId, InstanceEnum.authority.DEL.name());
                if (null == authority) {
                    return new GeneralResult<>(false, "实例无删除权限");
                }
            }
            List<InstanceValue> instanceValues = instanceValueRepository.listByInstanceId(id, valueTable);
            // 实例值记录中查询出实例名称
            List<Property> propertyList = queryModulePropertys(temp.getModuleId());
            Map<String, String> propertyMap = new HashMap<>();
            propertyList.forEach(item -> {
                propertyMap.put(item.getCode(), item.getName());
            });

            instanceValues.forEach(item -> {
                item.setName(propertyMap.get(item.getPropertyCode()));
            });
            temp.setInstanceValues(instanceValues);
            InstanceValue valueTemp = instanceValueRepository.query(valueTable, Common.NAME, id);
            if (null == valueTemp) {
                return new GeneralResult<>(false, "实例不存在");
            }
            instanceValueRepository.removeByInstanceId(id, valueTable, userId);
            instanceRepository.remove(id, table, userId);
            instanceAuthorityRepository.remove(moduleId, id, userId);
            instanceHistoryRepository.remove(moduleId, id);
            // 删除关系
            List<ModuleRelation> relations = moduleRelationRepository.queryByModuleId(moduleId);
            for (ModuleRelation relation : relations) {
                String relationTable =
                        CmdbUtils.CMDB_INSTANCE_RELATION + relation.getRelationCategoryCode().toLowerCase();
                instanceRelationRepository.removeByInstanceId(id, moduleId, relationTable, userId);
            }
            // 操作审计
            Map<String, Object> beforeMap =
                    JSON.parseObject(JSON.toJSONString(temp), new TypeReference<Map<String, Object>>() {
                    });
            OperationalAudit audit = operationalAuditService.buildLog(OperationalAuditType.DELETE, "实例",
                    valueTemp.getPropertyValue(), moduleCode, temp.getId(), userId, beforeMap, null);
            operationalAuditRepository.save(audit);
            return new GeneralResult<>(true, "实例删除成功");
        } catch (Exception e) {
            log.error("instance and value remove fail:", e);
            return new GeneralResult<>(false, "实例删除失败");
        }
    }

    @Override
    public GeneralResult<Void> batchModify(InstanceBean instanceModel, Long userId) {
        try {
            Long moduleId = instanceModel.getModuleId();
            String moduleCode = instanceModel.getModuleCode();
            String propertyCode = instanceModel.getPropertyCode();
            Object propertyValue = instanceModel.getPropertyValue();
            if (CollectionUtils.isEmpty(instanceModel.getIds())) {
                return new GeneralResult<>(false, "实例不存在");
            }
            boolean hasAdminRole = checkAdminRole(userId);
            CiModule module = moduleRepository.query(moduleId);
            if (null == module || !module.getCode().equals(moduleCode)) {
                return new GeneralResult<>(false, "模型不存在");
            }

            String instanceTable = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
            String instanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();

            List<Instance> instances = instanceRepository.queryList(instanceTable, instanceModel.getIds());
            if (CollectionUtils.isEmpty(instances)) {
                return new GeneralResult<>(false, "实例不存在");
            }
            Map<Long, Instance> instanceMap = instances.stream()
                    .collect(Collectors.toMap(Instance::getId, Function.identity(), (oldData, newData) -> newData));

            // 先校验当前用户是否拥有修改的权限，存在没权限直接跳过
            List<Long> instanceIds = new ArrayList<>();
            List<Instance> instanceList = new ArrayList<>();
            if (!hasAdminRole) {
                List<InstanceAuthority> instanceAuthoritys = instanceAuthorityRepository.queryByAuth(moduleId,
                        instanceModel.getIds(), userId, InstanceEnum.authority.EDIT.name());
                Map<Long, InstanceAuthority> authorityMap = instanceAuthoritys.stream().collect(Collectors
                        .toMap(InstanceAuthority::getInstanceId, Function.identity(), (oldData, newData) -> newData));

                for (Long id : instanceMap.keySet()) {
                    Instance temp = instanceMap.get(id);
                    if (userId.equals(temp.getCreatorId())) {
                        instanceIds.add(id);
                        instanceList.add(temp);
                        continue;
                    }
                    InstanceAuthority authority = authorityMap.get(id);
                    if (null != authority) {
                        instanceIds.add(id);
                        instanceList.add(temp);
                    }
                }

            } else {
                instanceIds = instanceModel.getIds();
                instanceList = instances;
            }

            if (CollectionUtils.isEmpty(instanceIds)) {
                return new GeneralResult<>(false, "可操作的实例不存在");
            }

            Property property = propertyRepository.queryByCode(moduleId, propertyCode);
            if (null == property) {
                return new GeneralResult<>(false, "模型属性不存在");
            }
            String propertyType = property.getType();
            // 清空数据
            if (InstanceEnum.type.CLEAN_DATA.name().equalsIgnoreCase(instanceModel.getType())) {
                propertyValue = "";
            }
            // 验证修改值是否满足属性约束
            boolean validateResult = validate(property, moduleCode, propertyValue);
            if (!validateResult) {
                return new GeneralResult<>(false, "属性值不符合属性约束");
            }

            List<InstanceValue> values = instanceValueRepository.queryByInstanceIds(instanceValueTable, instanceIds);
            // 实例值记录中查询出实例名称
            List<Property> propertyList = queryModulePropertys(moduleId);
            Map<String, String> propertyMap = new HashMap<>();
            propertyList.forEach(item -> {
                propertyMap.put(item.getCode(), item.getName());
            });

            values.forEach(item -> {
                item.setName(propertyMap.get(item.getPropertyCode()));
            });

            Map<Long, List<InstanceValue>> valueMap =
                    values.stream().collect(Collectors.groupingBy(InstanceValue::getInstanceId));

            Map<Long, InstanceValue> valueNameMap =
                    values.stream().filter(item -> item.getPropertyCode().equals(Common.NAME)).collect(
                            Collectors.toMap(InstanceValue::getInstanceId, Function.identity(), (oldData, newData) -> newData));
            Map<Long, InstanceValue> propertyValueMap =
                    values.stream().filter(item -> item.getPropertyCode().equals(propertyCode)).collect(
                            Collectors.toMap(InstanceValue::getInstanceId, Function.identity(), (oldData, newData) -> newData));

            List<InstanceValue> updateInstanceValueList = new ArrayList<>();
            List<InstanceValue> addInstanceValueList = new ArrayList<>();
            List<Instance> updateInstanceList = new ArrayList<>();

            Map<Long, Map<String, Object>> oldMap = new HashMap<>();
            for (Instance instance : instanceList) {
                InstanceValue nameValue = valueNameMap.get(instance.getId());
                if (null == nameValue) {
                    log.error("实例值不存在", instance);
                    continue;
                }
                InstanceValue instanceValue = propertyValueMap.get(instance.getId());
                String propertyValueStr = String.valueOf(propertyValue);
                if (PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                        || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
                        || PropertyConstant.PropertyType.JSON.equals(propertyType)
                        || PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                        || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {
                    propertyValueStr = JSON.toJSONString(propertyValue);
                }
                instance.setInstanceValues(valueMap.get(instance.getId()));
                Map<String, Object> beforeMap =
                        JSON.parseObject(JSON.toJSONString(instance), new TypeReference<Map<String, Object>>() {
                        });
                oldMap.put(instance.getId(), beforeMap);
                if (null == instanceValue) {
                    // 新增属性
                    instanceValue = new InstanceValue();
                    instanceValue.setInstanceId(instance.getId());
                    instanceValue.setPropertyId(property.getId());
                    instanceValue.setPropertyCode(property.getCode());
                    instanceValue.setPropertyValue(propertyValueStr);
                    instanceValue.setCreatorId(userId);
                    instanceValue.setGmtCreate(new Date());
                    addInstanceValueList.add(instanceValue);
                } else {
                    instanceValue.setPropertyValue(propertyValueStr);
                    instanceValue.setMenderId(userId);
                    instanceValue.setGmtModify(new Date());
                    updateInstanceValueList.add(instanceValue);
                }
                instance.setMenderId(userId);
                instance.setGmtModify(new Date());
                updateInstanceList.add(instance);

            }
            if (CollectionUtils.isNotEmpty(updateInstanceList)) {
                handler.batchUpdateInstance(updateInstanceList, instanceTable);
                //更新实例关系数据
                for (Instance instance : updateInstanceList) {
                    List<RelationCategory> categories = relationCategoryRepository.list(Lists.newArrayList(), Maps.newHashMap());
                    for (RelationCategory category : categories) {
                        String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + category.getCode().toLowerCase();
                        instanceRelationRepository.updateOfnameBySourceId(instance.getName(), relationTable, instance.getId(), userId);
                        instanceRelationRepository.updateOfnameByTargetId(instance.getName(), relationTable, instance.getId(), userId);
                    }
                }
            }
            if (CollectionUtils.isNotEmpty(updateInstanceValueList)) {
                handler.batchUpdateInstanceValue(updateInstanceValueList, instanceValueTable);
            }
            if (CollectionUtils.isNotEmpty(addInstanceValueList)) {
                handler.batchSaveInstanceValue(addInstanceValueList, instanceValueTable);
            }

            // 实例操作记录
            List<InstanceHistory> instanceHistoryList = new ArrayList<>();
            // 审计操作
            List<InstanceValue> newValues = instanceValueRepository.queryByInstanceIds(instanceValueTable, instanceIds);
            newValues.forEach(item -> {
                item.setName(propertyMap.get(item.getPropertyCode()));
            });
            Map<Long, List<InstanceValue>> newValueMap =
                    newValues.stream().collect(Collectors.groupingBy(InstanceValue::getInstanceId));
            List<OperationalAudit> audits = new ArrayList<>();
            for (Instance instance : instanceList) {
                InstanceValue nameValue = valueNameMap.get(instance.getId());
                if (null == nameValue) {
                    log.error("实例值不存在", instance);
                    continue;
                }
                instance.setInstanceValues(newValueMap.get(instance.getId()));
                InstanceHistory history = instanceHistoryHelper.getInstanceHistory(instance, module, nameValue,
                        InstanceEnum.catalog.INSTANCE.name(), InstanceEnum.category.BATCH_MODIFY.name(), userId);
                if (null != history) {
                    instanceHistoryList.add(history);
                }

                Map<String, Object> beforeMap = oldMap.get(instance.getId());
                Map<String, Object> afterMap =
                        JSON.parseObject(JSON.toJSONString(instance), new TypeReference<Map<String, Object>>() {
                        });
                OperationalAudit audit = operationalAuditService.buildLog(OperationalAuditType.BATCH_MODIFY, "实例",
                        nameValue.getPropertyValue(), moduleCode, instance.getId(), userId, beforeMap, afterMap);
                audits.add(audit);
            }
            if (CollectionUtils.isNotEmpty(audits)) {
                operationalAuditRepository.batchSave(audits, 100);
            }

            if (CollectionUtils.isNotEmpty(instanceHistoryList)) {
                instanceHistoryRepository.batchSave(instanceHistoryList, 100);
            }

            return new GeneralResult<>(true, "实例批量编辑成功");
        } catch (Exception e) {
            log.error("instance and value batch modify fail:", e);
            return new GeneralResult<>(false, "实例批量编辑失败");
        }
    }

    @Override
    public GeneralResult modify(JSONObject object, Long userId) {
        // 模型判断及实例属性值校验
        GeneralResult initResult = initInstance(object, InstanceEnum.category.MODIFY.name());
        if (initResult.isFailed()) {
            log.error("实例预校验失败{}", initResult.getMessage());
            return initResult;
        }
        Instance instance = (Instance) initResult.getData();
        String path = Instance.class.getSimpleName() + "_" + instance.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            String table = CmdbUtils.CMDB_INSTANCE + instance.getModuleCode().toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + instance.getModuleCode().toLowerCase();
            Instance temp = instanceRepository.query(table, instance.getId());
            Instance oldInstance = new Instance();
            BeanUtils.copyProperties(temp, oldInstance);
            if (null == temp) {
                return new GeneralResult<>(false, "实例不存在");
            }
            temp.setName(object.getString("name"));
            boolean hasAdminRole = checkAdminRole(userId);
            if (!hasAdminRole && !temp.getCreatorId().equals(userId)) {
                InstanceAuthority authority = instanceAuthorityRepository.queryByAuth(instance.getModuleId(),
                        instance.getId(), userId, InstanceEnum.authority.EDIT.name());
                if (null == authority) {
                    return new GeneralResult<>(false, "实例无编辑权限");
                }
            }
            InstanceValue nameValue = instanceValueRepository.query(valueTable, Common.NAME, instance.getId());
            if (null == nameValue) {
                return new GeneralResult<>(false, "实例不存在");
            }
            // 实例值记录中查询出实例名称
            List<Property> propertyList = queryModulePropertys(instance.getModuleId());
            Map<String, String> propertyMap = new HashMap<>();
            propertyList.forEach(item -> {
                propertyMap.put(item.getCode(), item.getName());
            });

            List<InstanceValue> oldValues = instanceValueRepository.listByInstanceId(instance.getId(), valueTable);
            oldValues.forEach(item -> {
                item.setName(propertyMap.get(item.getPropertyCode()));
            });
            temp.setInstanceValues(oldValues);
            Map<String, Object> beforeMap =
                    JSON.parseObject(JSON.toJSONString(temp), new TypeReference<Map<String, Object>>() {
                    });
            temp.setGmtModify(new Date());
            temp.setMenderId(userId);
            handler.updateInstance(temp, table);
            //更新实例关系数据
            List<RelationCategory> categories = relationCategoryRepository.list(Lists.newArrayList(), Maps.newHashMap());
            for (RelationCategory category : categories) {
                String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + category.getCode().toLowerCase();
                instanceRelationRepository.updateOfnameBySourceId(temp.getName(), relationTable, temp.getId(), userId);
                instanceRelationRepository.updateOfnameByTargetId(temp.getName(), relationTable, temp.getId(), userId);
            }
            List<InstanceValue> instanceValues = instance.getInstanceValues();
            // 不存在即新增
            if (CollectionUtils.isEmpty(oldValues)) {
                if (CollectionUtils.isNotEmpty(instanceValues)) {
                    for (InstanceValue instanceValue : instanceValues) {
                        instanceValue.setDeleted(false);
                        instanceValue.setCreatorId(userId);
                        instanceValue.setGmtCreate(new Date());
                        instanceValue.setInstanceId(instance.getId());
                        Property property = instanceValue.getProperty();
                        if (null == property) {
                            continue;
                        }
                        // 初始化时只保留了前缀
                        String defaultValueType = property.getDefaultValueType();
                        if (StringUtils.isNotEmpty(defaultValueType) && !"null".equalsIgnoreCase(defaultValueType)) {
                            String propertyValue = instanceValue.getPropertyValue();
                            Integer startNum = property.getStartNum();
                            if (null == startNum) {
                                startNum = 0;
                            }
                            // 计算后面部分的数值
                            Long endNum = instance.getId() + startNum.longValue();
                            if (PropertyConstant.DefaultValueType.SERIALNUM.equalsIgnoreCase(defaultValueType)) {
                                String value = propertyValue + endNum;
                                // defaultValue是设置的流水号长度
                                String defaultValue = property.getDefaultValue();
                                if (StringUtils.isEmpty(defaultValue) || "null".equalsIgnoreCase(defaultValue)) {
                                    defaultValue = "1";
                                }
                                // 如果当前属性值大于流水号长度则删除已经生成的实例
                                if (value.length() > Integer.valueOf(defaultValue)) {
                                    handler.updateInstance(oldInstance, table);
                                    return new GeneralResult<>(false, "实例更新失败:属性" + property.getName() + "内容超过流水号长度");
                                }
                                String valueMiddle = "";
                                Integer remainderLength;
                                remainderLength = Integer.valueOf(defaultValue) - value.length();
                                if (remainderLength > 0) {
                                    for (int i = 0; i < remainderLength; i++) {
                                        valueMiddle += "0";
                                    }
                                }
                                propertyValue = propertyValue + valueMiddle + endNum;
                            } else if (PropertyConstant.DefaultValueType.AUTOINCRID
                                    .equalsIgnoreCase(defaultValueType)) {
                                propertyValue = propertyValue + endNum;
                            } else if (PropertyConstant.DefaultValueType.INNERFUN.equalsIgnoreCase(defaultValueType)) {
                                propertyValue = IDFactory.instance().uuid();
                            }
                            instanceValue.setPropertyValue(propertyValue);
                        }
                    }
                    handler.batchSaveInstanceValue(instanceValues, valueTable);
                }
            } else {
                if (CollectionUtils.isNotEmpty(instanceValues)) {
                    List<InstanceValue> addList = new ArrayList<>();
                    List<InstanceValue> updateList = new ArrayList<>();
                    Map<Long, List<InstanceValue>> instanceValueMap =
                            oldValues.stream().collect(Collectors.groupingBy(InstanceValue::getId));
                    for (InstanceValue instanceValue : instanceValues) {
                        if (null == instanceValue.getId() || 0L == instanceValue.getId()) {
                            instanceValue.setDeleted(false);
                            instanceValue.setCreatorId(userId);
                            instanceValue.setGmtCreate(new Date());
                            instanceValue.setInstanceId(instance.getId());
                            // 判断类型
                            Property property = instanceValue.getProperty();
                            if (null != property) {
                                // 初始化时只保留了前缀
                                String defaultValueType = property.getDefaultValueType();
                                if (StringUtils.isNotEmpty(defaultValueType)
                                        && !"null".equalsIgnoreCase(defaultValueType)) {
                                    String propertyValue = instanceValue.getPropertyValue();
                                    Integer startNum = property.getStartNum();
                                    if (null == startNum) {
                                        startNum = 0;
                                    }
                                    // 计算后面部分的数值
                                    Long endNum = instance.getId() + startNum.longValue();
                                    if (PropertyConstant.DefaultValueType.SERIALNUM
                                            .equalsIgnoreCase(defaultValueType)) {
                                        String value = propertyValue + endNum;
                                        // defaultValue是设置的流水号长度
                                        String defaultValue = property.getDefaultValue();
                                        if (StringUtils.isEmpty(defaultValue)
                                                || "null".equalsIgnoreCase(defaultValue)) {
                                            defaultValue = "1";
                                        }
                                        // 如果当前属性值大于流水号长度则删除已经生成的实例
                                        if (value.length() > Integer.valueOf(defaultValue)) {
                                            handler.updateInstance(oldInstance, table);
                                            return new GeneralResult<>(false,
                                                    "实例更新失败:属性" + property.getName() + "内容超过流水号长度");
                                        }
                                        String valueMiddle = "";
                                        Integer remainderLength;
                                        remainderLength = Integer.valueOf(defaultValue) - value.length();
                                        if (remainderLength > 0) {
                                            for (int i = 0; i < remainderLength; i++) {
                                                valueMiddle += "0";
                                            }
                                        }
                                        propertyValue = propertyValue + valueMiddle + endNum;
                                    } else if (PropertyConstant.DefaultValueType.AUTOINCRID
                                            .equalsIgnoreCase(defaultValueType)) {
                                        propertyValue = propertyValue + endNum;
                                    } else if (PropertyConstant.DefaultValueType.INNERFUN
                                            .equalsIgnoreCase(defaultValueType)) {
                                        propertyValue = IDFactory.instance().uuid();
                                    }
                                    instanceValue.setPropertyValue(propertyValue);
                                }
                            }
                            addList.add(instanceValue);
                        } else {
                            instanceValue.setMenderId(userId);
                            instanceValue.setGmtModify(new Date());
                            updateList.add(instanceValue);
                            instanceValueMap.remove(instanceValue.getId());
                        }
                    }
                    if (CollectionUtils.isNotEmpty(addList)) {
                        handler.batchSaveInstanceValue(addList, valueTable);
                    }
                    if (CollectionUtils.isNotEmpty(updateList)) {
                        handler.batchUpdateInstanceValue(updateList, valueTable);
                    }
                    instanceValueMap.clear();
                }
            }
            instanceHistoryHelper.history(instance, InstanceEnum.catalog.INSTANCE.name(),
                    InstanceEnum.category.MODIFY.name(), userId);
            // 操作审计
            instanceValues = instanceValueRepository.listByInstanceId(instance.getId(), valueTable);
            instanceValues.forEach(item -> {
                item.setName(propertyMap.get(item.getPropertyCode()));
            });
            temp.setInstanceValues(instanceValues);
            Map<String, Object> afterMap =
                    JSON.parseObject(JSON.toJSONString(temp), new TypeReference<Map<String, Object>>() {
                    });
            OperationalAudit audit = operationalAuditService.buildLog(OperationalAuditType.MODIFY, "实例",
                    nameValue.getPropertyValue(), instance.getModuleCode(), instance.getId(), userId, beforeMap, afterMap);
            operationalAuditRepository.save(audit);
            return new GeneralResult<>(true, temp, "实例修改成功");
        } catch (Exception e) {
            log.error("instance and value modify fail:", e);
            return new GeneralResult<>(false, "实例修改失败");
        }
    }

    @Override
    public GeneralResult<Instance> detail(Long id, String moduleCode, String positionKey) {
        try {
            String table = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            Instance instance = instanceRepository.query(table, id, positionKey);
            List<InstanceValue> values = instanceValueRepository.listByInstanceId(id, valueTable, positionKey);
            // instance.setInstanceObject(CmdbUtils.instanceValueRowToCol(instanceValues));
            // convertInstanceValue(instance.getInstanceObject(), instance.getModuleId());
            instance.setInstanceValues(values);
            return new GeneralResult<>(true, instance, "实例详情查询成功");
        } catch (Exception e) {
            log.error("instance and value detail fail:", e);
            return new GeneralResult<>(false, "实例详情查询失败");
        }
    }

    @Override
    public GeneralResult export(Long moduleId, String moduleCode, String type, Long userId, String positionKey) {
        Workbook workbook = null;
        try {
            boolean hasAdminRole = checkAdminRole(userId);
            CiModule module = moduleRepository.query(moduleId, positionKey);
            if (null == module) {
                return new GeneralResult<>(false, "模型数据不存在");
            }
            if (!moduleCode.equals(module.getCode())) {
                return new GeneralResult<>(false, "模型数据不存在");
            }
            String table = CmdbUtils.CMDB_INSTANCE + module.getCode().toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + module.getCode().toLowerCase();
            List<Map<String, Object>> instances =
                    instanceRepository.listAll(table, valueTable, module.getId(), null, userId, hasAdminRole, positionKey);
            List<Property> properties = queryModulePropertys(module.getId());
            String fileName = module.getName() + "-实例.xls";
            if ("2007".equals(type)) {
                fileName = fileName + "x";
            }
            workbook = new ExcelFactory(fileName).newWorkbook();
            // 构建表头
            excelHelper.generateSheet(workbook, module, properties);
            // 构建数据
            excelHelper.generateDate(workbook, properties, instances);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            workbook.write(baos);
            byte[] ba = baos.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName);
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new GeneralResult<>(true, resultMap, "导出成功");
        } catch (Exception e) {
            log.error("instance data export fail:", e);
            return new GeneralResult<>(false, "导出失败");
        }
    }

    /***
     * 填充默认值、实例预校验（检验实例的属性值是否满足规则，包括是否必填，最大值、最小值、规则表达式、唯一性等 如果必填，需要补充默认值）
     *
     * @param object
     * @param type
     * @return
     */
    @Override
    public GeneralResult initInstance(JSONObject object, String type) {
        try {
            StringBuilder message = new StringBuilder();
            // 将数据循环转换
            Map<String, Object> objectMap = new HashMap<>();
            for (Map.Entry<String, Object> entry : object.entrySet()) {
                objectMap.put(entry.getKey(), entry.getValue());
            }
            // 校验模型是否存在
            Long moduleId = object.getLong("moduleId");
            String moduleCode = object.getString("moduleCode");
            CiModule module = moduleRepository.query(moduleId);
            if (null == module
                    || (StringUtils.isNotEmpty(moduleCode) && !moduleCode.equalsIgnoreCase(module.getCode()))) {
                log.error("预检失败，模型{}不存在", moduleCode);
                message.append(moduleCode + "模型不存在\n");
                return new GeneralResult<>(false, "模型不存在");
            }
            // 处理数据
            Instance instance = new Instance();
            instance.setModuleId(moduleId);
            instance.setModuleCode(moduleCode);
            // 当页面传递参数为空时，填充默认值
            String uuid = IDFactory.instance().uuid();
            if (InstanceEnum.category.CREATE.name().equalsIgnoreCase(type)) {
                instance.setUuid(uuid);
                instance.setName(object.getString("name"));
            }
            // 校验模型属性是否存在
            List<Property> properties = queryModulePropertys(module.getId());
            if (CollectionUtils.isEmpty(properties)) {
                log.error("预检失败，模型属性不存在");
                message.append(moduleCode + "模型属性不存在\n");
                return new GeneralResult<>(false, "模型属性不存在");
            }
            Map<String, Property> propertyMap = new HashMap<>();
            for (Property property : properties) {
                propertyMap.put(property.getCode(), property);
            }
            List<InstanceValue> instanceValues = new ArrayList<>();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            // 数据转换
            if (InstanceEnum.category.MODIFY.name().equalsIgnoreCase(type)) {
                instance.setId(object.getLong(Common.ID));
                List<InstanceValue> oldValues = instanceValueRepository.listByInstanceId(instance.getId(), valueTable);
                Iterator<InstanceValue> iterator = oldValues.iterator();
                while (iterator.hasNext()) {
                    InstanceValue oldInstanceValue = iterator.next();
                    Property property = propertyMap.get(oldInstanceValue.getPropertyCode());
                    if (null == property) {
                        continue;
                    }
                    if (!oldInstanceValue.getPropertyId().equals(property.getId())) {
                        continue;
                    }
                    String propertyType = property.getType();
                    Boolean propertyOnlyReader = property.getIsOnlyReader();
                    // 存在即替换值
                    if (objectMap.containsKey(oldInstanceValue.getPropertyCode())) {
                        InstanceValue instanceValue = oldInstanceValue;
                        // 只读不做编辑
                        if ((null == propertyOnlyReader || !propertyOnlyReader)) {
                            if (PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                                    || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
                                    || PropertyConstant.PropertyType.JSON.equals(propertyType)
                                    || PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                                    || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {
                                instanceValue.setPropertyValue(
                                        JSON.toJSONString(objectMap.get(oldInstanceValue.getPropertyCode())));
                            } else {
                                if (null != objectMap.get(oldInstanceValue.getPropertyCode())) {
                                    instanceValue.setPropertyValue(
                                            String.valueOf(objectMap.get(oldInstanceValue.getPropertyCode())));
                                }
                            }
                        }
                        instanceValues.add(instanceValue);
                        objectMap.remove(oldInstanceValue.getPropertyCode());
                    }
                }
                // 剩余的为新增属性及数据
                if (objectMap.size() > 0) {
                    for (Map.Entry<String, Object> entry : objectMap.entrySet()) {
                        InstanceValue instanceValue = new InstanceValue();
                        Property property = propertyMap.get(entry.getKey());
                        // 剔除不存在的属性
                        if (null == property) {
                            continue;
                        }
                        String propertyType = property.getType();
                        instanceValue.setPropertyId(property.getId());
                        instanceValue.setPropertyCode(entry.getKey());
                        if (PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                                || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
                                || PropertyConstant.PropertyType.JSON.equals(propertyType)
                                || PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                                || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {
                            instanceValue.setPropertyValue(JSON.toJSONString(entry.getValue()));
                        } else {
                            if (null != entry.getValue()) {
                                instanceValue.setPropertyValue(String.valueOf(entry.getValue()));
                            }
                        }
                        instanceValues.add(instanceValue);
                    }
                }
            } else if (InstanceEnum.category.CREATE.name().equalsIgnoreCase(type)) {
                for (Map.Entry<String, Object> entry : objectMap.entrySet()) {
                    InstanceValue instanceValue = new InstanceValue();
                    Property property = propertyMap.get(entry.getKey());
                    // 剔除不存在的属性
                    if (null == property) {
                        continue;
                    }
                    String propertyType = property.getType();
                    instanceValue.setPropertyId(property.getId());
                    instanceValue.setPropertyCode(entry.getKey());
                    if (PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                            || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
                            || PropertyConstant.PropertyType.JSON.equals(propertyType)
                            || PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                            || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {
                        instanceValue.setPropertyValue(JSON.toJSONString(entry.getValue()));
                    } else {
                        if (null != entry.getValue()) {
                            instanceValue.setPropertyValue(String.valueOf(entry.getValue()));
                        }
                    }
                    instanceValues.add(instanceValue);
                }
            }
            instance.setInstanceValues(instanceValues);
            // 检查数据值
            Iterator<InstanceValue> iterator = instanceValues.iterator();
            // 检验实例的属性值是否满足规则，包括是否必填，最大值、最小值、规则表达式、唯一性等
            // 如果必填，需要补充默认值
            while (iterator.hasNext()) {
                InstanceValue instanceValue = iterator.next();
                // 当前实例属性值
                Long propertyId = instanceValue.getPropertyId();
                String propertyValue = instanceValue.getPropertyValue();
                String propertyCode = instanceValue.getPropertyCode();
                Property property = propertyMap.get(propertyCode);
                if (null == property) {
                    iterator.remove();
                    continue;
                }
                String propertyName = property.getName();
                // 是否补充默认值
                if (StringUtils.isEmpty(propertyValue) && StringUtils.isNotEmpty(property.getDefaultValue())) {
                    instanceValue.setPropertyValue(property.getDefaultValue());
                }
                String propertyType = property.getType();

                // 单行文本中流水号、自增、内置函数不校验数据规则，值代码中自动生成
                String defaultValueType = property.getDefaultValueType();
                if (PropertyConstant.PropertyType.TEXTFIELD.equals(propertyType)) {
                    if (PropertyConstant.DefaultValueType.SERIALNUM.equalsIgnoreCase(defaultValueType)
                            || PropertyConstant.DefaultValueType.AUTOINCRID.equalsIgnoreCase(defaultValueType)
                            || PropertyConstant.DefaultValueType.INNERFUN.equalsIgnoreCase(defaultValueType)) {
                        propertyMap.remove(propertyCode);
                        // 存储起来供保存时使用
                        instanceValue.setProperty(property);
                        continue;
                    }
                }
                if (!"null".equalsIgnoreCase(propertyValue) && StringUtils.isNotEmpty(propertyValue)) {
                    // 验证属性项
                    if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                            || PropertyConstant.PropertyType.RADIO.equals(propertyType)) {
                        validatePropertyItem(instance, instanceValue, property, message);
                    } else if (PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                            || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)) {
                        validateArray(instance, instanceValue, property, message);
                    } else if (PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                            || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {
                        // 多行表格
                        validateProperty(instance, instanceValue, property, message);
                    } else if ((PropertyConstant.PropertyType.INTEGER.equals(propertyType)
                            || PropertyConstant.PropertyType.FLOAT.equals(propertyType))) {
                        // 验证数值
                        validateNumber(instance, instanceValue, property, message);
                    } else if (PropertyConstant.PropertyType.BOOLEAN.equals(propertyType)
                            && !CmdbUtils.isBool(propertyValue)) {
                        message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]非布尔类型\n");
                    } else if (PropertyConstant.PropertyType.JSON.equals(propertyType)) {
                        GeneralResult<JSONObject> result = JSONTools.isJSON(propertyValue);
                        if (result.isFailed()) {
                            JSONArray array = JSONTools.parseStr2JsonArray(propertyValue);
                            if (null == array) {
                                message
                                        .append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]非JSON\n");
                            }
                        }

                    } else if (PropertyConstant.PropertyType.DATE.equals(propertyType)
                            && !CmdbUtils.isDate(propertyValue)) {
                        message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]非日期类型\n");
                    } else if (PropertyConstant.PropertyType.TIME.equals(propertyType)
                            && !CmdbUtils.isTime(propertyValue)) {
                        message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]非时间类型\n");
                    } else if (PropertyConstant.PropertyType.TEXTENCRYPT.equals(propertyType)) {
                        // 加密的校验
                        if (StringUtils.isNotEmpty(propertyValue)) {
                            // 密码解密
                            Encryptor encryptor = new AESEncryptor();
                            String password = encryptor.decrypt(propertyValue, null);
                            if (null != property.getLength() && property.getLength() > 0
                                    && StringUtils.isNotEmpty(password) && password.length() > property.getLength()) {
                                message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]超过限制长度["
                                        + property.getLength() + "]\n");
                            }
                        }
                    }

                    // 规则表达式
                    if (null != property.getRegexpId()
                            && !PropertyConstant.PropertyType.TEXTENCRYPT.equals(propertyType)) {
                        RegularExpression expression = regularExpressionRepository.query(property.getRegexpId());
                        if (null != expression && StringUtils.isNotEmpty(expression.getExpression())) {
                            boolean flag =
                                    CmdbUtils.contains(expression.getExpression(), String.valueOf(propertyValue));
                            if (!flag) {
                                message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue
                                        + "]不满足属性表达式[" + expression.getExpression() + "]\n");
                            }
                        }
                    }
                }

                // 是否必填
                if (property.getIsRequired()
                        && (StringUtils.isEmpty(propertyValue) || "null".equalsIgnoreCase(propertyValue))) {
                    message.append(moduleCode + "实例属性[" + propertyName + "]是必填项\n");
                }
                // 是否超过长度限制
                if (!PropertyConstant.PropertyType.TEXTENCRYPT.equals(propertyType) && null != property.getLength()
                        && property.getLength() > 0 && StringUtils.isNotEmpty(propertyValue)
                        && propertyValue.length() > property.getLength()) {
                    message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]超过限制长度["
                            + property.getLength() + "]\n");
                }
                // 唯一性
                if (property.getIsUniqued() && StringUtils.isNotEmpty(propertyValue)) {
                    InstanceValue checkValue = instanceValueRepository.query(valueTable, propertyCode, propertyValue);
                    if (InstanceEnum.category.CREATE.name().equalsIgnoreCase(type)) {
                        if (null != checkValue) {
                            message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]已存在\n");
                        }
                    } else if (InstanceEnum.category.MODIFY.name().equalsIgnoreCase(type)) {
                        if (null != checkValue && !(checkValue.getInstanceId() + "-" + checkValue.getPropertyCode())
                                .equals(instance.getId() + "-" + instanceValue.getPropertyCode())) {
                            message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]已存在\n");
                        }
                    }
                }
                propertyMap.remove(propertyCode);
            }
            // 是否缺少属性
            if (!propertyMap.isEmpty()) {
                // 处理属性默认类型，包括内置函数、自增ID、流水号等
                Iterator<Map.Entry<String, Property>> it = propertyMap.entrySet().iterator();
                while (it.hasNext()) {
                    Map.Entry<String, Property> entry = it.next();
                    // 设置UUID
                    if (Common.UUID.equals(entry.getKey())
                            && InstanceEnum.category.CREATE.name().equalsIgnoreCase(type)) {
                        InstanceValue instanceValue = new InstanceValue();
                        instanceValue.setPropertyId(entry.getValue().getId());
                        instanceValue.setPropertyCode(entry.getValue().getCode());
                        instanceValue.setPropertyValue(uuid);
                        instanceValues.add(instanceValue);
                        it.remove();
                    } else if (Common.UUID.equals(entry.getKey())
                            && InstanceEnum.category.MODIFY.name().equalsIgnoreCase(type)) {
                        // 编辑情况下，UUID不处理
                        it.remove();
                    } else if (!Common.NAME.equals(entry.getKey()) && (PropertyConstant.DefaultValueType.FIXED
                            .equalsIgnoreCase(entry.getValue().getDefaultValueType())
                            || PropertyConstant.DefaultValueType.INNERFUN
                            .equalsIgnoreCase(entry.getValue().getDefaultValueType())
                            || PropertyConstant.DefaultValueType.AUTOINCRID
                            .equalsIgnoreCase(entry.getValue().getDefaultValueType())
                            || PropertyConstant.DefaultValueType.SERIALNUM
                            .equalsIgnoreCase(entry.getValue().getDefaultValueType()))) {
                        // 存在默认类型的需要特殊处理，自增ID、流水号
                        InstanceValue instanceValue = new InstanceValue();
                        instanceValue.setPropertyId(entry.getValue().getId());
                        instanceValue.setPropertyCode(entry.getValue().getCode());
                        setDefaultValue(entry.getValue(), instanceValue);
                        instanceValues.add(instanceValue);
                        it.remove();
                    }
                }
                for (Property property : propertyMap.values()) {
                    if (property.getIsRequired()) {
                        message.append(moduleCode + "对应模型属性[" + property.getName() + "]为必填属性\n");
                    }
                }
            }
            if (StringUtils.isNotEmpty(message.toString())) {
                return new GeneralResult<>(false, message.toString());
            }
            return new GeneralResult<>(true, instance, "实例预检成功");
        } catch (Exception e) {
            log.error("CmdbModuleInstance preCheck fail", e);
            return new GeneralResult<>(false, "实例预检失败:" + e.getMessage());
        }
    }

    /***
     * 设置默认值,只处理新增情况
     *
     * @param property
     * @param instanceValue
     */
    private void setDefaultValue(Property property, InstanceValue instanceValue) {
        try {
            String propertyValue = instanceValue.getPropertyValue();
            String defaultValueType = property.getDefaultValueType();
            String defaultValue = property.getDefaultValue();
            String prefixIdentifier = property.getPrefixIdentifier();
            if (StringUtils.isNotEmpty(defaultValueType) && !"null".equalsIgnoreCase(defaultValueType)) {
                if (StringUtils.isEmpty(propertyValue)) {
                    if (PropertyConstant.DefaultValueType.FIXED.equalsIgnoreCase(defaultValueType)) {
                        propertyValue = defaultValue;
                    } else if (PropertyConstant.DefaultValueType.INNERFUN.equalsIgnoreCase(defaultValueType)) {
                        // 存储起来供保存时使用
                        instanceValue.setProperty(property);
                        propertyValue = IDFactory.instance().uuid();
                    } else if (PropertyConstant.DefaultValueType.SERIALNUM.equalsIgnoreCase(defaultValueType)
                            || PropertyConstant.DefaultValueType.AUTOINCRID.equalsIgnoreCase(defaultValueType)) {
                        propertyValue = StringUtils.isNotEmpty(prefixIdentifier) ? prefixIdentifier : "";
                        // 存储起来供保存时使用
                        instanceValue.setProperty(property);
                    }
                } else {
                    propertyValue = defaultValue;
                }
                instanceValue.setPropertyValue(propertyValue);
            }
        } catch (Exception e) {
            log.error("设置默认值失败", e);
        }
    }

    /***
     * 验证存在子项
     *
     * @param instance
     * @param instanceValue
     * @param property
     * @param message
     */
    private void validatePropertyItem(Instance instance, InstanceValue instanceValue, Property property,
                                      StringBuilder message) {
        String moduleCode = instance.getModuleCode();
        String propertyValue = instanceValue.getPropertyValue();
        String propertyName = property.getName();
        try {
            // 判断属性值是否存在
            List<PropertyItem> propertyItems = property.getItemList();
            if (CollectionUtils.isEmpty(propertyItems)) {
                message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "] 的选项不存在\n");
                return;
            }
            Map<String, PropertyItem> propertyItemMap = new HashMap<>();
            for (PropertyItem propertyItem : propertyItems) {
                propertyItemMap.put(propertyItem.getValue(), propertyItem);
            }
            if (!propertyItemMap.containsKey(propertyValue)) {
                message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "] 的选项不存在\n");
            }
        } catch (Exception e) {
            log.error("验证子项失败", e);
            message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "] 的选项验证失败\n");
        }
    }

    /***
     * 验证复选框及复选下拉框
     *
     * @param instance
     * @param instanceValue
     * @param property
     * @param message
     */
    private void validateArray(Instance instance, InstanceValue instanceValue, Property property,
                               StringBuilder message) {
        String moduleCode = instance.getModuleCode();
        String propertyValue = instanceValue.getPropertyValue();
        String propertyName = property.getName();
        try {
            // 判断属性值是否存在
            List<PropertyItem> propertyItems = property.getItemList();
            if (CollectionUtils.isEmpty(propertyItems)) {
                message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "] 的选项不存在\n");
                return;
            }
            Map<String, PropertyItem> propertyItemMap = new HashMap<>();
            for (PropertyItem propertyItem : propertyItems) {
                propertyItemMap.put(propertyItem.getValue(), propertyItem);
            }
            List<String> values = JSONObject.parseArray(propertyValue, String.class);
            for (String value : values) {
                if (!propertyItemMap.containsKey(value)) {
                    message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "] 的选项不存在\n");
                    break;
                }
            }
        } catch (Exception e) {
            log.error("验证子项失败", e);
            message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "] 的选项验证失败\n");
        }
    }

    /***
     * 验证属性 表格
     *
     * @param instance
     * @param instanceValue
     * @param property
     * @param message
     */
    private void validateProperty(Instance instance, InstanceValue instanceValue, Property property,
                                  StringBuilder message) {
        String moduleCode = instance.getModuleCode();
        String tablePropertyValue = instanceValue.getPropertyValue();
        String propertyName = property.getName();
        try {
            List<PropertyItem> propertyItems = property.getItemList();
            if (CollectionUtils.isEmpty(propertyItems)) {
                message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + tablePropertyValue + "] 的结构项不存在\n");
                return;
            }

        } catch (Exception e) {
            log.error("验证子项失败", e);
            message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + tablePropertyValue + "] 的选项验证失败\n");
        }
    }

    /***
     * 验证属性值是否满足属性约束
     *
     * @param property
     * @param moduleCode
     * @param propertyValue
     * @return
     */
    private boolean validate(Property property, String moduleCode, Object propertyValue) {
        try {
            boolean result = true;
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            String propertyType = property.getType();
            if (null != propertyValue && !"null".equalsIgnoreCase(String.valueOf(propertyValue))) {
                if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                        || PropertyConstant.PropertyType.RADIO.equals(propertyType)) {
                    List<PropertyItem> propertyItems = propertyItemRepository.queryByPropertyId(property.getId());
                    Map<String, PropertyItem> propertyItemMap = new HashMap<>();
                    for (PropertyItem propertyItem : propertyItems) {
                        propertyItemMap.put(propertyItem.getValue(), propertyItem);
                    }
                    if (!propertyItemMap.containsKey(propertyValue)) {
                        result = false;
                    }
                } else if (PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)
                        || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)) {
                    List<PropertyItem> propertyItems = propertyItemRepository.queryByPropertyId(property.getId());
                    Map<String, PropertyItem> propertyItemMap = new HashMap<>();
                    for (PropertyItem propertyItem : propertyItems) {
                        propertyItemMap.put(propertyItem.getValue(), propertyItem);
                    }
                    List<String> values = JSONObject.parseArray(JSON.toJSONString(propertyValue), String.class);
                    for (String value : values) {
                        if (!propertyItemMap.containsKey(value)) {
                            result = false;
                            break;
                        }
                    }
                } else if (PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                        || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {// 多行表格
                    String propertyIds = property.getPropertyIds();
                    if (StringUtils.isEmpty(propertyIds)) {
                        List<PropertyItem> propertyItems =
                                propertyItemRepository.queryByPropertyId(Long.valueOf(property.getId()));
                        if (CollectionUtils.isEmpty(propertyItems)) {
                            result = false;
                        }
                    } else {
                        String[] propertyIdsplit = propertyIds.split(",");
                        for (String propertyId : propertyIdsplit) {
                            Property cmdbProperty = propertyRepository.query(Long.valueOf(propertyId));
                            if (null == cmdbProperty) {
                                result = false;
                                break;
                            }
                        }
                    }
                } else if (PropertyConstant.PropertyType.INTEGER.equals(propertyType)) { // 验证数值
                    if (CmdbUtils.isNumeric(String.valueOf(propertyValue))) {
                        // 最大值,判断数值
                        if (null != property.getMaxValue()
                                && property.getMaxValue() < Integer.valueOf(String.valueOf(propertyValue))) {
                            result = false;
                        }
                        // 最小值,判断数值
                        if (null != property.getMinValue()
                                && property.getMinValue() > Integer.valueOf(String.valueOf(propertyValue))) {
                            result = false;
                        }
                    } else {
                        result = false;
                    }
                } else if (PropertyConstant.PropertyType.FLOAT.equals(propertyType)) {
                    if (CmdbUtils.isNumeric(String.valueOf(propertyValue))) {
                        if (String.valueOf(propertyValue).contains(".")) {
                            String num = String.valueOf(propertyValue).split("\\.")[1];
                            if (num.length() > property.getDecimalNum()) {
                                result = false;
                            }
                        } else {
                            result = false;
                        }
                        // 最大值,判断数值
                        if (null != property.getMaxValue()
                                && property.getMaxValue().floatValue() < Float.valueOf(String.valueOf(propertyValue))) {
                            result = false;
                        }
                        // 最小值,判断数值
                        if (null != property.getMinValue()
                                && property.getMinValue().floatValue() > Float.valueOf(String.valueOf(propertyValue))) {
                            result = false;
                        }
                    } else {
                        result = false;
                    }
                } else if (PropertyConstant.PropertyType.BOOLEAN.equals(propertyType)
                        && !CmdbUtils.isBool(String.valueOf(propertyValue))) {
                    result = false;
                } else if (PropertyConstant.PropertyType.JSON.equals(propertyType)
                        && !CmdbUtils.isJSON(JSON.toJSONString(propertyValue))) {
                    result = false;
                } else if (PropertyConstant.PropertyType.DATE.equals(propertyType)
                        && !CmdbUtils.isDate(String.valueOf(propertyValue))) {
                    result = false;
                } else if (PropertyConstant.PropertyType.TIME.equals(propertyType)
                        && !CmdbUtils.isTime(String.valueOf(propertyValue))) {
                    result = false;
                }

                // 规则表达式
                if (null != property.getRegexpId() && !PropertyConstant.PropertyType.TEXTENCRYPT.equals(propertyType)) {
                    RegularExpression expression = regularExpressionRepository.query(property.getRegexpId());
                    if (null != expression && StringUtils.isNotEmpty(expression.getExpression())) {
                        boolean flag = CmdbUtils.contains(expression.getExpression(), String.valueOf(propertyValue));
                        if (!flag) {
                            result = false;
                        }
                    }
                }
            }

            // 是否必填
            if (property.getIsRequired() && null != propertyValue
                    && StringUtils.isEmpty(String.valueOf(propertyValue))) {
                result = false;
            }
            // 是否超过长度限制
            if (PropertyConstant.PropertyType.TEXTENCRYPT.equals(propertyType)) {
                // 加密的校验
                if (null != propertyValue && !"null".equalsIgnoreCase(String.valueOf(propertyValue))) {
                    // 密码解密
                    Encryptor encryptor = new AESEncryptor();
                    String password = encryptor.decrypt(String.valueOf(propertyValue), null);
                    if (null != property.getLength() && property.getLength() > 0 && StringUtils.isNotEmpty(password)
                            && password.length() > property.getLength()) {
                        result = false;
                    }
                }
            } else {
                if (null != property.getLength() && property.getLength() > 0 && null != propertyValue
                        && StringUtils.isNotEmpty(String.valueOf(propertyValue))
                        && String.valueOf(propertyValue).length() > property.getLength()) {
                    result = false;
                }
            }
            // 唯一性
            if (property.getIsUniqued() && null != propertyValue
                    && StringUtils.isNotEmpty(String.valueOf(propertyValue))) {
                InstanceValue checkValue =
                        instanceValueRepository.query(valueTable, property.getCode(), String.valueOf(propertyValue));
                if (null != checkValue) {
                    result = false;
                }
            }
            return result;
        } catch (Exception e) {
            log.error("验证失败", e);
            return false;
        }

    }

    /***
     * 验证数值
     *
     * @param instance
     * @param instanceValue
     * @param property
     * @param message
     */
    private void validateNumber(Instance instance, InstanceValue instanceValue, Property property,
                                StringBuilder message) {
        String moduleCode = instance.getModuleCode();
        String propertyValue = instanceValue.getPropertyValue();
        String propertyName = property.getName();
        String propertyType = property.getType();
        try {
            if (CmdbUtils.isNumeric(propertyValue)) {
                if (PropertyConstant.PropertyType.FLOAT.equals(propertyType)) {
                    if (propertyValue.contains(".")) {
                        String num = propertyValue.split("\\.")[1];
                        if (num.length() > property.getDecimalNum()) {
                            message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]保留小数大于["
                                    + property.getDecimalNum() + "]\n");
                        }
                    }
                    // 最大值,判断数值
                    if (null != property.getMaxValue()
                            && property.getMaxValue().floatValue() < Float.valueOf(propertyValue)) {
                        message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]大于属性最大值限制["
                                + property.getMaxValue() + "]\n");
                    }
                    // 最小值,判断数值
                    if (null != property.getMinValue()
                            && property.getMinValue().floatValue() > Float.valueOf(propertyValue)) {
                        message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]小于属性最小值限制["
                                + property.getMinValue() + "]\n");
                    }
                } else if (PropertyConstant.PropertyType.INTEGER.equals(propertyType)) {
                    // 最大值,判断数值
                    if (null != property.getMaxValue()
                            && property.getMaxValue().floatValue() < Integer.valueOf(propertyValue)) {
                        message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]大于属性最大值限制["
                                + property.getMaxValue() + "]\n");
                    }
                    // 最小值,判断数值
                    if (null != property.getMinValue()
                            && property.getMinValue().floatValue() > Integer.valueOf(propertyValue)) {
                        message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]小于属性最小值限制["
                                + property.getMinValue() + "]\n");
                    }
                }
            } else {
                message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "]非数值类型\n");

            }
        } catch (Exception e) {
            log.error("验证数值失败", e);
            message.append(moduleCode + "实例属性[" + propertyName + "]的值[" + propertyValue + "] 的选项["
                    + instanceValue.getPropertyValue() + "]验证数值失败\n");
        }
    }

    /***
     * 转换及设置权限
     *
     * @param list
     * @param moduleId
     * @param userId
     * @param hasAdminRole
     * @throws Exception
     */
    @Override
    public void convertInstanceValue(List<Map<String, Object>> list, Long moduleId, Long userId, boolean hasAdminRole, String positionKey)
            throws Exception {
        if (ListTool.isEmpty(list)) {
            return;
        }
        List<Property> properties = propertyRepository.listNotHiddenByCmdbModuleId(moduleId, positionKey);
        Map<String, Property> propertyMap = new HashMap<>();
        for (Property property : properties) {
            propertyMap.put(property.getCode(), property);
        }
        Iterator<Map<String, Object>> iterator = list.iterator();
        while (iterator.hasNext()) {
            Long creatorId = 0L;
            Map<String, Object> instanceValue = iterator.next();
            Iterator<Map.Entry<String, Object>> it = instanceValue.entrySet().iterator();
            Map<String, Object> instanceValueMap = new HashMap<>();
            while (it.hasNext()) {
                Map.Entry<String, Object> entry = it.next();
                instanceValueMap.put(entry.getKey(), instanceValue.get(entry.getKey()));
                it.remove();
            }
            for (String key : instanceValueMap.keySet()) {
                // 转换字段为驼峰
                if ("gmt_create".equals(key)) {
                    // Java 8 date/time type `java.time.LocalDateTime` not supported by default
                    // 需要针对数据转换
                    DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                    String gmtCreate = df.format((LocalDateTime) instanceValueMap.get(key));
                    instanceValue.put(CmdbUtils.toCamelCase(key), gmtCreate);
                } else if (Common.ID.equals(key) || "module_id".equals(key) || "module_code".equals(key)) {
                    instanceValue.put(CmdbUtils.toCamelCase(key), instanceValueMap.get(key));
                } else if ("creator_id".equals(key)) {
                    Object instanceVal = instanceValueMap.get(key);
                    if (null != instanceVal) {
                        creatorId = Long.parseLong(instanceVal.toString());
                    }
                    instanceValue.put(CmdbUtils.toCamelCase(key), instanceVal);
                } else {
                    String instanceKey = CmdbUtils.toCamelCase(key);
                    Object instanceVal = instanceValueMap.get(key);
                    if (propertyMap.containsKey(instanceKey)) {
                        Property property = propertyMap.get(instanceKey);
                        if ((PropertyConstant.PropertyType.CHECKBOX.equals(property.getType())
                                || PropertyConstant.PropertyType.MULTISELECT.equals(property.getType())
                                || PropertyConstant.PropertyType.SINGLELINETABLE.equals(property.getType())
                                || PropertyConstant.PropertyType.MULTILINETABLE.equals(property.getType()))
                                && null != instanceVal && StringUtils.isNotEmpty(String.valueOf(instanceVal))) {
                            instanceValue.put(instanceKey, JSONTools.parseStr2JsonArray(instanceVal.toString()));

                        } else if (PropertyConstant.PropertyType.JSON.equals(property.getType()) && null != instanceVal
                                && StringUtils.isNotEmpty(String.valueOf(instanceVal))) {

                            GeneralResult result = JSONTools.isJSON(instanceVal.toString());
                            if (result.isSuccess()) {
                                instanceValue.put(instanceKey, result.getData());
                            } else {
                                instanceValue.put(instanceKey, JSONTools.parseStr2JsonArray(instanceVal.toString()));

                            }
                        } else {
                            instanceValue.put(instanceKey, instanceVal);
                        }
                    } else {
                        instanceValue.put(key, instanceVal);
                    }
                }
            }
            // 设置实例的权限，包括编辑、删除、管理权限
            boolean authEdit = false;
            boolean authDelete = false;
            boolean authManage = false;
            if (hasAdminRole || userId.equals(creatorId)) {
                authEdit = true;
                authDelete = true;
                authManage = true;
            } else {
                List<InstanceAuthority> authorities = instanceAuthorityRepository.listByUserId(moduleId,
                        Long.valueOf(instanceValue.get(Common.ID).toString()), userId, positionKey);
                for (InstanceAuthority authority : authorities) {
                    if (StringUtils.isNotEmpty(authority.getAuths())
                            && authority.getAuths().contains(InstanceEnum.authority.EDIT.getValue())) {
                        authEdit = true;
                    }
                    if (StringUtils.isNotEmpty(authority.getAuths())
                            && authority.getAuths().contains(InstanceEnum.authority.DEL.getValue())) {
                        authDelete = true;
                    }
                    if (StringUtils.isNotEmpty(authority.getAuths())
                            && authority.getAuths().contains(InstanceEnum.authority.AUTH_MANAGE.getValue())) {
                        authManage = true;
                    }
                }
            }
            instanceValue.put("authEdit", authEdit);
            instanceValue.put("authDelete", authDelete);
            instanceValue.put("authManage", authManage);
        }
    }

    /***
     * 单一实例转换
     *
     * @param object
     * @param moduleId
     * @throws Exception
     */
    private void convertInstanceValue(JSONObject object, Long moduleId) throws Exception {
        if (null == object) {
            return;
        }
        List<Property> properties = propertyRepository.listNotHiddenByCmdbModuleId(moduleId);
        Map<String, Property> propertyMap = new HashMap<>();
        for (Property property : properties) {
            propertyMap.put(property.getCode(), property);
        }
        Iterator<Map.Entry<String, Object>> it = object.entrySet().iterator();
        Map<String, Object> instanceValueMap = new HashMap<>();
        while (it.hasNext()) {
            Map.Entry<String, Object> entry = it.next();
            instanceValueMap.put(entry.getKey(), object.get(entry.getKey()));
            it.remove();
        }
        for (String key : instanceValueMap.keySet()) {
            // 转换字段为驼峰
            if ("gmt_create".equals(key)) {
                // Java 8 date/time type `java.time.LocalDateTime` not supported by default
                // 需要针对数据转换
                DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                String gmtCreate = df.format((LocalDateTime) instanceValueMap.get(key));
                object.put(CmdbUtils.toCamelCase(key), gmtCreate);
            } else if (Common.ID.equals(key) || "module_id".equals(key) || "module_code".equals(key)) {
                object.put(CmdbUtils.toCamelCase(key), instanceValueMap.get(key));
            } else {
                String instanceKey = CmdbUtils.toCamelCase(key);
                Object instanceVal = instanceValueMap.get(key);
                if (propertyMap.containsKey(instanceKey)) {
                    Property property = propertyMap.get(instanceKey);
                    if ((PropertyConstant.PropertyType.CHECKBOX.equals(property.getType())
                            || PropertyConstant.PropertyType.MULTISELECT.equals(property.getType())
                            || PropertyConstant.PropertyType.JSON.equals(property.getType())
                            || PropertyConstant.PropertyType.SINGLELINETABLE.equals(property.getType())
                            || PropertyConstant.PropertyType.MULTILINETABLE.equals(property.getType()))
                            && null != instanceVal && StringUtils.isNotEmpty(String.valueOf(instanceVal))) {
                        GeneralResult result = JSONTools.isJSON(instanceVal.toString());
                        if (result.isSuccess()) {
                            object.put(instanceKey, JSONObject.parseObject(instanceVal.toString()));
                        } else {
                            object.put(instanceKey, JSONArray.parseArray(instanceVal.toString()));
                        }
                    } else {
                        object.put(instanceKey, instanceVal);
                    }
                }
            }
        }
    }

    /***
     * 检查当前用户是否有超级管理员的角色
     *
     * @param userId
     * @return
     */
    @Override
    public boolean checkAdminRole(Long userId) {
        boolean hasAdminRole = false;
        GeneralResult roleResult = smsInternalService.getRolesByUserId(userId);
        if (roleResult.isFailed()) {
            log.error("查询当前用户角色失败:{}", roleResult.getMessage());
            return false;
        }
        List<Role> roles = JSONObject.parseArray(JSONObject.toJSONString(roleResult.getData()), Role.class);
        for (Role role : roles) {
            // 1 == role.getId() 超级管理员
            if (1 == role.getId()) {
                hasAdminRole = true;
                break;
            }
        }
        return hasAdminRole;
    }

    @Override
    public GeneralResult download(Long moduleId, String moduleCode, String type, Long userId) {
        Workbook workbook = null;
        try {
            CiModule module = moduleRepository.query(moduleId);
            if (null == module) {
                return new GeneralResult<>(false, "模型数据不存在");
            }
            if (!moduleCode.equals(module.getCode())) {
                return new GeneralResult<>(false, "模型数据不存在");
            }
            List<Property> properties = queryModulePropertys(module.getId());
            List<Property> propertieList = new ArrayList<>();
            // 移除uuid字段及单行文本中的内置函数、流水号和自增id
            for (Property property : properties) {
                if (Common.UUID.equals(property.getCode())) {
                    continue;
                }
                // 单行文本中流水号、自增、内置函数不校验数据规则，值代码中自动生成
                String defaultValueType = property.getDefaultValueType();
                if (PropertyConstant.PropertyType.TEXTFIELD.equals(property.getType())) {
                    if (PropertyConstant.DefaultValueType.SERIALNUM.equalsIgnoreCase(defaultValueType)
                            || PropertyConstant.DefaultValueType.AUTOINCRID.equalsIgnoreCase(defaultValueType)
                            || PropertyConstant.DefaultValueType.INNERFUN.equalsIgnoreCase(defaultValueType)) {
                        continue;
                    }
                }
                propertieList.add(property);
            }

            String fileName = module.getName() + "-实例模板.xls";
            if ("2007".equals(type)) {
                fileName = fileName + "x";
            }
            workbook = new ExcelFactory(fileName).newWorkbook();
            // 构建表头
            excelHelper.generateTemplateSheet(workbook, module, propertieList);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            workbook.write(baos);
            byte[] ba = baos.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName);
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new GeneralResult<>(true, resultMap, "模板下载成功");
        } catch (Exception e) {
            log.error("instance template download fail:", e);
            return new GeneralResult<>(false, "模板下载失败");
        }
    }

    @Override
    public GeneralResult upload(Long moduleId, String moduleCode, MultipartFile excel, RequestContext context) {
        try {
            CiModule module = moduleRepository.query(moduleId);
            if (null == module) {
                return new GeneralResult<>(false, "模型数据不存在");
            }
            if (!moduleCode.equals(module.getCode())) {
                return new GeneralResult<>(false, "模型数据不存在");
            }
            List<Property> properties = queryModulePropertys(module.getId());
            // 移除uuid字段
            List<Property> propertieList =
                    properties.stream().filter(item -> !Common.UUID.equals(item.getCode())).collect(Collectors.toList());
            String fileName = URLDecoder.decode(excel.getOriginalFilename(), "UTF-8");
            ExcelFactory excelFactory;
            excelFactory = new ExcelFactory(excel.getInputStream(), fileName);
            Workbook wb = excelFactory.newWorkbook();
            String uuid = IDFactory.instance().uuid();
            taskExecutor.execute(() -> {
                this.uploadInstances(uuid, wb, moduleId, moduleCode, propertieList, context);
            });
        } catch (Exception e) {
            log.error("实例数据导入失败", e);
            return new GeneralResult<>(false, "实例数据导入失败");
        }
        return new GeneralResult<>(true, "实例数据导入任务已下发");
    }

    private List<Property> queryModulePropertys(Long id) {
        List<Property> propertys = new ArrayList<>();
        GeneralResult<PropertyModel> result = property(id);
        if (result.isSuccess()) {
            propertys = result.getData().getProperties();
        }
        return propertys;
    }

    public GeneralResult<PropertyModel> property(Long moduleId) {
        try {
            List<PropertyGroup> groups = propertyGroupRepository.listByModuleId(moduleId);
            List<Property> properties = propertyRepository.listNotHiddenByCmdbModuleId(moduleId);
            List<PropertyItem> items = propertyItemRepository.queryByModuleId(moduleId);
            for (Property property : properties) {
                propertyService.propertyHandle(property, items, null);
            }
            PropertyModel model = new PropertyModel(moduleId, groups, properties);
            return new GeneralResult<>(true, model, "查询属性成功");
        } catch (Exception e) {
            log.error("moduleService property fail:", e);
            return new GeneralResult<>(false, "查询属性失败");
        }
    }

    @Override
    public GeneralResult<InstanceTopo> topo(Long id, Long relationCategoryId, Long moduleId, Long userId, String positionKey) {
        try {
            RelationCategory relationCategory = relationCategoryRepository.query(relationCategoryId, positionKey);
            if (null == relationCategory) {
                return new GeneralResult<>(false, "关系分类不存在");
            }
            String relationCategoryCode = relationCategory.getCode();
            CiModule module = moduleRepository.query(moduleId, positionKey);
            if (null == module) {
                return new GeneralResult<>(false, "模型不存在");
            }

            String moduleCode = module.getCode();
            String instanceTable = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
            String instanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            String instanceRelationTable = CmdbUtils.CMDB_INSTANCE_RELATION + relationCategoryCode.toLowerCase();
            Instance instance = instanceRepository.query(instanceTable, id, positionKey);
            InstanceValue instanceValue = instanceValueRepository.query(instanceValueTable, Common.NAME, id, positionKey);
            if (null == instance || null == instanceValue) {
                return new GeneralResult<>(false, "实例不存在");
            }
            List<ModuleRelation> moduleRelations =
                    moduleRelationRepository.queryAll(relationCategory.getId(), moduleId, positionKey);
            if (CollectionUtils.isEmpty(moduleRelations)) {
                InstanceTopo topo = new InstanceTopo();
                // 查询当前实例
                InstanceModel instanceModel = new InstanceModel();
                instanceModel.setInstanceId(id);
                instanceModel.setInstanceName(instanceValue.getPropertyValue());
                instanceModel.setModuleId(instance.getModuleId());
                instanceModel.setModuleCode(instance.getModuleCode());
                instanceModel.setIconImage(module.getIconImage());
                topo.setNodes(Arrays.asList(instanceModel));
                topo.setLinks(new ArrayList<>());
                return new GeneralResult<>(true, topo, "模型关系不存在");
            }
            // 反向模型关系名称
            Map<Long, String> relationReversedMap = new HashMap<>();
            Map<Long, String> relationMap = new HashMap<>();
            for (ModuleRelation relation : moduleRelations) {
                relationReversedMap.put(relation.getId(),
                        relation.getTargetName() + relation.getRelationReverseName() + relation.getSourceName());
                relationMap.put(relation.getId(),
                        relation.getSourceName() + relation.getRelationName() + relation.getTargetName());
            }
            List<InstanceRelation> instanceRelations = new ArrayList<>();
            List<InstanceRelation> sourceInstanceRelations =
                    instanceRelationRepository.querySources(instanceRelationTable, moduleId, id, positionKey);
            List<InstanceRelation> targetInstanceRelations =
                    instanceRelationRepository.queryTargets(instanceRelationTable, moduleId, id, positionKey);
            instanceRelations.addAll(sourceInstanceRelations);
            instanceRelations.addAll(targetInstanceRelations);
            // 所有实例 moduleCode+"-"+moduleId+"-"+instanceId
            Set<String> instanceSet = new HashSet<>();
            // 放入自身
            instanceSet.add(moduleCode + "-" + moduleId + "-" + id);
            List<InstanceModel> nodes = new ArrayList<>();
            List<InstanceRelationModel> links = new ArrayList<>();

            for (InstanceRelation relation : instanceRelations) {
                instanceSet.add(relation.getSourceModuleCode() + "-" + relation.getSourceModuleId() + "-"
                        + relation.getSourceInstanceId());
                instanceSet.add(relation.getTargetModuleCode() + "-" + relation.getTargetModuleId() + "-"
                        + relation.getTargetInstanceId());
                // 放入link
                InstanceRelationModel link = new InstanceRelationModel();
                BeanUtils.copyProperties(relation, link);
                link.setRelationId(relation.getModuleRelationId());
                link.setRelationName(relation.getModuleRelationName());
                // 反向的时候，箭头相反相关信息
                if (relation.getIsReversed()) {
                    link.setRelationName(relationReversedMap.get(relation.getModuleRelationId()));
                    link.setType(relation.getTargetType() + "-" + relation.getSourceType());

                    // 反向的时候源和目标信息对调
                    link.setSourceInstanceId(relation.getTargetInstanceId());
                    link.setSourceInstanceName(relation.getTargetInstanceName());
                    link.setSourceModuleCode(relation.getTargetModuleCode());
                    link.setSourceModuleId(relation.getTargetModuleId());
                    link.setTargetInstanceId(relation.getSourceInstanceId());
                    link.setTargetInstanceName(relation.getSourceInstanceName());
                    link.setTargetModuleCode(relation.getSourceModuleCode());
                    link.setTargetModuleId(relation.getSourceModuleId());
                } else {
                    link.setRelationName(relationMap.get(relation.getModuleRelationId()));
                    link.setType(relation.getSourceType() + "-" + relation.getTargetType());
                }
                links.add(link);
            }

            if (instanceSet.size() > 0) {
                Iterator<String> instances = instanceSet.iterator();
                while (instances.hasNext()) {
                    InstanceModel instanceModel = new InstanceModel();
                    String ins = instances.next();
                    String[] insSplit = ins.split("-");
                    String insModuleCode = insSplit[0];
                    Long insModuleId = Long.valueOf(insSplit[1]);
                    Long insId = Long.valueOf(insSplit[2]);
                    module = moduleRepository.query(insModuleId);
                    instanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + insModuleCode.toLowerCase();
                    instanceValue = instanceValueRepository.query(instanceValueTable, Common.NAME, insId, positionKey);
                    if (null == module || null == instanceValue) {
                        continue;
                    }
                    instanceModel.setInstanceId(insId);
                    instanceModel.setInstanceName(instanceValue.getPropertyValue());
                    instanceModel.setModuleId(insModuleId);
                    instanceModel.setModuleCode(insModuleCode);
                    instanceModel.setIconImage(module.getIconImage());
                    nodes.add(instanceModel);
                }
            }
            InstanceTopo topo = new InstanceTopo();
            topo.setNodes(nodes);
            topo.setLinks(links);
            return new GeneralResult<>(true, topo, "实例拓扑查询成功");
        } catch (Exception e) {
            log.error("Cmdb instance topo query fail:", e);
            return new GeneralResult<>(false, "实例拓扑查询失败");
        }
    }

    public void uploadInstances(String uuid, Workbook wb, Long moduleId, String moduleCode, List<Property> properties,
                                RequestContext context) {
        boolean validateSuccess = true;
        Map<String, Property> propertyMap = new HashMap<>();
        for (Property property : properties) {
            propertyMap.put(property.getName(), property);
        }
        String instanceTable = CmdbUtils.CMDB_INSTANCE + moduleCode.toLowerCase();
        String instanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
        try {
            validataWb(wb, propertyMap);
        } catch (Exception e) {
            validateSuccess = false;
        }
        int total = countDataNum(wb);
        int success = 0;
        int fail = 0;
        if (validateSuccess) {
            // 遍历sheet页
            for (int n = 0; n < wb.getNumberOfSheets(); n++) {
                try {
                    Sheet sheet = wb.getSheetAt(n);
                    // 属性
                    Map<String, Integer> header = ExcelHelper.readHeader4NameKey(sheet.getRow(0), 1);
                    Map<Integer, Property> headerProperty = Maps.newHashMap();
                    for (Property property : properties) {
                        Integer index = header.get(property.getName());
                        String propertyType = property.getType();
                        if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                                || PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
                                || PropertyConstant.PropertyType.RADIO.equals(propertyType)
                                || PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)) {
                            List<PropertyItem> items = property.getItemList();
                            property.setItemList(items);
                        }
                        if (null != index && index > 0) {
                            headerProperty.put(index, property);
                        }
                    }
                    // 表头单元格开始位置，依次每行按照其索引位置开始读取
                    int startIndex = 1;
                    // 最后一列数
                    int lastColumn = header.size() + startIndex;
                    // 遍历行
                    for (int i = 2; i <= sheet.getLastRowNum(); i++) {

                        Row row = sheet.getRow(i);
                        if (row == null) {
                            throw new Exception("请检查第" + (i + 1) + "行下面是否有干扰数据。");
                        }
                        JSONObject instanceValueObj = new JSONObject();
                        for (int j = startIndex; j < lastColumn; j++) {
                            Cell cell = row.getCell(j);
                            Property property = headerProperty.get(j);
                            if (null != property) {
                                String propertyType = property.getType();
                                String propertyCode = property.getCode();
                                Object propertyValue = ExcelHelper.getCellValue(cell);
                                // 导出的模板显示的是name，需要重新赋值
                                if (PropertyConstant.PropertyType.SINGLESELECT.equals(propertyType)
                                        || PropertyConstant.PropertyType.RADIO.equals(propertyType)) {
                                    List<PropertyItem> items = property.getItemList();
                                    if (CollectionUtils.isNotEmpty(items)) {
                                        for (PropertyItem item : items) {
                                            if (item.getName().equals(propertyValue)) {
                                                propertyValue = item.getValue();
                                            }
                                        }
                                    }
                                } else if (PropertyConstant.PropertyType.MULTISELECT.equals(propertyType)
                                        || PropertyConstant.PropertyType.CHECKBOX.equals(propertyType)) {
                                    List<PropertyItem> items = property.getItemList();
                                    if (CollectionUtils.isNotEmpty(items)) {
                                        List<String> itemValueList = new ArrayList<>();
                                        Map<String, PropertyItem> itemMap = new HashMap<>();
                                        for (PropertyItem item : items) {
                                            itemMap.put(item.getName(), item);
                                        }
                                        if (StringUtils.isNotEmpty(String.valueOf(propertyValue))) {
                                            String[] propertyValues = String.valueOf(propertyValue).split(",");
                                            for (String value : propertyValues) {
                                                if (itemMap.containsKey(value)) {
                                                    itemValueList.add(itemMap.get(value).getValue());
                                                }
                                            }
                                        }
                                        propertyValue = itemValueList;
                                    }
                                } else if (PropertyConstant.PropertyType.SINGLELINETABLE.equals(propertyType)
                                        || PropertyConstant.PropertyType.MULTILINETABLE.equals(propertyType)) {

                                    if (StringUtils.isEmpty(String.valueOf(propertyValue))) {
                                        propertyValue = new JSONArray();
                                    } else {
                                        propertyValue = JSONArray.parseArray(propertyValue.toString());
                                    }
                                } else if (PropertyConstant.PropertyType.JSON.equals(propertyType)) {
                                    if (StringUtils.isNotEmpty(String.valueOf(propertyValue))) {
                                        GeneralResult<JSONObject> result = JSONTools.isJSON(String.valueOf(propertyValue));
                                        if (result.isSuccess()) {
                                            propertyValue = result.getData();
                                        } else {
                                            JSONArray array = JSONTools.parseStr2JsonArray(String.valueOf(propertyValue));
                                            if (null != array) {
                                                propertyValue = array;
                                            } else {
                                                log.error("json类型属性格式有误{}", propertyValue);
                                            }

                                        }
                                    } else {
                                        //json默认值为空对象 {}
                                        propertyValue = new JSONObject();
                                    }

                                } else if (PropertyConstant.PropertyType.TEXTENCRYPT.equals(propertyType)) {

                                    if (StringUtils.isNotEmpty(String.valueOf(propertyValue))) {
                                        //密码加密，和页面添加实例密文加密方式一致
                                        AESEncryptor encrptor = new AESEncryptor();
                                        propertyValue = encrptor.encrypt(String.valueOf(propertyValue), null);
                                    }
                                }
                                instanceValueObj.put(propertyCode, propertyValue);
                            }
                        }
                        instanceValueObj.put("moduleId", moduleId);
                        instanceValueObj.put("moduleCode", moduleCode);
                        // 当前存在的实例
                        List<Instance> instances = instanceRepository.queryByModuleId(instanceTable, instanceValueTable,
                                moduleId, Common.NAME);
                        Map<String, Instance> instanceMap = new HashMap<>();
                        for (Instance instance : instances) {
                            instanceMap.put(instance.getName(), instance);
                        }
                        GeneralResult result = new GeneralResult(false, "导入失败");
                        // 判断是否存在，存在即更新
                        if (instanceMap.containsKey(instanceValueObj.getString(Common.NAME))) {
                            instanceValueObj.put(Common.ID,
                                    instanceMap.get(instanceValueObj.getString(Common.NAME)).getId());
                            result = this.modify(instanceValueObj, context.getTarget());
                        } else {
                            result = this.create(instanceValueObj, context.getTarget(), true);
                        }
                        if (result.isSuccess()) {
                            success++;
                        } else {
                            fail++;
                        }
                        ExcelHelper.promptErrorMessage(row, result.getMessage());
                        processMessage(uuid, context, total, success, fail);
                    }
                } catch (Exception e) {
                    total = 0;
                    wb.getSheetAt(0).setColumnWidth(0, 20000);
                    ExcelHelper.promptErrorMessage(wb, e.getMessage());
                    log.info("导入失败", e);
                }
            }
        } else {
            processMessage(uuid, context, 0, 0, 0);
        }
    }

    private void processMessage(String uuid, RequestContext context, int total, int success, int fail) {
        Map<String, Object> messageContent = Maps.newHashMap();
        messageContent.put("total", total);
        if (total == 0) {
            messageContent.put("progress", 100);
            messageContent.put("success", 0);
            messageContent.put("fail", "fail");
        } else {
            BigDecimal progress = new BigDecimal(success + fail).multiply(new BigDecimal(100))
                    .divide(new BigDecimal(total), 10, RoundingMode.HALF_DOWN);
            double progresses = Math.floor(progress.doubleValue());
            messageContent.put("progress", progresses);

            if (progresses == progressLast && progresses != 100) {
                return;
            }
            progressLast = progresses;
            messageContent.put("success", success);
            messageContent.put("fail", fail);
        }
        messageContent.put("uuid", uuid);
        eventPublisher.send(new OperateResult(true, "导入进度推送", "cmdb_instance", messageContent,
                OperateResult.OperateCategory.CMDBIMPORT, context));
    }

    private void validataWb(Workbook wb, Map<String, Property> propertyMap) throws Exception {
        for (int n = 0; n < wb.getNumberOfSheets(); n++) {
            Sheet sheet = wb.getSheetAt(n);
            Map<String, Integer> header = ExcelHelper.readHeader4NameKey(sheet.getRow(0), 1);
            Set<String> propNotContain = new HashSet<>();
            for (String name : header.keySet()) {
                if (!propertyMap.containsKey(name)) {
                    propNotContain.add(name);
                }
            }
            if (!propNotContain.isEmpty()) {
                throw new Exception("表头中缺少以下属性：" + propNotContain.toString());
            }
        }
    }

    private int countDataNum(Workbook wb) {
        int total = 0;
        for (int n = 0; n < wb.getNumberOfSheets(); n++) {
            Sheet sheet = wb.getSheetAt(n);
            total += ExcelHelper.getRowNumberOfSheet(sheet) - 2;
        }
        return total;

    }
}
