package com.bocloud.ams.service.module.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.collection.CollectionComponent;
import com.bocloud.ams.entity.enums.PropertyType;
import com.bocloud.ams.entity.model.PropertyModel;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.ModuleGroup;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.entity.module.PropertyGroup;
import com.bocloud.ams.entity.module.PropertyItem;
import com.bocloud.ams.entity.module.PropertyPool;
import com.bocloud.ams.entity.module.TopologyConfig;
import com.bocloud.ams.repository.collection.CollectionComponentRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.ModuleGroupRepository;
import com.bocloud.ams.repository.module.ModuleRelationRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.module.PropertyGroupRepository;
import com.bocloud.ams.repository.module.PropertyItemRepository;
import com.bocloud.ams.repository.module.PropertyPoolRepository;
import com.bocloud.ams.repository.module.PropertyRepository;
import com.bocloud.ams.repository.module.RegularExpressionRepository;
import com.bocloud.ams.repository.module.TopologyConfigRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.instance.InstanceService;
import com.bocloud.ams.service.internal.CmpInternalService;
import com.bocloud.ams.service.module.ModuleService;
import com.bocloud.ams.service.module.PropertyService;
import com.bocloud.ams.service.utils.ModuleConstant;
import com.bocloud.ams.service.utils.PropertyConstant;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtilsBean;
import org.apache.commons.beanutils.locale.LocaleBeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.Assert;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author 胡立伟
 * @program: bocloud.ams
 * @description:
 * @date 2022/5/5 11:55
 */
@Service
@Slf4j
public class ModuleServiceImpl implements ModuleService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    PropertyRepository propertyRepository;
    @Autowired
    PropertyGroupRepository propertyGroupRepository;
    @Autowired
    PropertyItemRepository propertyItemRepository;
    @Autowired
    RegularExpressionRepository regularExpressionRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private ModuleRelationRepository moduleRelationRepository;
    @Autowired
    private InstanceService instanceService;
    @Autowired
    private PropertyPoolRepository propertyPoolRepository;
    @Autowired
    private PropertyService propertyService;
    @Autowired
    private TopologyConfigRepository topologyConfigRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private CollectionComponentRepository collectionComponentRepository;
    @Autowired
    private ModuleGroupRepository moduleGroupRepository;
    @Autowired
    private CmpInternalService cmpInternalService;

    @Override
    public GeneralResult<GridBean<CiModule>> getModelList(Pager pager, String positionKey) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            // flag = false 仅查询当前分组下的模型数据；flag = true 查询当前分组下的所有模型数据包括子分组的模型数据
            List<Param> params = pager.getParams();
            List<Map<String, Object>> collectMap = params.stream().map(Param::getParam).collect(Collectors.toList());
            Map<String, Object> map =
                    collectMap.stream().filter(item -> item.containsKey("flag")).findFirst().orElse(null);
            if (null != map) {
                Param newParam = null;
                Iterator<Param> iterator = params.iterator();
                while (iterator.hasNext()) {
                    Map<String, Object> parMap = iterator.next().getParam();
                    if (parMap.containsKey("moduleGroupId") && parMap.containsKey("flag")) {
                        Long groupId = Long.valueOf((Integer) parMap.get("moduleGroupId"));
                        if (parMap.get("flag").equals(true)) {
                            // 递归获取所有子groupId
                            List<ModuleGroup> groups = new ArrayList<>();
                            if (StringUtils.isEmpty(positionKey)) {
                                groups = moduleGroupRepository.all();
                            } else {
                                groups = moduleGroupRepository.all(positionKey);
                            }
                            Map<Long, List<ModuleGroup>> groupMap =
                                    groups.stream().collect(Collectors.groupingBy(ModuleGroup::getParentId));
                            List<Long> groupIds = new ArrayList<>();
                            this.getGroups(groupIds, groupId, groupMap);
                            List<String> groupIdList =
                                    groupIds.stream().map(String::valueOf).collect(Collectors.toList());
                            String ids = String.join(",", groupIdList);
                            Map<String, Object> groupIdMap = new HashMap<>();
                            groupIdMap.put("moduleGroupId", ids);
                            newParam = new Param(groupIdMap, Sign.IN);
                        } else {
                            Map<String, Object> groupIdMap = new HashMap<>();
                            groupIdMap.put("moduleGroupId", groupId);
                            newParam = new Param(groupIdMap, Sign.EQ);
                        }
                        iterator.remove();
                        break;
                    }
                }
                if (null != newParam) {
                    params.add(newParam);
                }
            }
            int total = moduleRepository.count(pager.getParams(), positionKey);
            List<CiModule> modules = this.moduleRepository.list(pager, positionKey);
            if (CollectionUtils.isNotEmpty(modules)) {
                List<Long> groupIds = modules.stream().map(CiModule::getGroupId).collect(Collectors.toList());
                List<ModuleGroup> groups = moduleGroupRepository.queryByIds(groupIds, positionKey);
                if (CollectionUtils.isNotEmpty(groups)) {
                    for (CiModule module : modules) {
                        ModuleGroup group = groups.stream().filter(item -> item.getId().equals(module.getGroupId()))
                                .findFirst().orElse(null);
                        if (null != group) {
                            module.setParentGroupId(group.getParentId());
                        }
                    }
                }
            }
            GridBean<CiModule> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, modules);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("CmdbPropertyPool list query fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<CiModule>> list( String positionKey) {
        try {
            List<CiModule> modules = moduleRepository.all(positionKey);
            List<CiModule> list = modules.stream().filter(item -> ModuleConstant.STATUS_ENABLE.equals(item.getStatus()))
                    .collect(Collectors.toList());
            return new GeneralResult<>(true, list, "查询成功！");
        } catch (Exception e) {
            log.error("query module is error:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<CiModule> detail(Long id, String positionKey) {
        try {
            CiModule module = moduleRepository.query(id, positionKey);
            return new GeneralResult<>(true, module, "查询成功");
        } catch (Exception e) {
            log.error("moduleService detail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> save(CiModule module, Long target) {
        try {
            int sameName = moduleRepository.countSameNameOrCode(module.getName(), null);
            if (sameName > 0) {
                return new GeneralResult<>(false, "添加失败，该名称已存在，请尝试其他名称");
            }
            int sameCode = moduleRepository.countSameNameOrCode(null, module.getCode());
            if (sameCode > 0) {
                return new GeneralResult<>(false, "添加失败，该编码已存在，请尝试其他编码");
            }
            if (null == module.getGroupId()) {
                return new GeneralResult<>(false, "模型要有对应分组！");
            }
            String instanceTable = CmdbUtils.CMDB_INSTANCE + module.getCode().toLowerCase();
            if (instanceRepository.countTable(instanceTable) > 0) {
                log.warn("创建表空间{}失败", instanceTable);
                return new GeneralResult<>(false, "创建失败，" + instanceTable + "实例表已存");
            }
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + module.getCode().toLowerCase();
            if (instanceValueRepository.countTable(valueTable) > 0) {
                log.warn("创建表空间{}失败", valueTable);
                return new GeneralResult<>(false, "创建失败，" + valueTable + "实例表已存");
            }
            if (null == module.getStatus()) {
                module.setStatus(ModuleConstant.STATUS_ENABLE);
            }
            module.setGmtCreate(new Date());
            module.setGmtModify(new Date());
            module.setCreatorId(target);
            module.setMenderId(target);
            module.setReserved(false);
            moduleRepository.save(module);
            // 新增初始化数据property
            if (null == module.getId()) {
                log.warn("Create notice module fail!");
                return new GeneralResult<>(false, "新增模型失败");
            }
            List<Property> properties = Lists.newArrayList();
            PropertyGroup group = new PropertyGroup();
            group.setName("基础属性");
            group.setIsReserved(true);
            group.setModuleId(module.getId());
            group.setOrderNum(0);
            group.setCreatorId(target);
            group.setMenderId(target);
            propertyGroupRepository.save(group);
            if (null == group.getId()) {
                log.warn("Create PropertyGroup fail!");
                return new GeneralResult<>(false, "新增模型属性分组失败");
            }
            Property id = new Property("uuid", "唯一标识", PropertyType.TEXTFIELD.name(), group.getName(), 100, true, true,
                    1, module.getId(), true, false, false, target, PropertyConstant.DefaultValueType.INNERFUN,
                    PropertyConstant.GLOBALLY_UNIQUE_IDENTIFIER, true, false);
            // name设置为标题列
            Property name = new Property("name", "名称", PropertyType.TEXTFIELD.name(), group.getName(), 100, true, true,
                    2, module.getId(), true, false, true, target, PropertyConstant.DefaultValueType.FIXED, null, false,
                    true);
            properties.add(id);
            properties.add(name);
            propertyRepository.batchSave(properties, 10);
            // 创建模型对应的实例表
            instanceService.createTable(module.getCode(), target);
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("moduleService save fail:", e);
            // 手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> modify(CiModule module, Long target) {
        String path = CiModule.class.getSimpleName() + "_" + module.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CiModule object = moduleRepository.query(module.getId());
            if (null == object) {
                return new GeneralResult<>(false, "模型数据不存在");
            }
            if (!object.getName().equals(module.getName())) {
                int sameName = moduleRepository.countSameNameOrCode(module.getName(), null);
                if (sameName > 0) {
                    return new GeneralResult<>(false, "修改失败，该名称已存在，请尝试其他名称");
                }
            }
            if (!object.getCode().equals(module.getCode())) {
                int sameCode = moduleRepository.countSameNameOrCode(null, module.getCode());
                if (sameCode > 0) {
                    return new GeneralResult<>(false, "修改失败，该编码已存在，请尝试其他编码");
                }
            }
            LocaleBeanUtils.copyProperties(object, module);
            if (null == object.getStatus()) {
                object.setStatus(ModuleConstant.STATUS_ENABLE);
            }
            object.setGmtModify(new Date());
            object.setMenderId(target);
            moduleRepository.update(object);
            return new GeneralResult<>(true, "变更成功");
        } catch (Exception e) {
            log.error("moduleService modify fail:", e);
            // 手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new GeneralResult<>(false, "变更成功");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = CiModule.class.getSimpleName() + "_sasasas" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CiModule module = moduleRepository.query(id);
            if (null == module) {
                return new GeneralResult<>(false, "模型数据不存在");
            }
            List<TopologyConfig> configs = topologyConfigRepository.listAll();
            // 获取拓扑路径涉及到的模型数据
            List<Long> modules = configs.stream().filter(item -> StringUtils.isNotEmpty(item.getTopologyPath()))
                    .flatMap(config -> Arrays.stream(config.getTopologyPath().split(","))).distinct()
                    .map(item -> Long.valueOf(item)).collect(Collectors.toList());
            List<Long> ignores = configs.stream().filter(item -> StringUtils.isNotEmpty(item.getIgnoreModuleIds()))
                    .flatMap(config -> Arrays.stream(config.getIgnoreModuleIds().split(","))).distinct()
                    .map(item -> Long.valueOf(item)).collect(Collectors.toList());
            modules.addAll(ignores);
            modules.removeAll(Collections.singleton(null)); // 移除所有的null元素
            if (modules.contains(id)) {
                return new GeneralResult<>(false, "实例拓扑中使用到此模型，删除失败！");
            }
            // 校验是否被其他模型属性引用（多行表格和单行表格存在引入其他模型属性）
            List<Property> linkModules = propertyRepository.queryByLinkModuleId(id);
            if (CollectionUtils.isNotEmpty(linkModules)) {
                return new GeneralResult<>(false, "存在其他模型引用此模型属性，模型删除失败！");
            }
            List<CollectionComponent> collectionComponents = collectionComponentRepository.queryByModuleId(id);
            if (CollectionUtils.isNotEmpty(collectionComponents)) {
                return new GeneralResult<>(false, "当前模型已被采集插件使用，模型删除失败！");
            }
            // 删除模型数据
            moduleRepository.delete(id, userId);
            // 删除模型属性
            propertyRepository.deleteByModuleId(id, userId);
            // 删除模型属性分组
            propertyGroupRepository.deleteByModuleId(id, userId);
            // 删除模型属性明细数据
            propertyItemRepository.deleteByModuleId(id, userId);
            // 删除模型的关联关系
            moduleRelationRepository.deleteByModuleId(id, userId);
            // 删除模型实例表
            instanceService.dropTable(module.getCode(), userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("moduleService remove fail:", e);
            // 手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<PropertyModel> property(Long moduleId, String positionKey) {
        try {
            List<PropertyGroup> groups = propertyGroupRepository.listByModuleId(moduleId, positionKey);
            List<Property> properties = propertyRepository.listNotHiddenByCmdbModuleId(moduleId, positionKey);
            List<PropertyItem> items = propertyItemRepository.queryByModuleId(moduleId, positionKey);
            for (Property property : properties) {
                propertyService.propertyHandle(property, items, positionKey);
            }
            PropertyModel model = new PropertyModel(moduleId, groups, properties);
            return new GeneralResult<>(true, model, "查询属性成功");
        } catch (Exception e) {
            log.error("moduleService property fail:", e);
            return new GeneralResult<>(false, "查询属性失败");
        }
    }

    @Override
    public GeneralResult<GridBean<Property>> propertyList(Long moduleId, Pager pager, String positionKey) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put("id", Common.ONE);
            int total = propertyRepository.count(pager.getParams(), moduleId, positionKey);
            List<Property> properties = propertyRepository.list(pager, moduleId, positionKey);
            for (Property property : properties) {
                if (null != property.getPropertyPoolId()) {
                    PropertyPool pool = propertyPoolRepository.query(property.getPropertyPoolId(), positionKey);
                    this.copyProperties(property, pool);
                }
                property.setPropertyId(property.getId());
            }
            GridBean<Property> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, properties);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("moduleService propertyList fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> modifyProperty(PropertyModel model, Long id, RequestContext context) {
        String path = Property.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 数据处理
            return this.baseModifyProperty(id, model.getGroups(), model.getProperties(), context.getTarget());
        } catch (Exception e) {
            log.error("moduleService modifyProperty fail:", e);
            // 手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new GeneralResult<>(false, "保存失败," + e.getMessage());
        }
    }

    @Override
    public GeneralResult statistics(RequestContext context, String positionKey) {
        try {
            List<String> nameList = Arrays.asList("网络设备", "物理服务器", "安全设备");
            //查询根节点
            List<ModuleGroup> rootGroups = moduleGroupRepository.queryRootInName(nameList, positionKey);

            Map<String, Object> resultMap = new HashMap<>();

            for (ModuleGroup rootGroup : rootGroups) {
                //获取每个模型的所有实例总数
                getInstaceCount(rootGroup, resultMap, context.getTarget(), positionKey);
            }

            //查询云主机总数
            Pager pager = new Pager();
            pager.setPage(1);
            pager.setRows(Integer.MAX_VALUE);
            List<Param> paramList = new ArrayList<>();
            Param param = new Param();
            param.setParam(MapTools.simpleMap("isTemplate", false));
            param.setSign(Sign.EQ);
            paramList.add(param);
            pager.setParams(paramList);
            GeneralResult<JSONObject> result = cmpInternalService.list(pager.toMap());
            if (result.isFailed()) {
                log.info("list:{}", JSON.toJSONString(result));
                return new GeneralResult(false, "查询失败");
            }

            resultMap.put("vm", result.getData().getInteger("total"));

            return new GeneralResult<>(true, resultMap, "查询成功");
        } catch (Exception e) {
            log.info("查询失败", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private void getInstaceCount(ModuleGroup rootGroup, Map<String, Object> resultMap, Long userId,  String positionKey) throws Exception {
        //获取当前ModuleGroup下所有的modle
        Pager pager = new Pager();
        List<Param> modelParamList = new ArrayList<>();
        Param modelParam = new Param();
        Map<String, Object> map = MapTools.simpleMap("moduleGroupId", Integer.valueOf(String.valueOf(rootGroup.getId())));
        map.put("flag",true);
        modelParam.setParam(map);
        modelParam.setSign(Sign.EQ);
        pager.setPage(1);
        pager.setRows(Integer.MAX_VALUE);
        modelParamList.add(modelParam);
        pager.setParams(modelParamList);
        GeneralResult<GridBean<CiModule>> modelList = getModelList(pager, positionKey);
        if (modelList.isFailed()) {
            log.info("modelList:{}", JSON.toJSONString(modelList));
            throw new Exception("查询实例失败");
        }
        Integer total = 0;
        for (CiModule model : modelList.getData().getRows()) {
            //获取每个模型下所有的实例总数
            String table = CmdbUtils.CMDB_INSTANCE + model.getCode().toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + model.getCode().toLowerCase();
            List<Param> paramList = new ArrayList<>();
            Param param = new Param();
            param.setParam(MapTools.simpleMap("moduleId", model.getId()));
            param.setSign(Sign.EQ);
            paramList.add(param);
            boolean adminRole = instanceService.checkAdminRole(userId);
            int count = instanceRepository.count(paramList, table, valueTable, userId, adminRole, positionKey);
            total += count;
        }
        resultMap.put(rootGroup.getCode(), total);
    }

    private GeneralResult baseModifyProperty(Long moduleId, List<PropertyGroup> groups, List<Property> properties,
                                             Long userId) throws Exception {
        CiModule module = moduleRepository.query(moduleId);
        if (module == null) {
            return GeneralResult.FAILED("修改属性对应的配置模型不存在");
        }
        // 参数的数据校验
        this.validate(properties, groups);
        // 查询原本的property数据
        List<Property> oldProperties = propertyRepository.listNotHiddenByCmdbModuleId(moduleId);
        // 查询原本的propertyItem数据
        List<PropertyItem> oldItems = propertyItemRepository.queryByModuleId(moduleId);
        // 查询原本的group数据
        List<PropertyGroup> oldGroups = propertyGroupRepository.listByModuleId(moduleId);
        // 1-add：新增property数据 以及对应的propertyItem数据
        List<Property> addProperties =
                properties.stream().filter(item -> null == item.getId()).collect(Collectors.toList());
        // 3-update:更新property数据（不允许从池表中引入的数据被更新） 以及对应的propertyItem数据
        List<Property> updateProperties = properties.stream()
                .filter(item -> null != item.getId()).collect(Collectors.toList());
        // 2-delete：删除去掉的property数据 以及对应的propertyItem数据
        List<Long> inputPropertyIds =
                properties.stream().filter(item -> null != item.getId()).map(Property::getId).collect(Collectors.toList());
        List<Property> deleteProperties = oldProperties.stream()
                .filter(item -> !inputPropertyIds.contains(item.getId())).collect(Collectors.toList());
        // 1-delete：删除去掉的property数据 以及 对应的propertyItem数据
        if (CollectionUtils.isNotEmpty(deleteProperties)) {
            // 先校验是否存在被其他模型引用的属性，如果存在就不允许删除
            List<Long> deletePropertyIds = deleteProperties.stream().map(Property::getId).collect(Collectors.toList());
            List<Property> linkModuleIds = propertyRepository.queryByLinkModuleId(moduleId);
            if (CollectionUtils.isNotEmpty(linkModuleIds)) {
                long count = linkModuleIds.stream().map(Property::getPropertyIds).map(item -> item.split(","))
                        .flatMap(item -> Arrays.stream(item)).filter(item -> deletePropertyIds.contains(item)).count();
                if (count > 0) {
                    return GeneralResult.FAILED("删除属性失败，存在其他模型引用此属性");
                }
            }
            // 校验当前模型属性是否被采集插件引用
            List<CollectionComponent> collectionComponents = collectionComponentRepository.queryByModuleId(moduleId);
            if (CollectionUtils.isNotEmpty(collectionComponents)) {
                long count = collectionComponents.stream().map(CollectionComponent::getPropertyId)
                        .filter(item -> deletePropertyIds.contains(item)).count();
                if (count > 0) {
                    return GeneralResult.FAILED("删除属性失败，存在属性被采集插件使用");
                }
            }
            deleteProperties.stream().forEach(item -> {
                item.setGmtModify(new Date());
                item.setMenderId(userId);
                item.setDeleted(true);
            });
            String instanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + module.getCode().toLowerCase();
            for (Property deleteProperty : deleteProperties) {
                propertyRepository.update(deleteProperty);
                // 删除对应的实例数据
                instanceValueRepository.removeByPropertyId(deleteProperty.getId(), instanceValueTable, userId);
            }
            // propertyRepository.batchUpdate(deleteProperties, deleteProperties.size());
            // 根据需要删除的property数据删除对应的propertyItem数据
            List<Long> propertyIds = deleteProperties.stream().map(Property::getId).collect(Collectors.toList());
            List<PropertyItem> deleteItems = oldItems.stream()
                    .filter(item -> propertyIds.contains(item.getPropertyId())).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(deleteItems)) {
                deleteItems.stream().forEach(item -> {
                    item.setGmtModify(new Date());
                    item.setMenderId(userId);
                    item.setDeleted(true);
                });
                propertyItemRepository.batchUpdate(deleteItems, deleteItems.size());
            }
        }

        // 2-add：新增property数据并新增对应的propertyItem数据
        for (Property property : addProperties) {
            // 新增Property
            property.setMenderId(userId);
            property.setCreatorId(userId);
            property.setModuleId(moduleId);
            if (PropertyConstant.RESERVED_MODULE_CODES.contains(property.getCode().toLowerCase())) {
                throw new Exception(property.getName() + "属性指定的code[" + property.getCode() + "]为系统内置code，请修改后重试");
            }
            try {
                propertyRepository.save(property);
            } catch (Exception e) {
                throw new Exception("创建属性失败");
            }
            // 新增PropertyItem
            if (CollectionUtils.isNotEmpty(property.getItemList())) {
                property.getItemList().stream().forEach(item -> {
                    if (CollectionUtils.isNotEmpty(item.getItemList())) {
                        // 表格属性中结构项数据的下拉选项数据 转成 JSON 存储
                        item.setItemValue(JSON.toJSONString(item.getItemList()));
                    }
                    item.setModuleId(property.getModuleId());
                    item.setPropertyId(property.getId());
                    item.setCreatorId(userId);
                    item.setMenderId(userId);
                });
                propertyItemRepository.batchSave(property.getItemList(), property.getItemList().size());
            }
        }
        // 3-update:更新数据
        for (Property property : updateProperties) {
            if (null != property.getPropertyPoolId()) {
                property.setGmtModify(new Date());
                property.setMenderId(userId);
                propertyRepository.update(property);
                continue;
            }
            // 获取新增item数据
            List<PropertyItem> addItems = new ArrayList<>();
            if (!Common.ONE.equals(property.getTableType())) {
                addItems = property.getItemList().stream().filter(item -> null == item.getId()).collect(Collectors.toList());
                addItems.stream().forEach(item -> {
                    item.setModuleId(property.getModuleId());
                    item.setPropertyId(property.getId());
                    item.setCreatorId(userId);
                    item.setMenderId(userId);
                    if (CollectionUtils.isNotEmpty(item.getItemList())) {
                        // 表格属性中结构项数据的下拉选项数据 转成 JSON 存储
                        item.setItemValue(JSON.toJSONString(item.getItemList()));
                    }
                });
            }
            // 获取更新的item数据
            List<PropertyItem> updateItems = new ArrayList<>();
            List<Long> updateItemIds = new ArrayList<>();
            if (!Common.ONE.equals(property.getTableType())) {
                updateItems = property.getItemList().stream().filter(item -> null != item.getId()).collect(Collectors.toList());
                updateItems.stream().forEach(item -> {
                    item.setMenderId(userId);
                    if (CollectionUtils.isNotEmpty(item.getItemList())) {
                        // 表格属性中结构项数据的下拉选项数据 转成 JSON 存储
                        item.setItemValue(JSON.toJSONString(item.getItemList()));
                    }
                    updateItemIds.add(item.getId());
                });
            }
            // 根据当前属性id查询出之前的item
            List<PropertyItem> updateOldItems = oldItems.stream()
                    .filter(item -> item.getPropertyId().equals(property.getId())).collect(Collectors.toList());
            // 获取需要删除的item数据（就是原始的减去现在的）
            List<PropertyItem> deleteItems = updateOldItems.stream()
                    .filter(item -> !updateItemIds.contains(item.getId())).collect(Collectors.toList());
            deleteItems.stream().forEach(item -> {
                item.setGmtModify(new Date());
                item.setMenderId(userId);
                item.setDeleted(true);
            });
            // 删除 更新的property数据中被删除的item数据
            if (CollectionUtils.isNotEmpty(deleteItems)) {
                propertyItemRepository.batchUpdate(deleteItems, deleteItems.size());
            }
            // 新增 更新的property数据中新增的item数据
            if (CollectionUtils.isNotEmpty(addItems)) {
                propertyItemRepository.batchSave(addItems, addItems.size());
            }
            // 更新 更新的property数据中更新的item数据
            if (CollectionUtils.isNotEmpty(updateItems)) {
                propertyItemRepository.batchUpdate(updateItems, updateItems.size());
            }
            property.setGmtModify(new Date());
            property.setMenderId(userId);
            propertyRepository.update(property);
        }

        // 4-对group的新增、更新、删除操作
        // 获取需要新增的group数据
        if (CollectionUtils.isNotEmpty(groups)) {
            List<PropertyGroup> addGroups =
                    groups.stream().filter(item -> null == item.getId()).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(addGroups)) {
                propertyGroupRepository.batchSave(addGroups, addGroups.size());
            }
            // 获取需要更新的group数据
            List<PropertyGroup> updateGroups =
                    groups.stream().filter(item -> null != item.getId()).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(updateGroups)) {
                updateGroups.stream().forEach(item -> {
                    item.setMenderId(userId);
                });
                propertyGroupRepository.batchUpdate(updateGroups, updateGroups.size());
            }
            List<Long> groupIds = updateGroups.stream().map(PropertyGroup::getId).collect(Collectors.toList());
            // 获取需要删除的数据
            List<PropertyGroup> deleteGroups =
                    oldGroups.stream().filter(item -> !groupIds.contains(item.getId())).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(deleteGroups)) {
                deleteGroups.stream().forEach(item -> {
                    item.setModuleId(userId);
                    item.setDeleted(true);
                });
                propertyGroupRepository.batchUpdate(deleteGroups, deleteGroups.size());
            }
        }
        return new GeneralResult<>(true, "保存成功");
    }

    /**
     * 数据校验
     *
     * @param
     * @throws Exception
     */
    private void validate(List<Property> properties, List<PropertyGroup> groups) throws Exception {
        if (ListTool.isEmpty(properties)) {
            throw new Exception("不可删除所有属性");
        }
        // 校验属性以及属性的item数据
        Map<String, String> codeMap = new HashMap<>();
        Map<String, String> nameMap = new HashMap<>();
        for (Property property : properties) {
            String code = property.getCode().toLowerCase();
            if (code == null || "".equals(code)) {
                throw new Exception("属性编码不能为空");
            }
            code = code.trim();
            Pattern compile = Pattern.compile(
                    "^(create|alter|select|update|union|and|or|delete|insert|trancate|char|substr|commit|ascii|declare|exec|count|show|into|drop|execute)$");
            if (compile.matcher(code).find()) {
                throw new Exception("属性编码不能是关键字：" + code);
            }
            if (!code.matches("^[a-zA-Z][0-9a-zA-Z_]{0,}$")) {
                throw new Exception("唯一标识必须以字母开头且由字母数字或者下划线组成");
            }
            if (codeMap.get(code) != null) {
                throw new Exception("属性编码重复：" + code);
            }
            codeMap.put(code, code);
            String name = property.getName();
            if (name == null || "".equals(name)) {
                throw new Exception("属性名称不能为空");
            }
            if (nameMap.get(name) != null) {
                throw new Exception("属性名称重复：" + name);
            }
            nameMap.put(name, name);
            List<PropertyItem> items = property.getItemList();
            if (CollectionUtils.isNotEmpty(items)) {
                if (property.getType().equals(PropertyConstant.PropertyType.MULTILINETABLE)
                        || property.getType().equals(PropertyConstant.PropertyType.SINGLELINETABLE)) {
                    Map<String, String> itemNameMap = new HashMap<>();
                    Map<String, String> itemCodeMap = new HashMap<>();
                    List<PropertyItem> tableItems = property.getItemList();
                    int i = 1;
                    for (PropertyItem tableItem : tableItems) {
                        String itemName = tableItem.getName();
                        if (itemName == null || "".equals(itemName)) {
                            throw new Exception("结构项名称不能为空！");
                        }
                        if (itemNameMap.get(itemName) != null) {
                            throw new Exception("结构项名称重复：" + itemName);
                        }
                        itemNameMap.put(itemName, itemName);
                        String itemCode = tableItem.getCode();
                        if (org.apache.commons.lang3.StringUtils.isEmpty(itemCode)) {
                            throw new Exception("结构项ID不能为空");
                        }
                        if (itemCodeMap.get(itemCode) != null) {
                            throw new Exception("结构项ID重复：" + itemCode);
                        }
                        itemCodeMap.put(itemCode, itemCode);
                        if (tableItem.getType().equals(PropertyConstant.PropertyType.SINGLESELECT)
                                || tableItem.getType().equals(PropertyConstant.PropertyType.MULTISELECT)
                                || tableItem.getType().equals(PropertyConstant.PropertyType.RADIO)
                                || tableItem.getType().equals(PropertyConstant.PropertyType.CHECKBOX)) {
                            validateItemValues(tableItem);
                        }
                    }
                } else {
                    Map<String, String> itemValueMap = new HashMap<>();
                    Map<String, String> itemNameMap = new HashMap<>();
                    for (PropertyItem item : items) {
                        // value校验
                        String itemValue = item.getValue();
                        if (StringUtils.isEmpty(itemValue)) {
                            throw new Exception("枚举值不能为空");
                        }
                        if (itemValueMap.get(itemValue) != null) {
                            throw new Exception("枚举值必须唯一");
                        }
                        itemValueMap.put(itemValue, itemValue);
                        // name校验
                        String itemName = item.getName();
                        if (StringUtils.isEmpty(itemName)) {
                            throw new Exception("枚举值描述不能为空");
                        }
                        if (itemNameMap.get(itemName) != null) {
                            throw new Exception("枚举值描述必须唯一");
                        }
                        itemNameMap.put(itemName, itemName);
                    }
                }
            }
        }
        // 校验属性分组数据
        Map<String, String> groupNameMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(groups)) {
            for (PropertyGroup group : groups) {
                String name = group.getName();
                if (name == null || "".equals(name)) {
                    throw new Exception("属性名称不能为空");
                }
                if (groupNameMap.get(name) != null) {
                    throw new Exception("属性名称重复：" + name);
                }
                groupNameMap.put(name, name);
            }
        }
    }

    private void validateItemValues(PropertyItem tableItem) throws Exception {
        List<PropertyItem> itemValues = tableItem.getItemList();
        Map<String, String> itemValueMap = new HashMap<>();
        Map<String, String> itemNameMap = new HashMap<>();
        for (PropertyItem item : itemValues) {
            String itemName = item.getName();
            if (org.apache.commons.lang3.StringUtils.isEmpty(itemName)) {
                throw new Exception("结构项【" + tableItem.getName() + "】的枚举值描述不能为空");
            }
            if (itemNameMap.get(itemName) != null) {
                throw new Exception("结构项【" + tableItem.getName() + "】的枚举值描述必须唯一");
            }
            itemNameMap.put(itemName, itemName);
            String itemValue = item.getValue();
            if (org.apache.commons.lang3.StringUtils.isEmpty(itemValue)) {
                throw new Exception("结构项【" + tableItem.getName() + "】的枚举值不能为空");
            }
            if (itemValueMap.get(itemValue) != null) {
                throw new Exception("结构项【" + tableItem.getName() + "】的枚举值必须唯一");
            }
            itemValueMap.put(itemValue, itemValue);
        }
    }

    private void copyProperties(Property property, PropertyPool pool)
            throws IllegalAccessException, InvocationTargetException {
        Long id = property.getId();
        BeanUtilsBean.getInstance().copyProperties(property, pool);
        property.setId(id);
    }

    public void getGroups(List<Long> groupIds, Long groupId, Map<Long, List<ModuleGroup>> moduleGroups) {
        List<ModuleGroup> groups = moduleGroups.get(groupId);
        if (CollectionUtils.isNotEmpty(groups)) {
            for (ModuleGroup moduleGroup : groups) {
                this.getGroups(groupIds, moduleGroup.getId(), moduleGroups);
            }
        }
        groupIds.add(groupId);
    }

}
