package com.bocloud.ams.service.module.impl;

import com.bocloud.ams.entity.module.RegularExpression;
import com.bocloud.ams.repository.module.RegularExpressionRepository;
import com.bocloud.ams.service.module.RegularExpressionService;
import com.bocloud.ams.service.utils.PropertyConstant;
import com.megatron.common.model.GeneralResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * CiRegularExpression Service实现层
 *
 * @author 胡立伟
 * @Version 1.0
 * @since 2017年9月15日
 */
@Slf4j
@Service("regularExpressionService")
public class RegularExpressionServiceImpl implements RegularExpressionService {

    @Autowired
    private RegularExpressionRepository regularExpressionRepository;

    @Override
    public GeneralResult<List<RegularExpression>> list(final String type, String positionKey) {
        try {
            List<RegularExpression> expressions = regularExpressionRepository.listAll();
            // 模型属性的类别为数据加密的单独处理
            if (type.equals(PropertyConstant.PropertyType.TEXTENCRYPT)) {
                expressions = expressions.stream().filter(item -> null != item.getCode())
                    .filter(item -> item.getCode().equals("SIMPLEPASSWORD") || item.getCode().equals("COMPLEXPASSWORD"))
                    .collect(Collectors.toList());
            }
            return new GeneralResult<>(true, expressions, "查询成功");
        } catch (Exception e) {
            log.error("Query cmdbRegularExpression list error:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

}
