package com.bocloud.ams.service.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.audit.OperationalAudit;
import com.bocloud.ams.entity.enums.OperationalAuditType;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.instance.InstanceRelationHistoryRecord;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.model.InstanceRelationHistoryAlterModel;
import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.repository.audit.OperationalAuditRepository;
import com.bocloud.ams.repository.instance.InstanceRelationHistoryRecordRepository;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.audit.OperationalAuditService;
import com.bocloud.ams.service.utils.AutoModuleInstanceRelationUtils;
import com.bocloud.ams.service.utils.ModuleConstant;
import com.bocloud.ams.service.utils.RelationHistoryConstant;
import com.bocloud.ams.service.utils.RepositoryHandler;

import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @Description: 模型实例自动关联任务
 * @Title: ModuleInstanceRelationTask
 * @Author: majiali
 * @CreateTime: 2022/6/9 15:09
 */
@Slf4j
public class ModuleInstanceRelationTask implements Runnable {

    private AutoRelationRule relationRule;
    private ModuleRelation moduleRelation;
    private InstanceRelationRepository instanceRelationRepository;
    private InstanceRepository instanceRepository;
    private InstanceValueRepository instanceValueRepository;
    private InstanceRelationHistoryRecordRepository instanceRelationHistoryRecordRepository;
    private RepositoryHandler handler;
    private LockFactory lockFactory;
    private OperationalAuditService operationalAuditService;
    private OperationalAuditRepository operationalAuditRepository;

    public ModuleInstanceRelationTask(AutoRelationRule relationRule, ModuleRelation moduleRelation,
                                      InstanceRelationRepository instanceRelationRepository, InstanceRepository instanceRepository,
                                      InstanceValueRepository instanceValueRepository,
                                      InstanceRelationHistoryRecordRepository instanceRelationHistoryRecordRepository, RepositoryHandler handler,
                                      LockFactory lockFactory, OperationalAuditService operationalAuditService,OperationalAuditRepository operationalAuditRepository) {
        this.relationRule = relationRule;
        this.moduleRelation = moduleRelation;
        this.instanceRelationRepository = instanceRelationRepository;
        this.instanceRepository = instanceRepository;
        this.instanceValueRepository = instanceValueRepository;
        this.instanceValueRepository = instanceValueRepository;
        this.instanceRelationHistoryRecordRepository = instanceRelationHistoryRecordRepository;
        this.handler = handler;
        this.lockFactory = lockFactory;
        this.operationalAuditService = operationalAuditService;
        this.operationalAuditRepository = operationalAuditRepository;

    }

    @SneakyThrows
    @Override
    public void run() {
        log.info("开始处理模型实例关系，模型关系id：{}", moduleRelation.getId());
        String path = "auto_relatiom_module_relation_id_" + moduleRelation.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            String sourceInstanceTable = CmdbUtils.CMDB_INSTANCE + moduleRelation.getSourceCode().toLowerCase();
            List<Instance> sourceInstances =
                    instanceRepository.queryByModuleId(sourceInstanceTable, moduleRelation.getSourceId());
            buildInstanceData(sourceInstances, moduleRelation.getSourceCode());
            // 根据目标模型查询模型实例数据
            String instanceTable = CmdbUtils.CMDB_INSTANCE + moduleRelation.getTargetCode().toLowerCase();
            List<Instance> targetInstances =
                    instanceRepository.queryByModuleId(instanceTable, moduleRelation.getTargetId());
            buildInstanceData(targetInstances, moduleRelation.getTargetCode());
            if (CollectionUtils.isNotEmpty(sourceInstances) && CollectionUtils.isNotEmpty(targetInstances)) {
                for (Instance instance : sourceInstances) {
                    // 处理当前实例，与目标实例的关系
                    dealPositiveInstanceRelation(instance, targetInstances, moduleRelation, relationRule, 1L);
            }
            }
        } catch (Exception e) {
            log.error("模型实例自动关联失败", e);
        }
        log.info("结束模型实例关系,模型关系id：{}", moduleRelation.getId());

    }

    /**
     * @param sourceInstance  当前实例信息（源）
     * @param targetInstances 目标实例
     * @param moduleRelation  模型关系
     * @param relationRule    关联规则
     * @Description:处理正向关系
     * @Author majiali
     * @CreateTime 2022/5/20 16:35
     */
    private void dealPositiveInstanceRelation(Instance sourceInstance, List<Instance> targetInstances,
                                              ModuleRelation moduleRelation, AutoRelationRule relationRule, Long userId) throws Exception {

        log.info("开始处理实例关系，当实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
        // 有关系目标实例id
        Map<Long, Instance> targetRelationMap = new HashMap<>();
        // 实例模型为源模型,则查询目标模型实例挂关系
        for (Instance target : targetInstances) {
            boolean flag = AutoModuleInstanceRelationUtils.isInstanceRelation(sourceInstance, target, relationRule);
            if (flag) {
                targetRelationMap.put(target.getId(), target);
            }
        }
        if (!targetRelationMap.isEmpty()) {
            String sourceType = moduleRelation.getSourceType();
            String targetType = moduleRelation.getTargetType();
            // 1-1 1-N N-1 N-N 关系校验
            if (ModuleConstant.RELATION_TYPE_1.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_1.equals(targetType)) {
                // 保存一条记录 判断源实例是否有关系记录，同时判断目标实例是否已经有其他实例关系
                deal11PositiveRelation(sourceInstance, targetRelationMap, moduleRelation, relationRule, userId);
            } else if (ModuleConstant.RELATION_TYPE_1.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_N.equals(targetType)) {
                // 保存全部记录
                deal1NPositiveRelation(sourceInstance, targetRelationMap, moduleRelation, relationRule, userId);
            } else if (ModuleConstant.RELATION_TYPE_N.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_1.equals(targetType)) {
                // 保存一条记录
                dealN1PositiveRelation(sourceInstance, targetRelationMap, moduleRelation, relationRule, userId);
            } else if (ModuleConstant.RELATION_TYPE_N.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_N.equals(targetType)) {
                // 保存全部记录
                dealNNPositiveRelation(sourceInstance, targetRelationMap, moduleRelation, relationRule, userId);
            }
        } else {
            log.info("没有满足规则的实例数据，删除表中已有实例关系");
            deletePositiveRelation(sourceInstance, moduleRelation, userId);

        }
        log.info("结束处理实例关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
    }

    /**
     * @param sourceInstance    源实例
     * @param targetRelationMap 满足规则的关系
     * @param moduleRelation    模型关系
     * @param userId
     * @Description: 正向1-1关系
     * @CreateTime 2022/5/26 16:28
     */
    private void deal11PositiveRelation(Instance sourceInstance, Map<Long, Instance> targetRelationMap,
                                        ModuleRelation moduleRelation, AutoRelationRule relationRule, Long userId) throws Exception {
        log.info("开始处理正向1-1关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
        List<Long> relationIds = new ArrayList<>(targetRelationMap.keySet());
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 查询该模型关系下的所有实例关系
        List<InstanceRelation> oldRelations = instanceRelationRepository.queryByCondition(instanceRelationTable,
                moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), null);
        // 库中已有的关系
        List<InstanceRelation> oldSourceRelations = oldRelations.stream()
                .filter(item -> item.getSourceInstanceId().equals(sourceInstance.getId())).collect(Collectors.toList());
        // 库中已有满足条件的关系
        List<InstanceRelation> oldKeepRelations = oldSourceRelations.stream()
                .filter(item -> relationIds.contains(item.getTargetInstanceId())).collect(Collectors.toList());
        // 库中已有不满足条件的关系
        List<InstanceRelation> oldNoKeepRelations = oldSourceRelations.stream()
                .filter(item -> !relationIds.contains(item.getTargetInstanceId())).collect(Collectors.toList());
        // 冲突数据
        List<InstanceRelation> oldConflictRelations =
                oldRelations.stream().filter(item -> relationIds.contains(item.getTargetInstanceId())
                        && !item.getSourceInstanceId().equals(sourceInstance.getId())).collect(Collectors.toList());
        Map<Long, List<InstanceRelation>> conflictRelationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(oldConflictRelations)) {
            conflictRelationMap =
                    oldConflictRelations.stream().collect(Collectors.groupingBy(InstanceRelation::getTargetInstanceId));
        }
        // 剩余满足条件的targetIds
        relationIds.removeAll(
                oldKeepRelations.stream().map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList()));
        List<InstanceRelationHistoryRecord> list = new ArrayList<>();
        if (CollectionUtils.isEmpty(oldKeepRelations)) {
            // 判断当前目标实例中是否有被占用的
            // 剩余满足条件且目标实例被使用的原实例关系
            if (relationIds.size() == 1 && CollectionUtils.isEmpty(oldConflictRelations)) {
                Instance targetInstance = targetRelationMap.get(relationIds.get(0));
                InstanceRelation relation = AutoModuleInstanceRelationUtils.getInstanceRelation(sourceInstance,
                        targetInstance, moduleRelation, RelationHistoryConstant.Way.AUTO.getValue(), userId);
                // 保存关系
                handler.saveInstanceRelation(relation, instanceRelationTable);
                InstanceRelationHistoryRecord record = AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true,
                        sourceInstance, targetInstance, moduleRelation, relationRule,
                        RelationHistoryConstant.Way.AUTO.getValue(), RelationHistoryConstant.Status.SUCCESS.getValue(),
                        RelationHistoryConstant.Type.RELATION.getValue(), userId, null);
                list.add(record);
            } else {
                for (Long id : relationIds) {
                    list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                            targetRelationMap.get(id), moduleRelation, relationRule,
                            RelationHistoryConstant.Way.AUTO.getValue(),
                            RelationHistoryConstant.Status.RECONCILED.getValue(),
                            RelationHistoryConstant.Type.RELATION.getValue(), userId, conflictRelationMap.get(id)));
                }
            }
        } else {
            if (relationIds.size() > 0 || oldKeepRelations.size() > 1) {
                // 不满足约束条件
                for (InstanceRelation relation : oldKeepRelations) {
                    List<InstanceRelation> conflictRelations = new ArrayList<>();
                    conflictRelations.add(relation);
                    List<InstanceRelation> relations = conflictRelationMap.get(relation.getTargetInstanceId());
                    if (CollectionUtils.isNotEmpty(relations)) {
                        conflictRelations.addAll(relations);
                    }
                    list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                            targetRelationMap.get(relation.getTargetInstanceId()), moduleRelation, relationRule,
                            RelationHistoryConstant.Way.AUTO.getValue(),
                            RelationHistoryConstant.Status.RECONCILED.getValue(),
                            RelationHistoryConstant.Type.RELATION.getValue(), userId, conflictRelations));
                }
                for (Long id : relationIds) {
                    list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                            targetRelationMap.get(id), moduleRelation, relationRule,
                            RelationHistoryConstant.Way.AUTO.getValue(),
                            RelationHistoryConstant.Status.RECONCILED.getValue(),
                            RelationHistoryConstant.Type.RELATION.getValue(), userId, conflictRelationMap.get(id)));
                }
            }

            if (relationIds.size() == 0 && oldKeepRelations.size() == 1) {
                InstanceRelation relation = oldKeepRelations.get(0);
                List<InstanceRelation> relations = conflictRelationMap.get(relation.getTargetInstanceId());
                //关联方不满足约束条件
                if (CollectionUtils.isNotEmpty(relations)) {
                    List<InstanceRelation> conflictRelations = new ArrayList<>();
                    conflictRelations.add(relation);
                    conflictRelations.addAll(relations);
                    list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                            targetRelationMap.get(relation.getTargetInstanceId()), moduleRelation, relationRule,
                            RelationHistoryConstant.Way.AUTO.getValue(),
                            RelationHistoryConstant.Status.RECONCILED.getValue(),
                            RelationHistoryConstant.Type.RELATION.getValue(), userId, conflictRelations));
                }

            }
        }
        // 已有不满足条件的关系ids
        List<Long> oldNoKeepRelarionIds = new ArrayList<>();
        for (InstanceRelation relation : oldNoKeepRelations) {
            list.add(0, AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance, null,
                    moduleRelation, relationRule, RelationHistoryConstant.Way.AUTO.getValue(),
                    RelationHistoryConstant.Status.SUCCESS.getValue(), RelationHistoryConstant.Type.CANCEL.getValue(),
                    userId, Arrays.asList(relation)));
            oldNoKeepRelarionIds.add(relation.getId());
        }
        // 删除已有不满足条件的
        if (CollectionUtils.isNotEmpty(oldNoKeepRelarionIds)) {
            instanceRelationRepository.removeByIds(oldNoKeepRelarionIds, instanceRelationTable, userId);
        }
        if (CollectionUtils.isNotEmpty(list)) {
            instanceRelationHistoryRecordRepository.batchSave(list, 100);
            this.addOperationalAudit(list,userId);
        }

        log.info("结束处理正向1-1关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
    }

    /**
     * @param sourceInstance    源实例
     * @param targetRelationMap 满足规则的关系
     * @param moduleRelation    模型关系
     * @param userId
     * @Description: 正向1-N关系
     * @CreateTime 2022/5/26 16:28
     */
    private void deal1NPositiveRelation(Instance sourceInstance, Map<Long, Instance> targetRelationMap,
                                        ModuleRelation moduleRelation, AutoRelationRule relationRule, Long userId) throws Exception {
        log.info("开始处理正向1-N关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
        List<Long> relationIds = new ArrayList<>(targetRelationMap.keySet());
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 查询该模型关系下的所有实例关系
        List<InstanceRelation> oldRelations = instanceRelationRepository.queryByCondition(instanceRelationTable,
                moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), null);
        // 库中已有的关系
        List<InstanceRelation> oldSourceRelations = oldRelations.stream()
                .filter(item -> item.getSourceInstanceId().equals(sourceInstance.getId())).collect(Collectors.toList());
        // 库中已有满足条件的关系
        List<InstanceRelation> oldKeepRelations = oldSourceRelations.stream()
                .filter(item -> relationIds.contains(item.getTargetInstanceId())).collect(Collectors.toList());
        // 库中已有不满足条件的关系
        List<InstanceRelation> oldNoKeepRelations = oldSourceRelations.stream()
                .filter(item -> !relationIds.contains(item.getTargetInstanceId())).collect(Collectors.toList());
        // 剩余满足条件的targetIds
        relationIds.removeAll(
                oldKeepRelations.stream().map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList()));
        // 执行历史
        List<InstanceRelationHistoryRecord> list = new ArrayList<>();
        // 冲突数据
        List<InstanceRelation> oldConflictRelations =
                oldRelations.stream().filter(item -> relationIds.contains(item.getTargetInstanceId())
                        && !item.getSourceInstanceId().equals(sourceInstance.getId())).collect(Collectors.toList());
        Map<Long, List<InstanceRelation>> conflictRelationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(oldConflictRelations)) {
            conflictRelationMap =
                    oldConflictRelations.stream().collect(Collectors.groupingBy(InstanceRelation::getTargetInstanceId));
        }
        // 实例关系
        List<InstanceRelation> relationList = new ArrayList<>();
        for (Long id : relationIds) {
            List<InstanceRelation> conflicts = conflictRelationMap.get(id);
            if (CollectionUtils.isEmpty(conflicts)) {
                relationList.add(AutoModuleInstanceRelationUtils.getInstanceRelation(sourceInstance,
                        targetRelationMap.get(id), moduleRelation, RelationHistoryConstant.Way.AUTO.getValue(), userId));
                list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                        targetRelationMap.get(id), moduleRelation, relationRule,
                        RelationHistoryConstant.Way.AUTO.getValue(), RelationHistoryConstant.Status.SUCCESS.getValue(),
                        RelationHistoryConstant.Type.RELATION.getValue(), userId, null));
            } else {
                list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                        targetRelationMap.get(id), moduleRelation, relationRule,
                        RelationHistoryConstant.Way.AUTO.getValue(), RelationHistoryConstant.Status.RECONCILED.getValue(),
                        RelationHistoryConstant.Type.RELATION.getValue(), userId, conflicts));
            }
        }

        //数据库中已有满足关联规则的关系,校验被关联方是否满足约束条件
        for (InstanceRelation relation : oldKeepRelations) {
            List<InstanceRelation> relations = conflictRelationMap.get(relation.getTargetInstanceId());
            //关联方不满足约束条件
            if (CollectionUtils.isNotEmpty(relations)) {
                List<InstanceRelation> conflictRelations = new ArrayList<>();
                conflictRelations.add(relation);
                conflictRelations.addAll(relations);
                list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                        targetRelationMap.get(relation.getTargetInstanceId()), moduleRelation, relationRule,
                        RelationHistoryConstant.Way.AUTO.getValue(),
                        RelationHistoryConstant.Status.RECONCILED.getValue(),
                        RelationHistoryConstant.Type.RELATION.getValue(), userId, conflictRelations));
            }
        }
        // 已有不满足条件的关系ids
        List<Long> oldNoKeepRelarionIds = new ArrayList<>();
        for (InstanceRelation relation : oldNoKeepRelations) {
            list.add(0, AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance, null,
                    moduleRelation, relationRule, RelationHistoryConstant.Way.AUTO.getValue(),
                    RelationHistoryConstant.Status.SUCCESS.getValue(), RelationHistoryConstant.Type.CANCEL.getValue(),
                    userId, Arrays.asList(relation)));
            oldNoKeepRelarionIds.add(relation.getId());
        }
        // 删除已有不满足条件的
        if (CollectionUtils.isNotEmpty(oldNoKeepRelarionIds)) {
            instanceRelationRepository.removeByIds(oldNoKeepRelarionIds, instanceRelationTable, userId);
        }
        if (CollectionUtils.isNotEmpty(relationList)) {
            handler.batchSaveInstanceRelation(relationList, instanceRelationTable);
        }
        if (CollectionUtils.isNotEmpty(list)) {
            instanceRelationHistoryRecordRepository.batchSave(list, 100);
            this.addOperationalAudit(list,userId);
        }
        log.info("结束处理正向1-N关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
    }

    /**
     * @param sourceInstance    源实例
     * @param targetRelationMap 满足规则的关系
     * @param moduleRelation    模型关系
     * @param userId
     * @Description: 正向N-1关系
     * @CreateTime 2022/5/26 16:28
     */
    private void dealN1PositiveRelation(Instance sourceInstance, Map<Long, Instance> targetRelationMap,
                                        ModuleRelation moduleRelation, AutoRelationRule relationRule, Long userId) throws Exception {
        log.info("开始处理正向N-1关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
        List<Long> relationIds = new ArrayList<>(targetRelationMap.keySet());
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 查询该模型关系下的所有实例关系
        List<InstanceRelation> oldRelations = instanceRelationRepository.queryByCondition(instanceRelationTable,
                moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), null);
        // 库中已有的关系
        List<InstanceRelation> oldSourceRelations = oldRelations.stream()
                .filter(item -> item.getSourceInstanceId().equals(sourceInstance.getId())).collect(Collectors.toList());
        // 库中已有满足条件的关系
        List<InstanceRelation> oldKeepRelations = oldSourceRelations.stream()
                .filter(item -> relationIds.contains(item.getTargetInstanceId())).collect(Collectors.toList());
        // 库中已有不满足条件的关系
        List<InstanceRelation> oldNoKeepRelations = oldSourceRelations.stream()
                .filter(item -> !relationIds.contains(item.getTargetInstanceId())).collect(Collectors.toList());
        // 剩余满足条件的targetIds
        relationIds.removeAll(
                oldKeepRelations.stream().map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList()));
        // 执行历史
        List<InstanceRelationHistoryRecord> list = new ArrayList<>();
        if (CollectionUtils.isEmpty(oldKeepRelations)) {
            if (relationIds.size() == 1) {
                Instance targetInstance = targetRelationMap.get(relationIds.get(0));
                InstanceRelation relation = AutoModuleInstanceRelationUtils.getInstanceRelation(sourceInstance,
                        targetInstance, moduleRelation, RelationHistoryConstant.Way.AUTO.getValue(), userId);
                // 保存关系
                handler.saveInstanceRelation(relation, instanceRelationTable);
                InstanceRelationHistoryRecord record = AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true,
                        sourceInstance, targetInstance, moduleRelation, relationRule,
                        RelationHistoryConstant.Way.AUTO.getValue(), RelationHistoryConstant.Status.SUCCESS.getValue(),
                        RelationHistoryConstant.Type.RELATION.getValue(), userId, null);
                list.add(record);
            } else {
                for (Long id : relationIds) {
                    list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                            targetRelationMap.get(id), moduleRelation, relationRule,
                            RelationHistoryConstant.Way.AUTO.getValue(),
                            RelationHistoryConstant.Status.RECONCILED.getValue(),
                            RelationHistoryConstant.Type.RELATION.getValue(), userId, null));
                }
            }

        } else {
            if (relationIds.size() > 0 || oldKeepRelations.size() > 1) {
                // 不满足约束条件
                for (InstanceRelation relation : oldKeepRelations) {
                    list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                            targetRelationMap.get(relation.getTargetInstanceId()), moduleRelation, relationRule,
                            RelationHistoryConstant.Way.AUTO.getValue(),
                            RelationHistoryConstant.Status.RECONCILED.getValue(),
                            RelationHistoryConstant.Type.RELATION.getValue(), userId, Arrays.asList(relation)));
                }
                for (Long id : relationIds) {
                    list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                            targetRelationMap.get(id), moduleRelation, relationRule,
                            RelationHistoryConstant.Way.AUTO.getValue(),
                            RelationHistoryConstant.Status.RECONCILED.getValue(),
                            RelationHistoryConstant.Type.RELATION.getValue(), userId, null));
                }
            }
        }
        // 已有不满足条件的关系ids
        List<Long> oldNoKeepRelarionIds = new ArrayList<>();
        for (InstanceRelation relation : oldNoKeepRelations) {
            list.add(0, AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance, null,
                    moduleRelation, relationRule, RelationHistoryConstant.Way.AUTO.getValue(),
                    RelationHistoryConstant.Status.SUCCESS.getValue(), RelationHistoryConstant.Type.CANCEL.getValue(),
                    userId, Arrays.asList(relation)));
            oldNoKeepRelarionIds.add(relation.getId());
        }
        // 删除已有不满足条件的
        if (CollectionUtils.isNotEmpty(oldNoKeepRelarionIds)) {
            instanceRelationRepository.removeByIds(oldNoKeepRelarionIds, instanceRelationTable, userId);
        }
        if (CollectionUtils.isNotEmpty(list)) {
            instanceRelationHistoryRecordRepository.batchSave(list, 100);
            this.addOperationalAudit(list,userId);

        }
        log.info("结束处理正向N-1关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
    }

    /**
     * @param sourceInstance    源实例
     * @param targetRelationMap 满足规则的关系
     * @param moduleRelation    模型关系
     * @param userId
     * @Description: 正向N-N关系
     * @CreateTime 2022/5/26 16:28
     */
    private void dealNNPositiveRelation(Instance sourceInstance, Map<Long, Instance> targetRelationMap,
                                        ModuleRelation moduleRelation, AutoRelationRule relationRule, Long userId) throws Exception {
        log.info("开始处理正向N-N关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
        List<Long> relationIds = new ArrayList<>(targetRelationMap.keySet());
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 查询该模型关系下的所有实例关系
        List<InstanceRelation> oldRelations = instanceRelationRepository.queryByCondition(instanceRelationTable,
                moduleRelation.getSourceId(), null, moduleRelation.getTargetId(), null);
        // 库中已有的关系
        List<InstanceRelation> oldSourceRelations = oldRelations.stream()
                .filter(item -> item.getSourceInstanceId().equals(sourceInstance.getId())).collect(Collectors.toList());
        // 库中已有满足条件的关系
        List<InstanceRelation> oldKeepRelations = oldSourceRelations.stream()
                .filter(item -> relationIds.contains(item.getTargetInstanceId())).collect(Collectors.toList());
        // 库中已有不满足条件的关系
        List<InstanceRelation> oldNoKeepRelations = oldSourceRelations.stream()
                .filter(item -> !relationIds.contains(item.getTargetInstanceId())).collect(Collectors.toList());
        // 剩余满足条件的targetIds
        relationIds.removeAll(
                oldKeepRelations.stream().map(InstanceRelation::getTargetInstanceId).collect(Collectors.toList()));
        // 执行历史
        List<InstanceRelationHistoryRecord> list = new ArrayList<>();
        // 实例关系
        List<InstanceRelation> relationList = new ArrayList<>();
        // 已有不满足条件的关系ids
        List<Long> oldNoKeepRelarionIds = new ArrayList<>();
        for (InstanceRelation relation : oldNoKeepRelations) {
            list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance, null,
                    moduleRelation, relationRule, RelationHistoryConstant.Way.AUTO.getValue(),
                    RelationHistoryConstant.Status.SUCCESS.getValue(), RelationHistoryConstant.Type.CANCEL.getValue(),
                    userId, Arrays.asList(relation)));
            oldNoKeepRelarionIds.add(relation.getId());
        }
        for (Long id : relationIds) {
            relationList.add(AutoModuleInstanceRelationUtils.getInstanceRelation(sourceInstance,
                    targetRelationMap.get(id), moduleRelation, RelationHistoryConstant.Way.AUTO.getValue(), userId));
            list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance,
                    targetRelationMap.get(id), moduleRelation, relationRule, RelationHistoryConstant.Way.AUTO.getValue(),
                    RelationHistoryConstant.Status.SUCCESS.getValue(), RelationHistoryConstant.Type.RELATION.getValue(),
                    userId, null));
        }
        if (CollectionUtils.isNotEmpty(relationList)) {
            handler.batchSaveInstanceRelation(relationList, instanceRelationTable);
        }
        if (CollectionUtils.isNotEmpty(oldNoKeepRelarionIds)) {
            instanceRelationRepository.removeByIds(oldNoKeepRelarionIds, instanceRelationTable, userId);
        }
        if (CollectionUtils.isNotEmpty(list)) {
            instanceRelationHistoryRecordRepository.batchSave(list, 100);
            this.addOperationalAudit(list,userId);
        }
        log.info("结束处理正向N-N关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
    }

    /**
     * 构建实例数据
     *
     * @param targetInstance
     * @param moduleCode
     * @throws Exception
     */
    private void buildInstanceData(List<Instance> targetInstance, String moduleCode) throws Exception {
        for (Instance instance : targetInstance) {
            // 查询实例数据
            List<InstanceValue> instanceValues = instanceValueRepository.listByInstanceId(instance.getId(),
                    CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase());
            instance.setInstanceValues(instanceValues);
        }
    }

    private void deletePositiveRelation(Instance sourceInstance,
                                        ModuleRelation moduleRelation, Long userId) throws Exception {
        log.info("开始处理正向没有满足条件的实例逻辑，删除库中已有的实例关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
        // 关系表
        String instanceRelationTable =
                CmdbUtils.CMDB_INSTANCE_RELATION + moduleRelation.getRelationCategoryCode().toLowerCase();
        // 查询该模型关系下的所有实例关系
        List<InstanceRelation> oldRelations = instanceRelationRepository.queryByCondition(instanceRelationTable,
                moduleRelation.getSourceId(), sourceInstance.getId(), moduleRelation.getTargetId(), null);

        if (CollectionUtils.isNotEmpty(oldRelations)) {
            log.info("删除库中已有的实例关系,数量：{}", oldRelations.size());
            // 执行历史
            List<InstanceRelationHistoryRecord> list = new ArrayList<>();
            // 已有不满足条件的关系ids
            List<Long> oldNoKeepRelarionIds = new ArrayList<>();
            for (InstanceRelation relation : oldRelations) {
                list.add(AutoModuleInstanceRelationUtils.getInstanceRelationHistory(true, sourceInstance, null,
                        moduleRelation, relationRule, RelationHistoryConstant.Way.AUTO.getValue(),
                        RelationHistoryConstant.Status.SUCCESS.getValue(), RelationHistoryConstant.Type.CANCEL.getValue(),
                        userId, Arrays.asList(relation)));
                oldNoKeepRelarionIds.add(relation.getId());
            }
            // 删除已有不满足条件的
            instanceRelationRepository.removeByIds(oldNoKeepRelarionIds, instanceRelationTable, userId);
            instanceRelationHistoryRecordRepository.batchSave(list, 100);
            this.addOperationalAudit(list,userId);
        }
        log.info("结束处理正向没有满足条件的实例逻辑，删除库中已有的实例关系，实例id：{},模型code：{},模型关系id：{}", sourceInstance.getId(), sourceInstance.getModuleCode(),
                moduleRelation.getId());
    }

    //操作审计
    private void addOperationalAudit(List<InstanceRelationHistoryRecord> list,Long userId) {
        try {
            List<OperationalAudit> auditList = new ArrayList<>();
            for (InstanceRelationHistoryRecord historyRecord : list) {
                if(!RelationHistoryConstant.Status.SUCCESS.getValue().equals(historyRecord.getStatus())){
                    continue;
                }
                String props = historyRecord.getProps();
                //关联
                InstanceRelationHistoryAlterModel alter = null;
                if (StringUtils.isNotEmpty(props)) {
                    alter = JSONObject.parseObject(historyRecord.getProps(), InstanceRelationHistoryAlterModel.class);
                }
                //关联
                Map<String, Object> afterMap = null;
                Map<String, Object> beforeMap = null;
                OperationalAudit audit = null;
                if (RelationHistoryConstant.Type.CANCEL.getValue().equals(historyRecord.getType())) {

                    if (null != alter && CollectionUtils.isNotEmpty(alter.getBefore())) {
                        beforeMap = JSON.parseObject(JSON.toJSONString(alter.getBefore().get(0)), new TypeReference<Map<String, Object>>() {
                        });
                    }
                    audit = operationalAuditService.buildLog(OperationalAuditType.ASSOCIATION, historyRecord.getModuleName(),
                            historyRecord.getName(), historyRecord.getModuleCode(), historyRecord.getId(), userId, beforeMap, afterMap);

                } else {
                    if (null != alter && null != alter.getAfter()) {
                        afterMap =
                                JSON.parseObject(JSON.toJSONString(alter.getAfter()), new TypeReference<Map<String, Object>>() {
                                });
                    }

                    audit = operationalAuditService.buildLog(OperationalAuditType.RELATION, historyRecord.getModuleName(),
                            historyRecord.getName(), historyRecord.getModuleCode(), historyRecord.getId(), userId, beforeMap, afterMap);
                }

                auditList.add(audit);
            }

            if(!auditList.isEmpty()){
                operationalAuditRepository.batchSave(auditList, 100);
            }

        }catch (Exception e){
            log.error("记录操作审计信息异常",e);
        }

    }
}
