package com.bocloud.ams.service.utils;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;

import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.model.InstanceModel;
import com.bocloud.ams.entity.module.ModuleRelation;

/**
 * @author 胡立伟
 * @program: bocloud.ams
 * @description:
 * @date 2022/5/11 13:47
 */
public class GraphAlgorithmUtils {

    /**
     * 深度优先遍历（递归方式）--- 获取出度的路径
     */
    public static List<List<Long>> outDegreeGraphAlgorithm(ModuleRelation relation) {
        List<List<Long>> result = new ArrayList<>();
        List<Long> path = new ArrayList<>();
        path.add(relation.getSourceId());
        findOutDegreePath(result, relation, path);
        return result;
    }

    private static void findOutDegreePath(List<List<Long>> result, ModuleRelation relation, List<Long> path) {
        if (CollectionUtils.isEmpty(relation.getCmdbModuleRelationList())) {
            result.add(path);
            return;
        }
        for (ModuleRelation moduleRelation : relation.getCmdbModuleRelationList()) {
            List<Long> cPath = new ArrayList<>();
            cPath.addAll(path);
            cPath.add(moduleRelation.getTargetId());
            findOutDegreePath(result, moduleRelation, cPath);
        }
    }

    /**
     * 深度优先遍历（递归方式）---- 获取入度的路径
     */
    public static List<List<Long>> penetrationGraphAlgorithm(ModuleRelation relation) {
        List<List<Long>> result = new ArrayList<>();
        List<Long> path = new ArrayList<>();
        path.add(relation.getTargetId());
        findPenetrationPath(result, relation, path);
        return result;
    }

    private static void findPenetrationPath(List<List<Long>> result, ModuleRelation relation, List<Long> path) {
        if (CollectionUtils.isEmpty(relation.getCmdbModuleRelationList())) {
            result.add(path);
            return;
        }
        for (ModuleRelation moduleRelation : relation.getCmdbModuleRelationList()) {
            List<Long> cPath = new ArrayList<>();
            cPath.addAll(path);
            cPath.add(moduleRelation.getSourceId());
            findPenetrationPath(result, moduleRelation, cPath);
        }
    }

    /**
     * 深度优先遍历（递归方式）--- 获取路径
     */
    public static List<List<InstanceModel>> resourceInstanceAlgorithm(InstanceRelation relation,
        Boolean pathDirection) {
        List<List<InstanceModel>> result = new ArrayList<>();
        List<InstanceModel> path = new ArrayList<>();
        InstanceModel model = new InstanceModel();
        model.setModuleId(relation.getSourceModuleId());
        model.setModuleCode(relation.getSourceModuleCode());
        model.setInstanceId(relation.getSourceInstanceId());
        model.setInstanceName(relation.getSourceInstanceName());
        path.add(model);
        resourceInstancePath(result, relation, path, pathDirection);
        return result;
    }

    private static void resourceInstancePath(List<List<InstanceModel>> result, InstanceRelation relation,
        List<InstanceModel> path, Boolean pathDirection) {
        if (CollectionUtils.isEmpty(relation.getCmdbInstanceRelations())) {
            result.add(path);
            return;
        }
        for (InstanceRelation instanceRelation : relation.getCmdbInstanceRelations()) {
            List<InstanceModel> cPath = new ArrayList<>();
            cPath.addAll(path);
            InstanceModel model = new InstanceModel();
            if (pathDirection) {
                model.setModuleId(instanceRelation.getSourceModuleId());
                model.setModuleCode(instanceRelation.getSourceModuleCode());
                model.setInstanceId(instanceRelation.getSourceInstanceId());
                model.setInstanceName(instanceRelation.getSourceInstanceName());
            } else {
                model.setModuleId(instanceRelation.getTargetModuleId());
                model.setModuleCode(instanceRelation.getTargetModuleCode());
                model.setInstanceId(instanceRelation.getTargetInstanceId());
                model.setInstanceName(instanceRelation.getTargetInstanceName());
            }
            cPath.add(model);
            resourceInstancePath(result, instanceRelation, cPath, pathDirection);
        }
    }

}
