package com.bocloud.ams.task.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.task.model.Job;
import com.bocloud.ams.task.model.JobResult;
import com.bocloud.ams.task.model.enums.TaskParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;

/**
 * 抽象任务监听器
 */
public abstract class AbstractTaskListener {
    static final String QUEUE = TaskParam.QUEUE.name();
    static final String DATA = TaskParam.TASKDATA.name();
    static final String PJOB = TaskParam.TASKPJOB.name();
    static final String CODE = TaskParam.TASKCODE.name();
    static final String TIME = TaskParam.TIMEOUT.name();
    static final String TYPE = TaskParam.TASKTYPE.name();
    static final String NAME = Common.NAME;

    /**
     * 处理接收到任务时的逻辑
     *
     * @param job
     *            接受到的任务参数对象
     * @return 任务执行结果
     */
    public abstract JobResult onTask(Job job);

    Integer buildTimeout(String time) {
        Integer timeout;
        try {
            timeout = Integer.valueOf(time);
            if (timeout <= 0) {
                timeout = 600;
            }
        } catch (Exception e) {
            timeout = 600;
        }
        return timeout;
    }

    public JobResult buildResult(GeneralResult<?> result, Job job) {
        return new JobResult(result.isSuccess(), JSONObject.toJSONString(result), job);
    }
}