package com.bocloud.ams.task.worker;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.bocloud.ams.task.annotation.Worker;
import com.megatron.common.locator.ClassLocator;

import lombok.extern.slf4j.Slf4j;

/**
 * Worker类定位器
 */
@Slf4j
public class WorkerClassLocator {
    private static Map<String, Class<?>> workerClassMap = new ConcurrentHashMap<>();

    public List<Class<?>> getWorkerClasses(String code) {
        @SuppressWarnings("rawtypes")
        List<Class<?>> allClass = ClassLocator.getAllClass(AbstractWorker.class.getPackage().getName(), "worker");
        List<Class<?>> fixClass = new ArrayList<>();
        for (Class<?> clazz : allClass) {
            if (clazz.isAnnotationPresent(Worker.class)) {
                Worker worker = (Worker)clazz.getAnnotation(Worker.class);
                if (worker.name().equalsIgnoreCase(code)) {
                    fixClass.add(clazz);
                }
            }
        }
        return fixClass;
    }

    public Class<?> getWorkerClass(String code) {
        if (workerClassMap.containsKey(code)) {
            return workerClassMap.get(code);
        }
        List<Class<?>> inputClassList = getWorkerClasses(code);
        if (inputClassList.isEmpty()) {
            return null;
        }
        if (inputClassList.size() > 1) {
            log.error("more than one class found!");
            return null;
        }
        workerClassMap.put(code, inputClassList.get(0));
        return inputClassList.get(0);
    }

}